@@ -2,6 +2,7 @@
 napari command line viewer.
 """
 import argparse
+import contextlib
 import logging
 import os
 import runpy
@@ -13,7 +14,7 @@ from pathlib import Path
 from textwrap import wrap
 from typing import Any, Dict, List
 
-from .utils.translations import trans
+from napari.utils.translations import trans
 
 
 class InfoAction(argparse.Action):
@@ -76,30 +77,24 @@ def validate_unknown_args(unknown: List[str]) -> Dict[str, Any]:
 
     out: Dict[str, Any] = {}
     valid = set.union(*valid_add_kwargs().values())
-    for i, arg in enumerate(unknown):
-        if not arg.startswith("--"):
+    for i, raw_arg in enumerate(unknown):
+        if not raw_arg.startswith("--"):
             continue
+        arg = raw_arg.lstrip('-')
 
-        if "=" in arg:
-            key, value = arg.split("=", maxsplit=1)
-        else:
-            key = arg
-        key = key.lstrip('-').replace("-", "_")
-
+        key, *value = arg.split("=", maxsplit=1)
+        key = key.replace('-', '_')
         if key not in valid:
-            sys.exit(f"error: unrecognized arguments: {arg}")
+            sys.exit(f"error: unrecognized argument: {raw_arg}")
 
-        if "=" not in arg:
-            try:
-                value = unknown[i + 1]
-                if value.startswith("--"):
-                    raise IndexError()
-            except IndexError:
-                sys.exit(f"error: argument {arg} expected one argument")
-        try:
+        if value:
+            value = value[0]
+        else:
+            if len(unknown) <= i + 1 or unknown[i + 1].startswith("--"):
+                sys.exit(f"error: argument {raw_arg} expected one argument")
+            value = unknown[i + 1]
+        with contextlib.suppress(Exception):
             value = literal_eval(value)
-        except Exception:
-            value = value
 
         out[key] = value
     return out
@@ -270,7 +265,11 @@ def _run():
 
     else:
         if args.with_:
-            from .plugins import _initialize_plugins, _npe2, plugin_manager
+            from napari.plugins import (
+                _initialize_plugins,
+                _npe2,
+                plugin_manager,
+            )
 
             # if a plugin widget has been requested, this will fail immediately
             # if the requested plugin/widget is not available.
@@ -382,10 +381,10 @@ def _run():
         # only necessary in bundled app, but see #3596
         from napari.utils.misc import (
             install_certifi_opener,
-            running_as_bundled_app,
+            running_as_constructor_app,
         )
 
-        if running_as_bundled_app():
+        if running_as_constructor_app():
             install_certifi_opener()
         run(gui_exceptions=True)
 
@@ -498,7 +497,7 @@ def _maybe_rerun_with_macos_fixes():
                 'please install python.app in conda using:\n'
                 'conda install -c conda-forge python.app'
             )
-            warnings.warn(msg)
+            warnings.warn(msg, stacklevel=2)
 
     # 3) Make sure the app name in the menu bar is 'napari', not 'python'
     tempdir = None
@@ -506,7 +505,7 @@ def _maybe_rerun_with_macos_fixes():
         # When napari is launched from the conda bundle shortcut
         # it already has the right 'napari' name in the app title
         # and __CFBundleIdentifier is set to 'com.napari._(<version>)'
-        "napari" not in os.environ.get("__CFBundleIdentifier", "")
+        "napari" not in os.environ.get("__CFBUNDLEIDENTIFIER", "")
         # with a sys.executable named napari,
         # macOS should have picked the right name already
         or os.path.basename(executable) != "napari"
@@ -532,6 +531,10 @@ def _maybe_rerun_with_macos_fixes():
         else:  # we assume it must have been launched via '-m' syntax
             cmd = [executable, "-m", "napari"]
 
+        # this fixes issues running from a venv/virtualenv based virtual
+        # environment with certain python distributions (e.g. pyenv, asdf)
+        env["PYTHONEXECUTABLE"] = sys.executable
+
         # Append original command line arguments.
         if len(sys.argv) > 1:
             cmd.extend(sys.argv[1:])
