@@ -1,17 +1,16 @@
 import os
-import sys
 from pathlib import Path
 from warnings import warn
 
-from ..utils.translations import trans
+from napari.utils.translations import trans
 
 try:
     from qtpy import API_NAME, QtCore
 except Exception as e:
     if 'No Qt bindings could be found' in str(e):
-        raise type(e)(
+        raise ImportError(
             trans._(
-                "No Qt bindings could be found.\n\nnapari requires either PyQt5 or PySide2 to be installed in the environment.\nTo install the default backend (currently PyQt5), run \"pip install napari[all]\" \nYou may also use \"pip install napari[pyside2]\"for Pyside2, or \"pip install napari[pyqt5]\" for PyQt5",
+                'No Qt bindings could be found.\n\nnapari requires either PyQt5 or PySide2 to be installed in the environment.\nTo install the default backend (currently PyQt5), run "pip install napari[all]" \nYou may also use "pip install napari[pyside2]"for Pyside2, or "pip install napari[pyqt5]" for PyQt5',
                 deferred=True,
             )
         ) from e
@@ -47,8 +46,10 @@ if tuple(int(x) for x in QtCore.__version__.split('.')[:3]) < (5, 12, 3):
             deferred=True,
             version=QtCore.__version__,
         )
-    warn(message=warn_message)
+    warn(message=warn_message, stacklevel=1)
 
 
-from .qt_event_loop import get_app, gui_qt, quit_app, run
-from .qt_main_window import Window
+from napari._qt.qt_event_loop import get_app, gui_qt, quit_app, run
+from napari._qt.qt_main_window import Window
+
+__all__ = ["get_app", "gui_qt", "quit_app", "run", "Window"]
