@@ -19,6 +19,7 @@ from napari._tests.utils import (
     skip_local_popups,
     skip_on_win_ci,
 )
+from napari._vispy._tests.utils import vispy_image_scene_size
 from napari._vispy.utils.gl import fix_data_dtype
 from napari.components.viewer_model import ViewerModel
 from napari.layers import Points
@@ -65,9 +66,26 @@ def test_qt_viewer_toggle_console(make_napari_viewer):
     assert view.dockConsole.widget() is view.console
 
 
+@skip_local_popups
+@pytest.mark.skipif(os.environ.get("MIN_REQ", "0") == "1", reason="min req")
+def test_qt_viewer_console_focus(qtbot, make_napari_viewer):
+    """Test console has focus when instantiating from viewer."""
+    viewer = make_napari_viewer(show=True)
+    view = viewer.window._qt_viewer
+    assert not view.console.hasFocus(), "console has focus before being shown"
+
+    view.toggle_console_visibility(None)
+
+    def console_has_focus():
+        assert (
+            view.console.hasFocus()
+        ), "console does not have focus when shown"
+
+    qtbot.waitUntil(console_has_focus)
+
+
 @pytest.mark.parametrize('layer_class, data, ndim', layer_test_data)
 def test_add_layer(make_napari_viewer, layer_class, data, ndim):
-
     viewer = make_napari_viewer(ndisplay=int(np.clip(ndim, 2, 3)))
     view = viewer.window._qt_viewer
 
@@ -331,6 +349,7 @@ def test_points_layer_display_correct_slice_on_scale(make_napari_viewer):
     np.testing.assert_equal(indices, [0])
 
 
+@skip_on_win_ci
 def test_qt_viewer_clipboard_with_flash(make_napari_viewer, qtbot):
     viewer = make_napari_viewer()
     # make sure clipboard is empty
@@ -349,15 +368,15 @@ def test_qt_viewer_clipboard_with_flash(make_napari_viewer, qtbot):
 
     # ensure the flash effect is applied
     assert (
-        viewer.window._qt_viewer._canvas_overlay.graphicsEffect() is not None
+        viewer.window._qt_viewer._welcome_widget.graphicsEffect() is not None
     )
     assert hasattr(
-        viewer.window._qt_viewer._canvas_overlay, "_flash_animation"
+        viewer.window._qt_viewer._welcome_widget, "_flash_animation"
     )
     qtbot.wait(500)  # wait for the animation to finish
-    assert viewer.window._qt_viewer._canvas_overlay.graphicsEffect() is None
+    assert viewer.window._qt_viewer._welcome_widget.graphicsEffect() is None
     assert not hasattr(
-        viewer.window._qt_viewer._canvas_overlay, "_flash_animation"
+        viewer.window._qt_viewer._welcome_widget, "_flash_animation"
     )
 
     # clear clipboard and grab image from application view
@@ -378,6 +397,7 @@ def test_qt_viewer_clipboard_with_flash(make_napari_viewer, qtbot):
     assert not hasattr(viewer.window._qt_window, "_flash_animation")
 
 
+@skip_on_win_ci
 def test_qt_viewer_clipboard_without_flash(make_napari_viewer):
     viewer = make_napari_viewer()
     # make sure clipboard is empty
@@ -395,9 +415,9 @@ def test_qt_viewer_clipboard_without_flash(make_napari_viewer):
     assert not clipboard_image.isNull()
 
     # ensure the flash effect is not applied
-    assert viewer.window._qt_viewer._canvas_overlay.graphicsEffect() is None
+    assert viewer.window._qt_viewer._welcome_widget.graphicsEffect() is None
     assert not hasattr(
-        viewer.window._qt_viewer._canvas_overlay, "_flash_animation"
+        viewer.window._qt_viewer._welcome_widget, "_flash_animation"
     )
 
     # clear clipboard and grab image from application view
@@ -495,9 +515,9 @@ def test_memory_leaking(qtbot, make_napari_viewer):
     assert labels() is None
 
 
+@skip_on_win_ci
 @skip_local_popups
 def test_leaks_image(qtbot, make_napari_viewer):
-
     viewer = make_napari_viewer(show=True)
     lr = weakref.ref(viewer.add_image(np.random.rand(10, 10)))
     dr = weakref.ref(lr().data)
@@ -510,6 +530,7 @@ def test_leaks_image(qtbot, make_napari_viewer):
     assert not dr()
 
 
+@skip_on_win_ci
 @skip_local_popups
 def test_leaks_labels(qtbot, make_napari_viewer):
     viewer = make_napari_viewer(show=True)
@@ -674,3 +695,17 @@ def test_create_non_empty_viewer_model(qtbot):
     del viewer
     qtbot.wait(50)
     gc.collect()
+
+
+def test_axes_labels(make_napari_viewer):
+    viewer = make_napari_viewer(ndisplay=3)
+    layer = viewer.add_image(np.zeros((2, 2, 2)), scale=(1, 2, 4))
+
+    layer_visual = viewer._window._qt_viewer.layer_to_visual[layer]
+    axes_visual = viewer._window._qt_viewer.overlay_to_visual[
+        viewer._overlays['axes']
+    ]
+
+    layer_visual_size = vispy_image_scene_size(layer_visual)
+    assert tuple(layer_visual_size) == (8, 4, 2)
+    assert tuple(axes_visual.node.text.text) == ('2', '1', '0')
