@@ -14,20 +14,23 @@ from qtpy.QtWidgets import (
 )
 from superqt import QLargeIntSpinBox
 
-from ...layers.labels._labels_constants import (
+from napari._qt.layer_controls.qt_layer_controls_base import QtLayerControls
+from napari._qt.utils import set_widgets_enabled_with_opacity
+from napari._qt.widgets._slider_compat import QSlider
+from napari._qt.widgets.qt_mode_buttons import (
+    QtModePushButton,
+    QtModeRadioButton,
+)
+from napari.layers.labels._labels_constants import (
     LABEL_COLOR_MODE_TRANSLATIONS,
     LabelsRendering,
     Mode,
 )
-from ...layers.labels._labels_utils import get_dtype
-from ...utils._dtype import get_dtype_limits
-from ...utils.action_manager import action_manager
-from ...utils.events import disconnect_events
-from ...utils.translations import trans
-from ..utils import disable_with_opacity
-from ..widgets._slider_compat import QSlider
-from ..widgets.qt_mode_buttons import QtModePushButton, QtModeRadioButton
-from .qt_layer_controls_base import QtLayerControls
+from napari.layers.labels._labels_utils import get_dtype
+from napari.utils._dtype import get_dtype_limits
+from napari.utils.action_manager import action_manager
+from napari.utils.events import disconnect_events
+from napari.utils.translations import trans
 
 if TYPE_CHECKING:
     import napari.layers
@@ -65,11 +68,15 @@ class QtLabelsControls(QtLayerControls):
         Button to select PAN_ZOOM mode on Labels layer.
     pick_button : qtpy.QtWidgets.QtModeRadioButton
         Button to select PICKER mode on Labels layer.
+    preserveLabelsCheckBox : qtpy.QtWidgets.QCheckBox
+        Checkbox to control if existing labels are preserved
     erase_button : qtpy.QtWidgets.QtModeRadioButton
         Button to select ERASE mode on Labels layer.
     selectionSpinBox : superqt.QLargeIntSpinBox
         Widget to select a specific label by its index.
         N.B. cannot represent labels > 2**53.
+    selectedColorCheckbox : qtpy.QtWidgets.QCheckBox
+        Checkbox to control if only currently selected label is shown.
 
     Raises
     ------
@@ -80,11 +87,10 @@ class QtLabelsControls(QtLayerControls):
 
     layer: 'napari.layers.Labels'
 
-    def __init__(self, layer):
+    def __init__(self, layer) -> None:
         super().__init__(layer)
 
         self.layer.events.mode.connect(self._on_mode_change)
-        self.layer.events._ndisplay.connect(self._on_ndisplay_change)
         self.layer.events.rendering.connect(self._on_rendering_change)
         self.layer.events.selected_label.connect(
             self._on_selected_label_change
@@ -95,10 +101,14 @@ class QtLabelsControls(QtLayerControls):
             self._on_n_edit_dimensions_change
         )
         self.layer.events.contour.connect(self._on_contour_change)
-        self.layer.events.editable.connect(self._on_editable_change)
+        self.layer.events.editable.connect(self._on_editable_or_visible_change)
+        self.layer.events.visible.connect(self._on_editable_or_visible_change)
         self.layer.events.preserve_labels.connect(
             self._on_preserve_labels_change
         )
+        self.layer.events.show_selected_label.connect(
+            self._on_show_selected_label_change
+        )
         self.layer.events.color_mode.connect(self._on_color_mode_change)
 
         # selection spinbox
@@ -136,7 +146,7 @@ class QtLabelsControls(QtLayerControls):
         self._on_n_edit_dimensions_change()
 
         self.contourSpinBox = QLargeIntSpinBox()
-        self.contourSpinBox.setRange(*dtype_lims)
+        self.contourSpinBox.setRange(0, dtype_lims[1])
         self.contourSpinBox.setToolTip(trans._('display contours of labels'))
         self.contourSpinBox.valueChanged.connect(self.change_contour)
         self.contourSpinBox.setKeyboardTracking(False)
@@ -157,6 +167,7 @@ class QtLabelsControls(QtLayerControls):
         )
         selectedColorCheckbox.stateChanged.connect(self.toggle_selected_mode)
         self.selectedColorCheckbox = selectedColorCheckbox
+        self._on_show_selected_label_change()
 
         # shuffle colormap button
         self.colormapUpdate = QtModePushButton(
@@ -168,22 +179,22 @@ class QtLabelsControls(QtLayerControls):
 
         self.panzoom_button = QtModeRadioButton(
             layer,
-            'zoom',
+            'pan',
             Mode.PAN_ZOOM,
             checked=True,
         )
         action_manager.bind_button(
-            'napari:activate_label_pan_zoom_mode', self.panzoom_button
+            'napari:activate_labels_pan_zoom_mode', self.panzoom_button
         )
 
         self.pick_button = QtModeRadioButton(layer, 'picker', Mode.PICK)
         action_manager.bind_button(
-            'napari:activate_label_picker_mode', self.pick_button
+            'napari:activate_labels_picker_mode', self.pick_button
         )
 
         self.paint_button = QtModeRadioButton(layer, 'paint', Mode.PAINT)
         action_manager.bind_button(
-            'napari:activate_paint_mode', self.paint_button
+            'napari:activate_labels_paint_mode', self.paint_button
         )
 
         self.fill_button = QtModeRadioButton(
@@ -192,7 +203,7 @@ class QtLabelsControls(QtLayerControls):
             Mode.FILL,
         )
         action_manager.bind_button(
-            'napari:activate_fill_mode',
+            'napari:activate_labels_fill_mode',
             self.fill_button,
         )
 
@@ -202,19 +213,26 @@ class QtLabelsControls(QtLayerControls):
             Mode.ERASE,
         )
         action_manager.bind_button(
-            'napari:activate_label_erase_mode',
+            'napari:activate_labels_erase_mode',
             self.erase_button,
         )
 
         # don't bind with action manager as this would remove "Toggle with {shortcut}"
 
+        self._EDIT_BUTTONS = (
+            self.paint_button,
+            self.pick_button,
+            self.fill_button,
+            self.erase_button,
+        )
+
         self.button_group = QButtonGroup(self)
         self.button_group.addButton(self.panzoom_button)
         self.button_group.addButton(self.paint_button)
         self.button_group.addButton(self.pick_button)
         self.button_group.addButton(self.fill_button)
         self.button_group.addButton(self.erase_button)
-        self._on_editable_change()
+        self._on_editable_or_visible_change()
 
         button_row = QHBoxLayout()
         button_row.addStretch(1)
@@ -237,7 +255,8 @@ class QtLabelsControls(QtLayerControls):
         renderComboBox.currentTextChanged.connect(self.changeRendering)
         self.renderComboBox = renderComboBox
         self.renderLabel = QLabel(trans._('rendering:'))
-        self._on_ndisplay_change()
+
+        self._on_ndisplay_changed()
 
         color_mode_comboBox = QComboBox(self)
         for index, (data, text) in enumerate(
@@ -338,7 +357,16 @@ class QtLabelsControls(QtLayerControls):
         self.setFocus()
 
     def toggle_selected_mode(self, state):
-        self.layer.show_selected_label = state == Qt.CheckState.Checked
+        """Toggle display of selected label only.
+
+        Parameters
+        ----------
+        state : int
+            Integer value of Qt.CheckState that indicates the check state of selectedColorCheckbox
+        """
+        self.layer.show_selected_label = (
+            Qt.CheckState(state) == Qt.CheckState.Checked
+        )
 
     def changeSize(self, value):
         """Change paint brush size.
@@ -355,10 +383,10 @@ class QtLabelsControls(QtLayerControls):
 
         Parameters
         ----------
-        state : QCheckBox
-            Checkbox indicating if labels are contiguous.
+        state : int
+            Integer value of Qt.CheckState that indicates the check state of contigCheckBox
         """
-        self.layer.contiguous = state == Qt.CheckState.Checked
+        self.layer.contiguous = Qt.CheckState(state) == Qt.CheckState.Checked
 
     def change_n_edit_dim(self, value):
         """Change the number of editable dimensions of label layer.
@@ -389,10 +417,12 @@ class QtLabelsControls(QtLayerControls):
 
         Parameters
         ----------
-        state : QCheckBox
-            Checkbox indicating if overwriting label is enabled.
+        state : int
+            Integer value of Qt.CheckState that indicates the check state of preserveLabelsCheckBox
         """
-        self.layer.preserve_labels = state == Qt.CheckState.Checked
+        self.layer.preserve_labels = (
+            Qt.CheckState(state) == Qt.CheckState.Checked
+        )
 
     def change_color_mode(self):
         """Change color mode of label layer"""
@@ -435,6 +465,13 @@ class QtLabelsControls(QtLayerControls):
         with self.layer.events.preserve_labels.blocker():
             self.preserveLabelsCheckBox.setChecked(self.layer.preserve_labels)
 
+    def _on_show_selected_label_change(self):
+        """Receive layer model show_selected_labels event and update the checkbox."""
+        with self.layer.events.show_selected_label.blocker():
+            self.selectedColorCheckbox.setChecked(
+                self.layer.show_selected_label
+            )
+
     def _on_color_mode_change(self):
         """Receive layer model color."""
         with self.layer.events.color_mode.blocker():
@@ -442,27 +479,12 @@ class QtLabelsControls(QtLayerControls):
                 self.colorModeComboBox.findData(self.layer.color_mode)
             )
 
-    def _on_editable_change(self):
-        """Receive layer model editable change event & enable/disable buttons."""
-        # In 3D mode, we need to disable all buttons other than picking
-        # (only picking works in 3D)
-        widget_list = [
-            'pick_button',
-            'fill_button',
-            'paint_button',
-            'erase_button',
-        ]
-        widgets_to_toggle = {
-            (2, True): widget_list,
-            (2, False): widget_list,
-            (3, True): widget_list,
-            (3, False): widget_list,
-        }
-
-        disable_with_opacity(
+    def _on_editable_or_visible_change(self):
+        """Receive layer model editable/visible change event & enable/disable buttons."""
+        set_widgets_enabled_with_opacity(
             self,
-            widgets_to_toggle[(self.layer._ndisplay, self.layer.editable)],
-            self.layer.editable,
+            self._EDIT_BUTTONS,
+            self.layer.editable and self.layer.visible,
         )
 
     def _on_rendering_change(self):
@@ -473,16 +495,11 @@ class QtLabelsControls(QtLayerControls):
             )
             self.renderComboBox.setCurrentIndex(index)
 
-    def _on_ndisplay_change(self):
-        """Toggle between 2D and 3D visualization modes."""
-        if self.layer._ndisplay == 2:
-            self.renderComboBox.hide()
-            self.renderLabel.hide()
-        else:
-            self.renderComboBox.show()
-            self.renderLabel.show()
-
-        self._on_editable_change()
+    def _on_ndisplay_changed(self):
+        render_visible = self.ndisplay == 3
+        self.renderComboBox.setVisible(render_visible)
+        self.renderLabel.setVisible(render_visible)
+        self._on_editable_or_visible_change()
 
     def deleteLater(self):
         disconnect_events(self.layer.events, self.colorBox)
@@ -498,7 +515,7 @@ class QtColorBox(QWidget):
         An instance of a napari layer.
     """
 
-    def __init__(self, layer):
+    def __init__(self, layer) -> None:
         super().__init__()
 
         self.layer = layer
@@ -552,8 +569,7 @@ class QtColorBox(QWidget):
                         painter.setBrush(QColor(25, 25, 25))
                     painter.drawRect(i * 4, j * 4, 5, 5)
         else:
-            color = np.multiply(self.layer._selected_color, self.layer.opacity)
-            color = np.round(255 * color).astype(int)
+            color = np.round(255 * self.layer._selected_color).astype(int)
             painter.setPen(QColor(*list(color)))
             painter.setBrush(QColor(*list(color)))
             painter.drawRect(0, 0, self._height, self._height)
