@@ -23,7 +23,16 @@ from typing import (
 )
 from weakref import WeakValueDictionary
 
-from qtpy.QtCore import QEvent, QEventLoop, QPoint, QProcess, QSize, Qt, Slot
+from qtpy.QtCore import (
+    QEvent,
+    QEventLoop,
+    QPoint,
+    QProcess,
+    QRect,
+    QSize,
+    Qt,
+    Slot,
+)
 from qtpy.QtGui import QIcon
 from qtpy.QtWidgets import (
     QApplication,
@@ -31,35 +40,46 @@ from qtpy.QtWidgets import (
     QDockWidget,
     QHBoxLayout,
     QMainWindow,
+    QMenu,
     QShortcut,
     QToolTip,
     QWidget,
 )
 from superqt.utils import QSignalThrottler
 
-from ..plugins import menu_item_template as plugin_menu_item_template
-from ..plugins import plugin_manager
-from ..settings import get_settings
-from ..utils import perf
-from ..utils._proxies import PublicOnlyProxy
-from ..utils.io import imsave
-from ..utils.misc import in_ipython, in_jupyter, running_as_bundled_app
-from ..utils.notifications import Notification
-from ..utils.theme import _themes, get_system_theme
-from ..utils.translations import trans
-from . import menus
-from .dialogs.confirm_close_dialog import ConfirmCloseDialog
-from .dialogs.qt_activity_dialog import QtActivityDialog
-from .dialogs.qt_notification import NapariQtNotification
-from .qt_event_loop import NAPARI_ICON_PATH, get_app, quit_app
-from .qt_resources import get_stylesheet
-from .qt_viewer import QtViewer
-from .utils import QImg2array, qbytearray_to_str, str_to_qbytearray
-from .widgets.qt_viewer_dock_widget import (
+from napari._app_model.constants import MenuId
+from napari._qt import menus
+from napari._qt._qapp_model import build_qmodel_menu
+from napari._qt._qapp_model.qactions import init_qactions
+from napari._qt.dialogs.confirm_close_dialog import ConfirmCloseDialog
+from napari._qt.dialogs.qt_activity_dialog import QtActivityDialog
+from napari._qt.dialogs.qt_notification import NapariQtNotification
+from napari._qt.qt_event_loop import NAPARI_ICON_PATH, get_app, quit_app
+from napari._qt.qt_resources import get_stylesheet
+from napari._qt.qt_viewer import QtViewer
+from napari._qt.utils import QImg2array, qbytearray_to_str, str_to_qbytearray
+from napari._qt.widgets.qt_viewer_dock_widget import (
     _SHORTCUT_DEPRECATION_STRING,
     QtViewerDockWidget,
 )
-from .widgets.qt_viewer_status_bar import ViewerStatusBar
+from napari._qt.widgets.qt_viewer_status_bar import ViewerStatusBar
+from napari.plugins import (
+    menu_item_template as plugin_menu_item_template,
+    plugin_manager,
+)
+from napari.settings import get_settings
+from napari.utils import perf
+from napari.utils._proxies import PublicOnlyProxy
+from napari.utils.io import imsave
+from napari.utils.misc import (
+    in_ipython,
+    in_jupyter,
+    in_python_repl,
+    running_as_constructor_app,
+)
+from napari.utils.notifications import Notification
+from napari.utils.theme import _themes, get_system_theme
+from napari.utils.translations import trans
 
 _sentinel = object()
 
@@ -67,7 +87,7 @@ if TYPE_CHECKING:
     from magicgui.widgets import Widget
     from qtpy.QtGui import QImage
 
-    from ..viewer import Viewer
+    from napari.viewer import Viewer
 
 
 class _QtMainWindow(QMainWindow):
@@ -82,15 +102,20 @@ class _QtMainWindow(QMainWindow):
     # *no* active windows, so we want to track the most recently active windows
     _instances: ClassVar[List['_QtMainWindow']] = []
 
-    def __init__(self, viewer: 'Viewer', parent=None) -> None:
+    # `window` is passed through on construction so it's available to a window
+    # provider for dependency injection
+    # See https://github.com/napari/napari/pull/4826
+    def __init__(
+        self, viewer: 'Viewer', window: 'Window', parent=None
+    ) -> None:
         super().__init__(parent)
         self._ev = None
+        self._window = window
         self._qt_viewer = QtViewer(viewer, show_welcome_screen=True)
         self._quit_app = False
 
         self.setWindowIcon(QIcon(self._window_icon))
         self.setAttribute(Qt.WidgetAttribute.WA_DeleteOnClose)
-        self.setUnifiedTitleAndToolBarOnMac(True)
         center = QWidget(self)
         center.setLayout(QHBoxLayout())
         center.layout().addWidget(self._qt_viewer)
@@ -104,9 +129,14 @@ class _QtMainWindow(QMainWindow):
         self._window_pos = None
         self._old_size = None
         self._positions = []
+        self._toggle_menubar_visibility = False
 
-        act_dlg = QtActivityDialog(self._qt_viewer._canvas_overlay)
-        self._qt_viewer._canvas_overlay.resized.connect(
+        self._is_close_dialog = {False: True, True: True}
+        # this ia sa workaround for #5335 issue. The dict is used to not
+        # collide shortcuts for close and close all windows
+
+        act_dlg = QtActivityDialog(self._qt_viewer._welcome_widget)
+        self._qt_viewer._welcome_widget.resized.connect(
             act_dlg.move_to_bottom_right
         )
         act_dlg.hide()
@@ -114,6 +144,10 @@ class _QtMainWindow(QMainWindow):
 
         self.setStatusBar(ViewerStatusBar(self))
 
+        # Prevent QLineEdit based widgets to keep focus even when clicks are
+        # done outside the widget. See #1571
+        self.setFocusPolicy(Qt.FocusPolicy.StrongFocus)
+
         settings = get_settings()
 
         # TODO:
@@ -133,9 +167,15 @@ class _QtMainWindow(QMainWindow):
                 self._qt_viewer.canvas._backend.screen_changed
             )
 
+        # this is the line that initializes any Qt-based app-model Actions that
+        # were defined somewhere in the `_qt` module and imported in init_qactions
+        init_qactions()
+
         self.status_throttler = QSignalThrottler(parent=self)
         self.status_throttler.setTimeout(50)
+        self._throttle_cursor_to_status_connection(viewer)
 
+    def _throttle_cursor_to_status_connection(self, viewer: 'Viewer'):
         # In the GUI we expect lots of changes to the cursor position, so
         # replace the direct connection with a throttled one.
         with contextlib.suppress(IndexError):
@@ -183,6 +223,32 @@ class _QtMainWindow(QMainWindow):
                 inst.append(inst.pop(inst.index(self)))
         return super().event(e)
 
+    def eventFilter(self, source, event):
+        # Handle showing hidden menubar on mouse move event.
+        # We do not hide menubar when a menu is being shown or
+        # we are not in menubar toggled state
+        if (
+            QApplication.activePopupWidget() is None
+            and self._toggle_menubar_visibility
+        ):
+            if event.type() == QEvent.MouseMove:
+                if self.menuBar().isHidden():
+                    rect = self.geometry()
+                    # set mouse-sensitive zone to trigger showing the menubar
+                    rect.setHeight(25)
+                    if rect.contains(event.globalPos()):
+                        self.menuBar().show()
+                else:
+                    rect = QRect(
+                        self.menuBar().mapToGlobal(QPoint(0, 0)),
+                        self.menuBar().size(),
+                    )
+                    if not rect.contains(event.globalPos()):
+                        self.menuBar().hide()
+            elif event.type() == QEvent.Leave and source is self:
+                self.menuBar().hide()
+        return QMainWindow.eventFilter(self, source, event)
+
     def _load_window_settings(self):
         """
         Load window layout settings from configuration.
@@ -195,9 +261,10 @@ class _QtMainWindow(QMainWindow):
         if not window_position:
             window_position = (self.x(), self.y())
         else:
-            width, height = window_position
-            screen_geo = QApplication.primaryScreen().geometry()
-            if screen_geo.width() < width or screen_geo.height() < height:
+            origin_x, origin_y = window_position
+            screen = QApplication.screenAt(QPoint(origin_x, origin_y))
+            screen_geo = screen.geometry() if screen else None
+            if not screen_geo:
                 window_position = (self.x(), self.y())
 
         return (
@@ -258,7 +325,7 @@ class _QtMainWindow(QMainWindow):
 
         # Toggling the console visibility is disabled when it is not
         # available, so ensure that it is hidden.
-        if in_ipython():
+        if in_ipython() or in_jupyter() or in_python_repl():
             self._qt_viewer.dockConsole.setVisible(False)
 
         if window_fullscreen:
@@ -298,13 +365,23 @@ class _QtMainWindow(QMainWindow):
             self.status_throttler._timer.stop()
         if not quit_app and not self._qt_viewer.viewer.layers:
             return super().close()
+        confirm_need_local = confirm_need and self._is_close_dialog[quit_app]
+        self._is_close_dialog[quit_app] = False
+        # here we save information that we could request confirmation on close
+        # So fi function `close` is called again, we don't ask again but just close
         if (
-            not confirm_need
+            not confirm_need_local
             or not get_settings().application.confirm_close_window
             or ConfirmCloseDialog(self, quit_app).exec_() == QDialog.Accepted
         ):
             self._quit_app = quit_app
+            self._is_close_dialog[quit_app] = True
+            # here we inform that confirmation dialog is not open
+            self._qt_viewer.dims.stop()
             return super().close()
+        self._is_close_dialog[quit_app] = True
+        return None
+        # here we inform that confirmation dialog is not open
 
     def close_window(self):
         """Close active dialog or active window."""
@@ -403,6 +480,8 @@ class _QtMainWindow(QMainWindow):
                 time.sleep(0.1)
                 QApplication.processEvents()
 
+        self._qt_viewer.dims.stop()
+
         if self._quit_app:
             quit_app()
 
@@ -413,12 +492,22 @@ class _QtMainWindow(QMainWindow):
         process = QProcess()
         process.setProgram(sys.executable)
 
-        if not running_as_bundled_app():
+        if not running_as_constructor_app():
             process.setArguments(sys.argv)
 
         process.startDetached()
         self.close(quit_app=True)
 
+    def toggle_menubar_visibility(self):
+        """
+        Change menubar to be shown or to be hidden and shown on mouse movement.
+
+        For the mouse movement functionality see the `eventFilter` implementation.
+        """
+        self._toggle_menubar_visibility = not self._toggle_menubar_visibility
+        self.menuBar().setVisible(not self._toggle_menubar_visibility)
+        return self._toggle_menubar_visibility
+
     @staticmethod
     @Slot(Notification)
     def show_notification(notification: Notification):
@@ -449,9 +538,9 @@ class Window:
         Window menu.
     """
 
-    def __init__(self, viewer: 'Viewer', *, show: bool = True):
+    def __init__(self, viewer: 'Viewer', *, show: bool = True) -> None:
         # create QApplication if it doesn't already exist
-        get_app()
+        qapp = get_app()
 
         # Dictionary holding dock widgets
         self._dock_widgets: Dict[
@@ -460,7 +549,8 @@ class Window:
         self._unnamed_dockwidget_count = 1
 
         # Connect the Viewer and create the Main Window
-        self._qt_window = _QtMainWindow(viewer)
+        self._qt_window = _QtMainWindow(viewer, self)
+        qapp.installEventFilter(self._qt_window)
 
         # connect theme events before collecting plugin-provided themes
         # to ensure icons from the plugins are generated correctly.
@@ -627,13 +717,17 @@ class Window:
 
         self.file_menu = menus.FileMenu(self)
         self.main_menu.addMenu(self.file_menu)
-        self.view_menu = menus.ViewMenu(self)
+        self.view_menu = build_qmodel_menu(
+            MenuId.MENUBAR_VIEW, title=trans._('&View'), parent=self._qt_window
+        )
         self.main_menu.addMenu(self.view_menu)
-        self.window_menu = menus.WindowMenu(self)
-        self.main_menu.addMenu(self.window_menu)
         self.plugins_menu = menus.PluginsMenu(self)
         self.main_menu.addMenu(self.plugins_menu)
-        self.help_menu = menus.HelpMenu(self)
+        self.window_menu = menus.WindowMenu(self)
+        self.main_menu.addMenu(self.window_menu)
+        self.help_menu = build_qmodel_menu(
+            MenuId.MENUBAR_HELP, title=trans._('&Help'), parent=self._qt_window
+        )
         self.main_menu.addMenu(self.help_menu)
 
         if perf.USE_PERFMON:
@@ -647,8 +741,8 @@ class Window:
         show the menubar, since menubar shortcuts are only available while the
         menubar is visible.
         """
-        self.main_menu.setVisible(not self.main_menu.isVisible())
-        self._main_menu_shortcut.setEnabled(not self.main_menu.isVisible())
+        toggle_menubar_visibility = self._qt_window.toggle_menubar_visibility()
+        self._main_menu_shortcut.setEnabled(toggle_menubar_visibility)
 
     def _toggle_fullscreen(self):
         """Toggle fullscreen mode."""
@@ -688,7 +782,7 @@ class Window:
             A 2-tuple containing (the DockWidget instance, the plugin widget
             instance).
         """
-        from ..plugins import _npe2
+        from napari.plugins import _npe2
 
         Widget = None
         dock_kwargs = {}
@@ -739,7 +833,7 @@ class Window:
         """
         full_name = plugin_menu_item_template.format(plugin_name, widget_name)
         if full_name in self._dock_widgets:
-            return
+            return None
 
         func = plugin_manager._function_widgets[plugin_name][widget_name]
 
@@ -757,8 +851,8 @@ class Window:
         allowed_areas: Optional[Sequence[str]] = None,
         shortcut=_sentinel,
         add_vertical_stretch=True,
-        menu=None,
         tabify: bool = False,
+        menu: Optional[QMenu] = None,
     ):
         """Convenience method to add a QDockWidget to the main window.
 
@@ -790,6 +884,10 @@ class Window:
                 The shortcut parameter is deprecated since version 0.4.8, please use
                 the action and shortcut manager APIs. The new action manager and
                 shortcut API allow user configuration and localisation.
+        tabify : bool
+            Flag to tabify dockwidget or not.
+        menu : QMenu, optional
+            Menu bar to add toggle action to. If `None` nothing added to menu.
 
         Returns
         -------
@@ -831,7 +929,7 @@ class Window:
                 add_vertical_stretch=add_vertical_stretch,
             )
 
-        self._add_viewer_dock_widget(dock_widget, menu=menu, tabify=tabify)
+        self._add_viewer_dock_widget(dock_widget, tabify=tabify, menu=menu)
 
         if hasattr(widget, 'reset_choices'):
             # Keep the dropdown menus in the widget in sync with the layer model
@@ -848,7 +946,10 @@ class Window:
         return dock_widget
 
     def _add_viewer_dock_widget(
-        self, dock_widget: QtViewerDockWidget, tabify=False, menu=None
+        self,
+        dock_widget: QtViewerDockWidget,
+        tabify: bool = False,
+        menu: Optional[QMenu] = None,
     ):
         """Add a QtViewerDockWidget to the main window
 
@@ -860,6 +961,8 @@ class Window:
             `dock_widget` will be added to the main window.
         tabify : bool
             Flag to tabify dockwidget or not.
+        menu : QMenu, optional
+            Menu bar to add toggle action to. If `None` nothing added to menu.
         """
         # Find if any othe dock widgets are currently in area
         current_dws_in_area = [
@@ -878,7 +981,7 @@ class Window:
                 dock_widget.show()
                 dock_widget.raise_()
             elif dock_widget.area in ('right', 'left'):
-                _wdg = current_dws_in_area + [dock_widget]
+                _wdg = [*current_dws_in_area, dock_widget]
                 # add sizes to push lower widgets up
                 sizes = list(range(1, len(_wdg) * 4, 4))
                 self._qt_window.resizeDocks(
@@ -899,7 +1002,6 @@ class Window:
                 action.setShortcut(shortcut)
 
             menu.addAction(action)
-        # self.window_menu.addAction(action)
 
         # see #3663, to fix #3624 more generally
         dock_widget.setFloating(False)
@@ -1022,13 +1124,13 @@ class Window:
                 allowed_areas=allowed_areas,
                 shortcut=shortcut,
             )
-        else:
-            return self.add_dock_widget(
-                widget,
-                name=name or function.__name__.replace('_', ' '),
-                area=area,
-                allowed_areas=allowed_areas,
-            )
+
+        return self.add_dock_widget(
+            widget,
+            name=name or function.__name__.replace('_', ' '),
+            area=area,
+            allowed_areas=allowed_areas,
+        )
 
     def resize(self, width, height):
         """Resize the window.
@@ -1086,32 +1188,32 @@ class Window:
         settings = get_settings()
         try:
             self._qt_window.show(block=block)
-        except (AttributeError, RuntimeError):
+        except (AttributeError, RuntimeError) as e:
             raise RuntimeError(
                 trans._(
                     "This viewer has already been closed and deleted. Please create a new one.",
                     deferred=True,
                 )
-            )
+            ) from e
 
         if settings.application.first_time:
             settings.application.first_time = False
             try:
                 self._qt_window.resize(self._qt_window.layout().sizeHint())
-            except (AttributeError, RuntimeError):
+            except (AttributeError, RuntimeError) as e:
                 raise RuntimeError(
                     trans._(
                         "This viewer has already been closed and deleted. Please create a new one.",
                         deferred=True,
                     )
-                )
+                ) from e
         else:
             try:
                 if settings.application.save_window_geometry:
                     self._qt_window._set_window_settings(
                         *self._qt_window._load_window_settings()
                     )
-            except Exception as err:
+            except Exception as err:  # noqa: BLE001
                 import warnings
 
                 warnings.warn(
@@ -1153,18 +1255,16 @@ class Window:
         """Update widget color theme."""
         settings = get_settings()
         with contextlib.suppress(AttributeError, RuntimeError):
-            if event:
-                value = event.value
-                self._qt_viewer.viewer.theme = value
-                settings.appearance.theme = value
-            else:
-                value = (
-                    get_system_theme()
-                    if settings.appearance.theme == "system"
-                    else self._qt_viewer.viewer.theme
+            value = event.value if event else settings.appearance.theme
+            self._qt_viewer.viewer.theme = value
+            if value == "system":
+                # system isn't a theme, so get the name and set style sheet
+                actual_theme_name = get_system_theme()
+                self._qt_window.setStyleSheet(
+                    get_stylesheet(actual_theme_name)
                 )
-
-            self._qt_window.setStyleSheet(get_stylesheet(value))
+            else:
+                self._qt_window.setStyleSheet(get_stylesheet(value))
 
     def _status_changed(self, event):
         """Update status bar.
@@ -1234,7 +1334,7 @@ class Window:
         -------
         img : QImage
         """
-        from .utils import add_flash_animation
+        from napari._qt.utils import add_flash_animation
 
         if canvas_only:
             canvas = self._qt_viewer.canvas
@@ -1259,7 +1359,7 @@ class Window:
             try:
                 img = self._qt_viewer.canvas.native.grabFramebuffer()
                 if flash:
-                    add_flash_animation(self._qt_viewer._canvas_overlay)
+                    add_flash_animation(self._qt_viewer._welcome_widget)
             finally:
                 # make sure we always go back to the right canvas size
                 if size is not None or scale is not None:
@@ -1343,7 +1443,7 @@ class Window:
 
 def _instantiate_dock_widget(wdg_cls, viewer: 'Viewer'):
     # if the signature is looking a for a napari viewer, pass it.
-    from ..viewer import Viewer
+    from napari.viewer import Viewer
 
     kwargs = {}
     try:
