@@ -11,7 +11,7 @@ from qtpy.QtWidgets import (
     QWidget,
 )
 
-from ...utils.translations import translator
+from napari.utils.translations import translator
 
 trans = translator.load()
 
@@ -29,7 +29,7 @@ class QtStar(QFrame):
         self,
         parent: QWidget = None,
         value: int = None,
-    ):
+    ) -> None:
         super().__init__(parent)
         self._value = value
 
@@ -115,7 +115,6 @@ class QtStar(QFrame):
             x_adj = star_center_x - x
             y_adj = star_center_y - y + 3
             if n == 0:
-
                 path.moveTo(x_adj, y_adj)
             else:
                 path.lineTo(x_adj, y_adj)
@@ -143,7 +142,7 @@ class QtTriangle(QFrame):
         value: int = 1,
         min_value: int = 1,
         max_value: int = 10,
-    ):
+    ) -> None:
         super().__init__(parent)
         self._max_value = max_value
         self._min_value = min_value
@@ -312,7 +311,7 @@ class QtHighlightSizePreviewWidget(QWidget):
         min_value: int = 1,
         max_value: int = 10,
         unit: str = "px",
-    ):
+    ) -> None:
         super().__init__(parent)
 
         self.setGeometry(300, 300, 125, 110)
