@@ -1,9 +1,39 @@
-from typing import Tuple
+from __future__ import annotations
+
+from typing import List, Tuple
 
 import numpy as np
 from vispy.scene.widgets.viewbox import ViewBox
 
-from ...layers import (
+from napari._vispy.layers.base import VispyBaseLayer
+from napari._vispy.layers.image import VispyImageLayer
+from napari._vispy.layers.labels import VispyLabelsLayer
+from napari._vispy.layers.points import VispyPointsLayer
+from napari._vispy.layers.shapes import VispyShapesLayer
+from napari._vispy.layers.surface import VispySurfaceLayer
+from napari._vispy.layers.tracks import VispyTracksLayer
+from napari._vispy.layers.vectors import VispyVectorsLayer
+from napari._vispy.overlays.axes import VispyAxesOverlay
+from napari._vispy.overlays.base import VispyBaseOverlay
+from napari._vispy.overlays.bounding_box import VispyBoundingBoxOverlay
+from napari._vispy.overlays.brush_circle import VispyBrushCircleOverlay
+from napari._vispy.overlays.interaction_box import (
+    VispySelectionBoxOverlay,
+    VispyTransformBoxOverlay,
+)
+from napari._vispy.overlays.scale_bar import VispyScaleBarOverlay
+from napari._vispy.overlays.text import VispyTextOverlay
+from napari.components.overlays import (
+    AxesOverlay,
+    BoundingBoxOverlay,
+    BrushCircleOverlay,
+    Overlay,
+    ScaleBarOverlay,
+    SelectionBoxOverlay,
+    TextOverlay,
+    TransformBoxOverlay,
+)
+from napari.layers import (
     Image,
     Labels,
     Layer,
@@ -13,16 +43,8 @@ from ...layers import (
     Tracks,
     Vectors,
 )
-from ...utils.config import async_octree
-from ...utils.translations import trans
-from ..layers.base import VispyBaseLayer
-from ..layers.image import VispyImageLayer
-from ..layers.labels import VispyLabelsLayer
-from ..layers.points import VispyPointsLayer
-from ..layers.shapes import VispyShapesLayer
-from ..layers.surface import VispySurfaceLayer
-from ..layers.tracks import VispyTracksLayer
-from ..layers.vectors import VispyVectorsLayer
+from napari.utils.config import async_octree
+from napari.utils.translations import trans
 
 layer_to_visual = {
     Image: VispyImageLayer,
@@ -35,9 +57,21 @@ layer_to_visual = {
 }
 
 
+overlay_to_visual = {
+    ScaleBarOverlay: VispyScaleBarOverlay,
+    TextOverlay: VispyTextOverlay,
+    AxesOverlay: VispyAxesOverlay,
+    BoundingBoxOverlay: VispyBoundingBoxOverlay,
+    TransformBoxOverlay: VispyTransformBoxOverlay,
+    SelectionBoxOverlay: VispySelectionBoxOverlay,
+    BrushCircleOverlay: VispyBrushCircleOverlay,
+}
+
 if async_octree:
-    from ...layers.image.experimental.octree_image import _OctreeImageBase
-    from ..experimental.vispy_tiled_image_layer import VispyTiledImageLayer
+    from napari._vispy.experimental.vispy_tiled_image_layer import (
+        VispyTiledImageLayer,
+    )
+    from napari.layers.image.experimental.octree_image import _OctreeImageBase
 
     # Insert _OctreeImageBase in front so it gets picked over plain Image.
     new_mapping = {_OctreeImageBase: VispyTiledImageLayer}
@@ -45,7 +79,7 @@ if async_octree:
     layer_to_visual = new_mapping
 
 
-def create_vispy_visual(layer: Layer) -> VispyBaseLayer:
+def create_vispy_layer(layer: Layer) -> VispyBaseLayer:
     """Create vispy visual for a layer based on its layer type.
 
     Parameters
@@ -55,8 +89,8 @@ def create_vispy_visual(layer: Layer) -> VispyBaseLayer:
 
     Returns
     -------
-    visual : vispy.scene.visuals.VisualNode
-        Vispy visual node
+    visual : VispyBaseLayer
+        Vispy layer
     """
     for layer_type, visual_class in layer_to_visual.items():
         if isinstance(layer, layer_type):
@@ -71,6 +105,33 @@ def create_vispy_visual(layer: Layer) -> VispyBaseLayer:
     )
 
 
+def create_vispy_overlay(overlay: Overlay, **kwargs) -> List[VispyBaseOverlay]:
+    """
+    Create vispy visuals for each overlay contained in an Overlays model based on their type,
+
+    Parameters
+    ----------
+    overlay : napari.components.overlays.VispyBaseOverlay
+        The overlay to create a visual for.
+
+    Returns
+    -------
+    visual : VispyBaseOverlay
+        Vispy overlay
+    """
+    for overlay_type, visual_class in overlay_to_visual.items():
+        if isinstance(overlay, overlay_type):
+            return visual_class(overlay=overlay, **kwargs)
+
+    raise TypeError(
+        trans._(
+            'Could not find VispyOverlay for overlay of type {dtype}',
+            deferred=True,
+            dtype=type(overlay),
+        )
+    )
+
+
 def get_view_direction_in_scene_coordinates(
     view: ViewBox,
     ndim: int,
