@@ -35,7 +35,7 @@ int detectAdjacentBackground(float val_neg, float val_pos)
 vec4 calculateCategoricalColor(vec4 betterColor, vec3 loc, vec3 step)
 {
     // Calculate color by incorporating ambient and diffuse lighting
-    vec4 color0 = $sample(u_volumetex, loc);
+    vec4 color0 = $get_data(loc);
     vec4 color1;
     vec4 color2;
     float val0 = colorToVal(color0);
@@ -48,22 +48,22 @@ vec4 calculateCategoricalColor(vec4 betterColor, vec3 loc, vec3 step)
 
     // calculate normal vector from gradient
     vec3 N; // normal
-    color1 = $sample( u_volumetex, loc+vec3(-step[0],0.0,0.0) );
-    color2 = $sample( u_volumetex, loc+vec3(step[0],0.0,0.0) );
+    color1 = $get_data(loc+vec3(-step[0],0.0,0.0));
+    color2 = $get_data(loc+vec3(step[0],0.0,0.0));
     val1 = colorToVal(color1);
     val2 = colorToVal(color2);
     N[0] = val1 - val2;
     n_bg_borders += detectAdjacentBackground(val1, val2);
 
-    color1 = $sample( u_volumetex, loc+vec3(0.0,-step[1],0.0) );
-    color2 = $sample( u_volumetex, loc+vec3(0.0,step[1],0.0) );
+    color1 = $get_data(loc+vec3(0.0,-step[1],0.0));
+    color2 = $get_data(loc+vec3(0.0,step[1],0.0));
     val1 = colorToVal(color1);
     val2 = colorToVal(color2);
     N[1] = val1 - val2;
     n_bg_borders += detectAdjacentBackground(val1, val2);
 
-    color1 = $sample( u_volumetex, loc+vec3(0.0,0.0,-step[2]) );
-    color2 = $sample( u_volumetex, loc+vec3(0.0,0.0,step[2]) );
+    color1 = $get_data(loc+vec3(0.0,0.0,-step[2]));
+    color2 = $get_data(loc+vec3(0.0,0.0,step[2]));
     val1 = colorToVal(color1);
     val2 = colorToVal(color2);
     N[2] = val1 - val2;
@@ -113,24 +113,24 @@ vec4 calculateCategoricalColor(vec4 betterColor, vec3 loc, vec3 step)
 }
 """
 
-ISO_CATEGORICAL_SNIPPETS = dict(
-    before_loop="""
+ISO_CATEGORICAL_SNIPPETS = {
+    "before_loop": """
         vec4 color3 = vec4(0.0);  // final color
         vec3 dstep = 1.5 / u_shape;  // step to sample derivative, set to match iso shader
         gl_FragColor = vec4(0.0);
         bool discard_fragment = true;
         """,
-    in_loop="""
+    "in_loop": """
         // check if value is different from the background value
         if ( floatNotEqual(val, categorical_bg_value) ) {
             // Take the last interval in smaller steps
             vec3 iloc = loc - step;
             for (int i=0; i<10; i++) {
-                color = $sample(u_volumetex, iloc);
-                if (floatNotEqual(color.g, categorical_bg_value) ) {
-                    // when the non-background value is reached
+                color = $get_data(iloc);
+                color = applyColormap(color.g);
+                if (floatNotEqual(color.a, 0) ) {
+                    // when the value mapped to non-transparent color is reached
                     // calculate the color (apply lighting effects)
-                    color = applyColormap(color.g);
                     color = calculateCategoricalColor(color, iloc, dstep);
                     gl_FragColor = color;
 
@@ -145,11 +145,11 @@ ISO_CATEGORICAL_SNIPPETS = dict(
             }
         }
         """,
-    after_loop="""
+    "after_loop": """
         if (discard_fragment)
             discard;
         """,
-)
+}
 
 shaders = BaseVolume._shaders.copy()
 before, after = shaders['fragment'].split('void main()')
