@@ -104,6 +104,21 @@ def test_add_multiscale():
     assert viewer.dims.ndim == 2
 
 
+def test_add_multiscale_image_with_negative_floats():
+    """See https://github.com/napari/napari/issues/5257"""
+    viewer = ViewerModel()
+    shapes = [(20, 10), (10, 5)]
+    data = [np.zeros(s, dtype=np.float64) for s in shapes]
+    data[0][-4:, -2:] = -1
+    data[1][-2:, -1:] = -1
+
+    viewer.add_image(data, multiscale=True)
+
+    assert len(viewer.layers) == 1
+    assert np.all(viewer.layers[0].data == data)
+    assert viewer.dims.ndim == 2
+
+
 def test_add_labels():
     """Test adding labels image."""
     viewer = ViewerModel()
@@ -303,17 +318,30 @@ def test_view_centering_with_points_add():
 
     viewer = ViewerModel()
     viewer.add_image(image)
-    assert tuple(viewer.dims.point) == (2, 5, 5)
+    assert tuple(viewer.dims.point) == (2, 4, 4)
 
     viewer.dims.set_point(0, 0)
     # viewer point shouldn't change after this
-    assert tuple(viewer.dims.point) == (0, 5, 5)
+    assert tuple(viewer.dims.point) == (0, 4, 4)
 
     pts_layer = viewer.add_points(ndim=3)
-    assert tuple(viewer.dims.point) == (0, 5, 5)
+    assert tuple(viewer.dims.point) == (0, 4, 4)
 
     pts_layer.add([(0, 8, 8)])
-    assert tuple(viewer.dims.point) == (0, 5, 5)
+    assert tuple(viewer.dims.point) == (0, 4, 4)
+
+
+def test_view_centering_with_scale():
+    """Regression test for issue #5735"""
+    image = np.zeros((5, 10, 10))
+
+    viewer = ViewerModel()
+    viewer.add_image(image, scale=(1, 1, 1))
+    assert tuple(viewer.dims.point) == (2, 4, 4)
+
+    viewer.layers.pop()
+    viewer.add_image(image, scale=(2, 1, 1))
+    assert tuple(viewer.dims.point) == (4, 4, 4)
 
 
 def test_new_shapes():
@@ -343,7 +371,7 @@ def test_swappable_dims():
     image_data = np.random.random((7, 12, 10, 15))
     image_name = viewer.add_image(image_data).name
     assert np.all(
-        viewer.layers[image_name]._data_view == image_data[3, 6, :, :]
+        viewer.layers[image_name]._data_view == image_data[3, 5, :, :]
     )
 
     points_data = np.random.randint(6, size=(10, 4))
@@ -357,17 +385,17 @@ def test_swappable_dims():
     # midpoints indices into the data below depend on the data range.
     # This depends on the values in vectors_data and thus the random seed.
     assert np.all(
-        viewer.layers[labels_name]._slice.image.raw == labels_data[3, 6, :, :]
+        viewer.layers[labels_name]._slice.image.raw == labels_data[3, 5, :, :]
     )
 
     # Swap dims
     viewer.dims.order = [0, 2, 1, 3]
     assert viewer.dims.order == (0, 2, 1, 3)
     assert np.all(
-        viewer.layers[image_name]._data_view == image_data[3, :, 5, :]
+        viewer.layers[image_name]._data_view == image_data[3, :, 4, :]
     )
     assert np.all(
-        viewer.layers[labels_name]._slice.image.raw == labels_data[3, :, 5, :]
+        viewer.layers[labels_name]._slice.image.raw == labels_data[3, :, 4, :]
     )
 
 
@@ -377,7 +405,7 @@ def test_grid():
 
     np.random.seed(0)
     # Add image
-    for i in range(6):
+    for _i in range(6):
         data = np.random.random((15, 15))
         viewer.add_image(data)
     assert not viewer.grid.enabled
@@ -697,7 +725,7 @@ def test_add_remove_layer_no_callbacks(Layer, data, ndim):
     assert layer.ndim == ndim
 
     # Check that no internal callbacks have been registered
-    len(layer.events.callbacks) == 0
+    assert len(layer.events.callbacks) == 0
     for em in layer.events.emitters.values():
         assert len(em.callbacks) == 0
 
@@ -734,7 +762,7 @@ def test_add_remove_layer_external_callbacks(Layer, data, ndim):
     layer.events.connect(my_custom_callback)
 
     # Check that no internal callbacks have been registered
-    len(layer.events.callbacks) == 1
+    assert len(layer.events.callbacks) == 1
     for em in layer.events.emitters.values():
         if not isinstance(em, WarningEmitter):
             assert len(em.callbacks) == 1
@@ -758,7 +786,7 @@ def test_add_remove_layer_external_callbacks(Layer, data, ndim):
 
 
 @pytest.mark.parametrize(
-    'field', ['camera', 'cursor', 'dims', 'grid', 'layers', 'scale_bar']
+    'field', ['camera', 'cursor', 'dims', 'grid', 'layers']
 )
 def test_not_mutable_fields(field):
     """Test appropriate fields are not mutable."""
@@ -770,7 +798,7 @@ def test_not_mutable_fields(field):
     assert not hasattr(viewer.events, field)
 
     # Check attribute is not settable
-    with pytest.raises(TypeError) as err:
+    with pytest.raises((TypeError, ValueError)) as err:
         setattr(viewer, field, 'test')
 
     assert 'has allow_mutation set to False and cannot be assigned' in str(
@@ -905,3 +933,24 @@ def test_open_or_get_error_preferred_fails(builtins, tmp_path):
         ReaderPluginError, match='Tried opening with napari, but failed.'
     ):
         viewer._open_or_raise_error([str(pth)])
+
+
+def test_slice_order_with_mixed_dims():
+    viewer = ViewerModel(ndisplay=2)
+    image_2d = viewer.add_image(np.zeros((4, 5)))
+    image_3d = viewer.add_image(np.zeros((3, 4, 5)))
+    image_4d = viewer.add_image(np.zeros((2, 3, 4, 5)))
+
+    # With standard ordering, the shapes of the slices match,
+    # so are trivially numpy-broadcastable.
+    assert image_2d._slice.image.view.shape == (4, 5)
+    assert image_3d._slice.image.view.shape == (4, 5)
+    assert image_4d._slice.image.view.shape == (4, 5)
+
+    viewer.dims.order = (2, 1, 0, 3)
+
+    # With non-standard ordering, the shapes of the slices do not match,
+    # and are not numpy-broadcastable.
+    assert image_2d._slice.image.view.shape == (4, 5)
+    assert image_3d._slice.image.view.shape == (3, 5)
+    assert image_4d._slice.image.view.shape == (2, 5)
