@@ -2,14 +2,13 @@ from __future__ import annotations
 
 from typing import TYPE_CHECKING
 
-from ..settings import get_settings
-from ..utils.action_manager import action_manager
-from ..utils.theme import available_themes
-from ..utils.translations import trans
-from .viewer_model import ViewerModel
+from napari.components.viewer_model import ViewerModel
+from napari.utils.action_manager import action_manager
+from napari.utils.theme import available_themes, get_system_theme
+from napari.utils.translations import trans
 
 if TYPE_CHECKING:
-    from ..viewer import Viewer
+    from napari.viewer import Viewer
 
 
 def register_viewer_action(description):
@@ -22,7 +21,7 @@ def register_viewer_action(description):
 
     def _inner(func):
         action_manager.register_action(
-            name='napari:' + func.__name__,
+            name=f'napari:{func.__name__}',
             command=func,
             description=description,
             keymapprovider=ViewerModel,
@@ -34,7 +33,6 @@ def register_viewer_action(description):
 
 @register_viewer_action(trans._("Reset scroll."))
 def reset_scroll_progress(viewer: Viewer):
-
     # on key press
     viewer.dims._scroll_progress = 0
     yield
@@ -56,18 +54,21 @@ def toggle_ndisplay(viewer: Viewer):
 # ```
 # RuntimeError: wrapped C/C++ object of type CanvasBackendDesktop has been deleted
 # ```
-@register_viewer_action(trans._("Toggle theme."))
-def toggle_theme(viewer: Viewer):
-    """Toggle theme for viewer"""
-    settings = get_settings()
+@register_viewer_action(trans._("Toggle current viewer theme."))
+def toggle_theme(viewer: ViewerModel):
+    """Toggle theme for current viewer"""
     themes = available_themes()
-    current_theme = settings.appearance.theme
+    current_theme = viewer.theme
+    # Check what the system theme is, to toggle properly
+    if current_theme == 'system':
+        current_theme = get_system_theme()
     idx = themes.index(current_theme)
-    idx += 1
-    if idx == len(themes):
-        idx = 0
+    idx = (idx + 1) % len(themes)
+    # Don't toggle to system, just among actual themes
+    if themes[idx] == 'system':
+        idx = (idx + 1) % len(themes)
 
-    settings.appearance.theme = themes[idx]
+    viewer.theme = themes[idx]
 
 
 @register_viewer_action(trans._("Reset view to original state."))
@@ -75,6 +76,11 @@ def reset_view(viewer: Viewer):
     viewer.reset_view()
 
 
+@register_viewer_action(trans._("Delete selected layers."))
+def delete_selected_layers(viewer: Viewer):
+    viewer.layers.remove_selected()
+
+
 @register_viewer_action(trans._("Increment dimensions slider to the left."))
 def increment_dims_left(viewer: Viewer):
     viewer.dims._increment_dims_left()
@@ -95,20 +101,24 @@ def focus_axes_down(viewer: Viewer):
     viewer.dims._focus_down()
 
 
+# Use non-breaking spaces and non-breaking hyphen for Preferences table
 @register_viewer_action(
-    trans._("Change order of the visible axes, e.g. [0, 1, 2] -> [2, 0, 1]."),
+    trans._(
+        "Change order of the visible axes, e.g.\u00A0[0,\u00A01,\u00A02]\u00A0\u2011>\u00A0[2,\u00A00,\u00A01]."
+    ),
 )
 def roll_axes(viewer: Viewer):
     viewer.dims._roll()
 
 
+# Use non-breaking spaces and non-breaking hyphen for Preferences table
 @register_viewer_action(
     trans._(
-        "Transpose order of the last two visible axes, e.g. [0, 1] -> [1, 0]."
+        "Transpose order of the last two visible axes, e.g.\u00A0[0,\u00A01]\u00A0\u2011>\u00A0[1,\u00A00]."
     ),
 )
 def transpose_axes(viewer: Viewer):
-    viewer.dims._transpose()
+    viewer.dims.transpose()
 
 
 @register_viewer_action(trans._("Toggle grid mode."))
@@ -128,3 +138,17 @@ def toggle_selected_visibility(viewer: Viewer):
 )
 def toggle_console_visibility(viewer: Viewer):
     viewer.window._qt_viewer.toggle_console_visibility()
+
+
+@register_viewer_action(trans._("Press and hold for pan/zoom mode"))
+def hold_for_pan_zoom(viewer: ViewerModel):
+    selected_layer = viewer.layers.selection.active
+    if selected_layer is None:
+        yield
+        return
+    previous_mode = selected_layer.mode
+    if previous_mode != selected_layer._modeclass.PAN_ZOOM:
+        selected_layer.mode = selected_layer._modeclass.PAN_ZOOM
+        yield
+
+        selected_layer.mode = previous_mode
