@@ -13,34 +13,41 @@ import magicgui as mgui
 import numpy as np
 from npe2 import plugin_manager as pm
 
-from ...utils._dask_utils import configure_dask
-from ...utils._magicgui import (
-    add_layer_to_viewer,
-    add_layers_to_viewer,
-    get_layers,
+from napari.layers.base._base_constants import Blending, Mode
+from napari.layers.base._base_mouse_bindings import (
+    highlight_box_handles,
+    transform_with_box,
 )
-from ...utils.events import EmitterGroup, Event
-from ...utils.events.event import WarningEmitter
-from ...utils.geometry import (
-    find_front_back_face,
-    intersect_line_with_axis_aligned_bounding_box_3d,
+from napari.layers.utils._slice_input import _SliceInput
+from napari.layers.utils.interactivity_utils import (
+    drag_data_to_projected_distance,
 )
-from ...utils.key_bindings import KeymapProvider
-from ...utils.mouse_bindings import MousemapProvider
-from ...utils.naming import magic_name
-from ...utils.status_messages import generate_layer_coords_status
-from ...utils.transforms import Affine, CompositeAffine, TransformChain
-from ...utils.translations import trans
-from ..utils.interactivity_utils import drag_data_to_projected_distance
-from ..utils.layer_utils import (
+from napari.layers.utils.layer_utils import (
     coerce_affine,
     compute_multiscale_level_and_corners,
     convert_to_uint8,
     dims_displayed_world_to_layer,
     get_extent_world,
 )
-from ..utils.plane import ClippingPlane, ClippingPlaneList
-from ._base_constants import Blending
+from napari.layers.utils.plane import ClippingPlane, ClippingPlaneList
+from napari.utils._dask_utils import configure_dask
+from napari.utils._magicgui import (
+    add_layer_to_viewer,
+    add_layers_to_viewer,
+    get_layers,
+)
+from napari.utils.events import EmitterGroup, Event, EventedDict
+from napari.utils.events.event import WarningEmitter
+from napari.utils.geometry import (
+    find_front_back_face,
+    intersect_line_with_axis_aligned_bounding_box_3d,
+)
+from napari.utils.key_bindings import KeymapProvider
+from napari.utils.mouse_bindings import MousemapProvider
+from napari.utils.naming import magic_name
+from napari.utils.status_messages import generate_layer_coords_status
+from napari.utils.transforms import Affine, CompositeAffine, TransformChain
+from napari.utils.translations import trans
 
 Extent = namedtuple('Extent', 'data world step')
 
@@ -64,7 +71,7 @@ def no_op(layer: Layer, event: Event) -> None:
     None
 
     """
-    return None
+    return
 
 
 @mgui.register_type(choices=get_layers, return_callback=add_layer_to_viewer)
@@ -188,6 +195,12 @@ class Layer(KeymapProvider, MousemapProvider, ABC):
         Displayed in status bar bottom right.
     interactive : bool
         Determine if canvas pan/zoom interactivity is enabled.
+        This attribute is deprecated since 0.5.0 and should not be used.
+        Use the mouse_pan and mouse_zoom attributes instead.
+    mouse_pan : bool
+        Determine if canvas interactive panning is enabled with the mouse.
+    mouse_zoom : bool
+        Determine if canvas interactive zooming is enabled with the mouse.
     cursor : str
         String identifying which cursor displayed over canvas.
     cursor_size : int | None
@@ -211,6 +224,22 @@ class Layer(KeymapProvider, MousemapProvider, ABC):
     * `_basename()`: base/default name of the layer
     """
 
+    _modeclass = Mode
+
+    _drag_modes = {
+        Mode.PAN_ZOOM: no_op,
+        Mode.TRANSFORM: transform_with_box,
+    }
+
+    _move_modes = {
+        Mode.PAN_ZOOM: no_op,
+        Mode.TRANSFORM: highlight_box_handles,
+    }
+    _cursor_modes = {
+        Mode.PAN_ZOOM: 'standard',
+        Mode.TRANSFORM: 'standard',
+    }
+
     def __init__(
         self,
         data,
@@ -229,7 +258,8 @@ class Layer(KeymapProvider, MousemapProvider, ABC):
         multiscale=False,
         cache=True,  # this should move to future "data source" object.
         experimental_clipping_planes=None,
-    ):
+        mode='pan_zoom',
+    ) -> None:
         super().__init__()
 
         if name is None and data is not None:
@@ -246,7 +276,7 @@ class Layer(KeymapProvider, MousemapProvider, ABC):
             )
 
         # Needs to be imported here to avoid circular import in _source
-        from .._source import current_source
+        from napari.layers._source import current_source
 
         self._source = current_source()
         self.dask_optimized_slicing = configure_dask(data, cache)
@@ -259,15 +289,21 @@ class Layer(KeymapProvider, MousemapProvider, ABC):
         self._help = ''
         self._cursor = 'standard'
         self._cursor_size = 1
-        self._interactive = True
+        self._mouse_pan = True
+        self._mouse_zoom = True
         self._value = None
         self.scale_factor = 1
         self.multiscale = multiscale
         self._experimental_clipping_planes = ClippingPlaneList()
+        self._mode = self._modeclass('pan_zoom')
 
         self._ndim = ndim
-        self._ndisplay = 2
-        self._dims_order = list(range(ndim))
+
+        self._slice_input = _SliceInput(
+            ndisplay=2,
+            point=(0,) * ndim,
+            order=tuple(range(ndim)),
+        )
 
         # Create a transform chain consisting of four transforms:
         # 1. `tile2data`: An initial transform only needed to display tiles
@@ -304,7 +340,6 @@ class Layer(KeymapProvider, MousemapProvider, ABC):
             ]
         )
 
-        self._dims_point = [0] * ndim
         self.corner_pixels = np.zeros((2, ndim), dtype=int)
         self._editable = True
         self._array_like = False
@@ -315,6 +350,15 @@ class Layer(KeymapProvider, MousemapProvider, ABC):
         self._name = ''
         self.experimental_clipping_planes = experimental_clipping_planes
 
+        # circular import
+        from napari.components.overlays.bounding_box import BoundingBoxOverlay
+        from napari.components.overlays.interaction_box import (
+            SelectionBoxOverlay,
+            TransformBoxOverlay,
+        )
+
+        self._overlays = EventedDict()
+
         self.events = EmitterGroup(
             source=self,
             refresh=Event,
@@ -332,28 +376,50 @@ class Layer(KeymapProvider, MousemapProvider, ABC):
             thumbnail=Event,
             status=Event,
             help=Event,
-            interactive=Event,
+            interactive=WarningEmitter(
+                trans._(
+                    "layer.events.interactive is deprecated since 0.4.18 and will be removed in 0.6.0. Please use layer.events.mouse_pan and layer.events.mouse_zoom",
+                    deferred=True,
+                ),
+                type_name='interactive',
+            ),
+            mouse_pan=Event,
+            mouse_zoom=Event,
             cursor=Event,
             cursor_size=Event,
             editable=Event,
             loaded=Event,
-            _ndisplay=Event,
+            extent=Event,
+            _overlays=Event,
             select=WarningEmitter(
                 trans._(
                     "'layer.events.select' is deprecated and will be removed in napari v0.4.9, use 'viewer.layers.selection.events.changed' instead, and inspect the 'added' attribute on the event.",
                     deferred=True,
                 ),
-                type='select',
+                type_name='select',
             ),
             deselect=WarningEmitter(
                 trans._(
                     "'layer.events.deselect' is deprecated and will be removed in napari v0.4.9, use 'viewer.layers.selection.events.changed' instead, and inspect the 'removed' attribute on the event.",
                     deferred=True,
                 ),
-                type='deselect',
+                type_name='deselect',
             ),
+            mode=Event,
         )
         self.name = name
+        self.mode = mode
+        self._overlays.update(
+            {
+                'transform_box': TransformBoxOverlay(),
+                'selection_box': SelectionBoxOverlay(),
+                'bounding_box': BoundingBoxOverlay(),
+            }
+        )
+
+        # TODO: we try to avoid inner event connection, but this might be the only way
+        #       until we figure out nested evented objects
+        self._overlays.events.connect(self.events._overlays)
 
     def __str__(self):
         """Return self.name."""
@@ -361,39 +427,35 @@ class Layer(KeymapProvider, MousemapProvider, ABC):
 
     def __repr__(self):
         cls = type(self)
-        return f"<{cls.__name__} layer {repr(self.name)} at {hex(id(self))}>"
+        return f"<{cls.__name__} layer {self.name!r} at {hex(id(self))}>"
 
-    def _mode_setter_helper(self, mode, Modeclass):
+    def _mode_setter_helper(self, mode):
         """
         Helper to manage callbacks in multiple layers
 
         Parameters
         ----------
-        mode : Modeclass | str
+        mode : type(self._modeclass) | str
             New mode for the current layer.
-        Modeclass : Enum
-            Enum for the current class representing the modes it can takes,
-            this is usually specific on each subclass.
 
         Returns
         -------
-        tuple (new Mode, mode changed)
+        bool : whether mode changed
 
         """
-        mode = Modeclass(mode)
+        mode = self._modeclass(mode)
         assert mode is not None
         if not self.editable:
-            mode = Modeclass.PAN_ZOOM
+            mode = self._modeclass.PAN_ZOOM
         if mode == self._mode:
-            return mode, False
-        if mode.value not in Modeclass.keys():
+            return mode
+
+        if mode.value not in self._modeclass.keys():
             raise ValueError(
                 trans._(
                     "Mode not recognized: {mode}", deferred=True, mode=mode
                 )
             )
-        old_mode = self._mode
-        self._mode = mode
 
         for callback_list, mode_dict in [
             (self.mouse_drag_callbacks, self._drag_modes),
@@ -405,13 +467,44 @@ class Layer(KeymapProvider, MousemapProvider, ABC):
                 ),
             ),
         ]:
-            if mode_dict[old_mode] in callback_list:
-                callback_list.remove(mode_dict[old_mode])
+            if mode_dict[self._mode] in callback_list:
+                callback_list.remove(mode_dict[self._mode])
             callback_list.append(mode_dict[mode])
         self.cursor = self._cursor_modes[mode]
 
-        self.interactive = mode == Modeclass.PAN_ZOOM
-        return mode, True
+        self.mouse_pan = mode == self._modeclass.PAN_ZOOM
+        self._overlays['transform_box'].visible = (
+            mode == self._modeclass.TRANSFORM
+        )
+
+        if mode == self._modeclass.TRANSFORM:
+            self.help = trans._(
+                'hold <space> to pan/zoom, hold <shift> to preserve aspect ratio and rotate in 45° increments'
+            )
+        elif mode == self._modeclass.PAN_ZOOM:
+            self.help = ''
+
+        return mode
+
+    @property
+    def mode(self) -> str:
+        """str: Interactive mode
+
+        Interactive mode. The normal, default mode is PAN_ZOOM, which
+        allows for normal interactivity with the canvas.
+
+        TRANSFORM allows for manipulation of the layer transform.
+        """
+        return str(self._mode)
+
+    @mode.setter
+    def mode(self, mode):
+        mode = self._mode_setter_helper(mode)
+        if mode == self._mode:
+            return
+        self._mode = mode
+
+        self.events.mode(mode=str(mode))
 
     @classmethod
     def _basename(cls):
@@ -511,30 +604,36 @@ class Layer(KeymapProvider, MousemapProvider, ABC):
         self.events.blending()
 
     @property
-    def visible(self):
+    def visible(self) -> bool:
         """bool: Whether the visual is currently being displayed."""
         return self._visible
 
     @visible.setter
-    def visible(self, visibility):
-        self._visible = visibility
+    def visible(self, visible: bool):
+        self._visible = visible
         self.refresh()
         self.events.visible()
-        self.editable = self._set_editable() if self.visible else False
 
     @property
-    def editable(self):
+    def editable(self) -> bool:
         """bool: Whether the current layer data is editable from the viewer."""
         return self._editable
 
     @editable.setter
-    def editable(self, editable):
+    def editable(self, editable: bool):
         if self._editable == editable:
             return
         self._editable = editable
-        self._set_editable(editable=editable)
+        self._on_editable_changed()
         self.events.editable()
 
+    def _reset_editable(self) -> None:
+        """Reset this layer's editable state based on layer properties."""
+        self.editable = True
+
+    def _on_editable_changed(self) -> None:
+        """Executes side-effects on this layer related to changes of the editable state."""
+
     @property
     def scale(self):
         """list: Anisotropy factors to scale data into world coordinates."""
@@ -545,7 +644,7 @@ class Layer(KeymapProvider, MousemapProvider, ABC):
         if scale is None:
             scale = [1] * self.ndim
         self._transforms['data2physical'].scale = np.array(scale)
-        self._update_dims()
+        self._clear_extent()
         self.events.scale()
 
     @property
@@ -556,7 +655,7 @@ class Layer(KeymapProvider, MousemapProvider, ABC):
     @translate.setter
     def translate(self, translate):
         self._transforms['data2physical'].translate = np.array(translate)
-        self._update_dims()
+        self._clear_extent()
         self.events.translate()
 
     @property
@@ -567,7 +666,7 @@ class Layer(KeymapProvider, MousemapProvider, ABC):
     @rotate.setter
     def rotate(self, rotate):
         self._transforms['data2physical'].rotate = rotate
-        self._update_dims()
+        self._clear_extent()
         self.events.rotate()
 
     @property
@@ -578,7 +677,7 @@ class Layer(KeymapProvider, MousemapProvider, ABC):
     @shear.setter
     def shear(self, shear):
         self._transforms['data2physical'].shear = shear
-        self._update_dims()
+        self._clear_extent()
         self.events.shear()
 
     @property
@@ -594,7 +693,7 @@ class Layer(KeymapProvider, MousemapProvider, ABC):
         self._transforms[2] = coerce_affine(
             affine, ndim=self.ndim, name='physical2world'
         )
-        self._update_dims()
+        self._clear_extent()
         self.events.affine()
 
     @property
@@ -642,78 +741,34 @@ class Layer(KeymapProvider, MousemapProvider, ABC):
             assert self._moving_coordinates is not None
         self._private_is_moving = value
 
-    @property
-    def _dims_displayed(self):
-        """To be removed displayed dimensions."""
-        # Ultimately we aim to remove all slicing information from the layer
-        # itself so that layers can be sliced in different ways for multiple
-        # canvas. See https://github.com/napari/napari/pull/1919#issuecomment-738585093
-        # for additional discussion.
-        return self._dims_order[-self._ndisplay :]
-
-    @property
-    def _dims_not_displayed(self):
-        """To be removed not displayed dimensions."""
-        # Ultimately we aim to remove all slicing information from the layer
-        # itself so that layers can be sliced in different ways for multiple
-        # canvas. See https://github.com/napari/napari/pull/1919#issuecomment-738585093
-        # for additional discussion.
-        return self._dims_order[: -self._ndisplay]
-
-    @property
-    def _dims_displayed_order(self):
-        """To be removed order of displayed dimensions."""
-        # Ultimately we aim to remove all slicing information from the layer
-        # itself so that layers can be sliced in different ways for multiple
-        # canvas. See https://github.com/napari/napari/pull/1919#issuecomment-738585093
-        # for additional discussion.
-        displayed = self._dims_displayed
-        # equivalent to: order = np.argsort(displayed)
-        order = sorted(range(len(displayed)), key=lambda x: displayed[x])
-        return tuple(order)
-
-    def _update_dims(self, event=None):
-        """Update the dims model and clear the extent cache.
-
-        This function needs to be called whenever data or transform information
-        changes, and should be called before events get emitted.
-        """
-        from ...components.dims import reorder_after_dim_reduction
-
+    def _update_dims(self):
+        """Update the dimensionality of transforms and slices when data changes."""
         ndim = self._get_ndim()
 
         old_ndim = self._ndim
         if old_ndim > ndim:
             keep_axes = range(old_ndim - ndim, old_ndim)
             self._transforms = self._transforms.set_slice(keep_axes)
-            self._dims_point = self._dims_point[-ndim:]
-            self._dims_order = list(
-                reorder_after_dim_reduction(self._dims_order[-ndim:])
-            )
         elif old_ndim < ndim:
             new_axes = range(ndim - old_ndim)
             self._transforms = self._transforms.expand_dims(new_axes)
-            self._dims_point = [0] * (ndim - old_ndim) + self._dims_point
-            self._dims_order = list(range(ndim - old_ndim)) + [
-                o + ndim - old_ndim for o in self._dims_order
-            ]
+
+        self._slice_input = self._slice_input.with_ndim(ndim)
 
         self._ndim = ndim
-        if 'extent' in self.__dict__:
-            del self.extent
 
-        self.refresh()  # This call is need for invalidate cache of extent in LayerList. If you remove it pleas ad another workaround.
+        self._clear_extent()
 
     @property
     @abstractmethod
     def data(self):
         # user writes own docstring
-        raise NotImplementedError()
+        raise NotImplementedError
 
     @data.setter
     @abstractmethod
     def data(self, data):
-        raise NotImplementedError()
+        raise NotImplementedError
 
     @property
     @abstractmethod
@@ -724,7 +779,7 @@ class Layer(KeymapProvider, MousemapProvider, ABC):
         -------
         extent_data : array, shape (2, D)
         """
-        raise NotImplementedError()
+        raise NotImplementedError
 
     @property
     def _extent_world(self) -> np.ndarray:
@@ -753,61 +808,32 @@ class Layer(KeymapProvider, MousemapProvider, ABC):
             step=abs(data_to_world.scale),
         )
 
+    def _clear_extent(self):
+        """Clears the cached extent.
+
+        This should be called whenever this data or transform information
+        changes, and should be called before any related events get emitted
+        so that they use the updated extent values.
+        """
+        if 'extent' in self.__dict__:
+            del self.extent
+        self.events.extent()
+        self.refresh()
+
     @property
     def _slice_indices(self):
         """(D, ) array: Slice indices in data coordinates."""
-
-        if len(self._dims_not_displayed) == 0:
-            # all dims are displayed dimensions
+        if len(self._slice_input.not_displayed) == 0:
+            # All dims are displayed dimensions
             return (slice(None),) * self.ndim
-
-        if self.ndim > self._ndisplay:
-            inv_transform = self._data_to_world.inverse
-            # Subspace spanned by non displayed dimensions
-            non_displayed_subspace = np.zeros(self.ndim)
-            for d in self._dims_not_displayed:
-                non_displayed_subspace[d] = 1
-            # Map subspace through inverse transform, ignoring translation
-            _inv_transform = Affine(
-                ndim=self.ndim,
-                linear_matrix=inv_transform.linear_matrix,
-                translate=None,
-            )
-            mapped_nd_subspace = _inv_transform(non_displayed_subspace)
-            # Look at displayed subspace
-            displayed_mapped_subspace = (
-                mapped_nd_subspace[d] for d in self._dims_displayed
-            )
-            # Check that displayed subspace is null
-            if any(abs(v) > 1e-8 for v in displayed_mapped_subspace):
-                warnings.warn(
-                    trans._(
-                        'Non-orthogonal slicing is being requested, but is not fully supported. Data is displayed without applying an out-of-slice rotation or shear component.',
-                        deferred=True,
-                    ),
-                    category=UserWarning,
-                )
-
-        slice_inv_transform = inv_transform.set_slice(self._dims_not_displayed)
-        world_pts = [self._dims_point[ax] for ax in self._dims_not_displayed]
-        data_pts = slice_inv_transform(world_pts)
-        if getattr(self, "_round_index", True):
-            # A round is taken to convert these values to slicing integers
-            data_pts = np.round(data_pts).astype(int)
-
-        indices = [slice(None)] * self.ndim
-        for i, ax in enumerate(self._dims_not_displayed):
-            indices[ax] = data_pts[i]
-
-        return tuple(indices)
+        return self._slice_input.data_indices(
+            self._data_to_world.inverse,
+            getattr(self, '_round_index', True),
+        )
 
     @abstractmethod
     def _get_ndim(self):
-        raise NotImplementedError()
-
-    def _set_editable(self, editable=None):
-        if editable is None:
-            self.editable = True
+        raise NotImplementedError
 
     def _get_base_state(self):
         """Get dictionary of attributes on base layer.
@@ -836,7 +862,7 @@ class Layer(KeymapProvider, MousemapProvider, ABC):
 
     @abstractmethod
     def _get_state(self):
-        raise NotImplementedError()
+        raise NotImplementedError
 
     @property
     def _type_string(self):
@@ -857,9 +883,7 @@ class Layer(KeymapProvider, MousemapProvider, ABC):
         if 0 in thumbnail.shape:
             thumbnail = np.zeros(self._thumbnail_shape, dtype=np.uint8)
         if thumbnail.dtype != np.uint8:
-            with warnings.catch_warnings():
-                warnings.simplefilter("ignore")
-                thumbnail = convert_to_uint8(thumbnail)
+            thumbnail = convert_to_uint8(thumbnail)
 
         padding_needed = np.subtract(self._thumbnail_shape, thumbnail.shape)
         pad_amounts = [(p // 2, (p + 1) // 2) for p in padding_needed]
@@ -887,23 +911,65 @@ class Layer(KeymapProvider, MousemapProvider, ABC):
         return self._help
 
     @help.setter
-    def help(self, help):
-        if help == self.help:
+    def help(self, help_text):
+        if help_text == self.help:
             return
-        self._help = help
-        self.events.help(help=help)
+        self._help = help_text
+        self.events.help(help=help_text)
 
     @property
-    def interactive(self):
-        """bool: Determine if canvas pan/zoom interactivity is enabled."""
-        return self._interactive
+    def interactive(self) -> bool:
+        warnings.warn(
+            trans._(
+                "Layer.interactive is deprecated since napari 0.4.18 and will be removed in 0.6.0. Please use Layer.mouse_pan and Layer.mouse_zoom instead"
+            ),
+            FutureWarning,
+            stacklevel=2,
+        )
+        return self.mouse_pan or self.mouse_zoom
 
     @interactive.setter
-    def interactive(self, interactive):
-        if interactive == self._interactive:
+    def interactive(self, interactive: bool):
+        warnings.warn(
+            trans._(
+                "Layer.interactive is deprecated since napari 0.4.18 and will be removed in 0.6.0. Please use Layer.mouse_pan and Layer.mouse_zoom instead"
+            ),
+            FutureWarning,
+            stacklevel=2,
+        )
+        with self.events.interactive.blocker():
+            self.mouse_pan = interactive
+        self.mouse_zoom = interactive
+
+    @property
+    def mouse_pan(self) -> bool:
+        """bool: Determine if canvas interactive panning is enabled with the mouse."""
+        return self._mouse_pan
+
+    @mouse_pan.setter
+    def mouse_pan(self, mouse_pan: bool):
+        if mouse_pan == self._mouse_pan:
             return
-        self._interactive = interactive
-        self.events.interactive(interactive=interactive)
+        self._mouse_pan = mouse_pan
+        self.events.mouse_pan(mouse_pan=mouse_pan)
+        self.events.interactive(
+            interactive=self.mouse_pan or self.mouse_zoom
+        )  # Deprecated since 0.5.0
+
+    @property
+    def mouse_zoom(self) -> bool:
+        """bool: Determine if canvas interactive zooming is enabled with the mouse."""
+        return self._mouse_zoom
+
+    @mouse_zoom.setter
+    def mouse_zoom(self, mouse_zoom: bool):
+        if mouse_zoom == self._mouse_zoom:
+            return
+        self._mouse_zoom = mouse_zoom
+        self.events.mouse_zoom(mouse_zoom=mouse_zoom)
+        self.events.interactive(
+            interactive=self.mouse_pan or self.mouse_zoom
+        )  # Deprecated since 0.5.0
 
     @property
     def cursor(self):
@@ -954,13 +1020,17 @@ class Layer(KeymapProvider, MousemapProvider, ABC):
             plane.update(new_plane)
             self._experimental_clipping_planes.append(plane)
 
+    @property
+    def bounding_box(self):
+        return self._overlays['bounding_box']
+
     def set_view_slice(self):
         with self.dask_optimized_slicing():
             self._set_view_slice()
 
     @abstractmethod
     def _set_view_slice(self):
-        raise NotImplementedError()
+        raise NotImplementedError
 
     def _slice_dims(self, point=None, ndisplay=2, order=None):
         """Slice data with values from a global dims model.
@@ -977,52 +1047,38 @@ class Layer(KeymapProvider, MousemapProvider, ABC):
             Order of dimensions, where last `ndisplay` will be
             rendered in canvas.
         """
-        if point is None:
-            ndim = self.ndim
-        else:
-            ndim = len(point)
+        point = (0,) * self.ndim if point is None else tuple(point)
+
+        ndim = len(point)
 
         if order is None:
-            order = list(range(ndim))
+            order = tuple(range(ndim))
 
-        # adjust the order of the global dims based on the number of
-        # dimensions that a layer has - for example a global order of
-        # [2, 1, 0, 3] -> [0, 1] for a layer that only has two dimensions
-        # or -> [1, 0, 2] for a layer with three as that corresponds to
-        # the relative order of the last two and three dimensions
-        # respectively
-        order = self._world_to_data_dims_displayed(order, ndim_world=ndim)
+        # Correspondence between dimensions across all layers and
+        # dimensions of this layer.
+        point = point[-self.ndim :]
+        order = tuple(
+            self._world_to_layer_dims(world_dims=order, ndim_world=ndim)
+        )
 
-        if point is None:
-            point = [0] * ndim
-            nd = min(self.ndim, ndisplay)
-            for i in order[-nd:]:
-                point[i] = slice(None)
-        else:
-            point = list(point)
+        slice_input = _SliceInput(
+            ndisplay=ndisplay,
+            point=point,
+            order=order,
+        )
 
-        # If no slide data has changed, then do nothing
-        offset = ndim - self.ndim
-        if (
-            np.all(order == self._dims_order)
-            and ndisplay == self._ndisplay
-            and np.all(point[offset:] == self._dims_point)
-        ):
+        if self._slice_input == slice_input:
             return
 
-        self._dims_order = order
-        if self._ndisplay != ndisplay:
-            self._ndisplay = ndisplay
-            self.events._ndisplay()
+        self._slice_input = slice_input
 
         # Update the point values
-        self._dims_point = point[offset:]
-        self._update_dims()
-        self._set_editable()
+        self.refresh()
+        self._reset_editable()
 
     @abstractmethod
     def _update_thumbnail(self):
-        raise NotImplementedError()
+        raise NotImplementedError
 
     @abstractmethod
     def _get_value(self, position):
@@ -1038,7 +1094,7 @@ class Layer(KeymapProvider, MousemapProvider, ABC):
         value : tuple
             Value of the data.
         """
-        raise NotImplementedError()
+        raise NotImplementedError
 
     def get_value(
         self,
@@ -1120,7 +1176,9 @@ class Layer(KeymapProvider, MousemapProvider, ABC):
         start_point: np.ndarray,
         end_point: np.ndarray,
         dims_displayed: List[int],
-    ) -> Union[float, int]:
+    ) -> Union[
+        float, int, None, Tuple[Union[float, int, None], Optional[int]]
+    ]:
         """Get the layer data value along a ray
 
         Parameters
@@ -1136,9 +1194,7 @@ class Layer(KeymapProvider, MousemapProvider, ABC):
         -------
         value
             The data value along the supplied ray.
-
         """
-        return None
 
     def projected_distance_from_mouse_drag(
         self,
@@ -1201,13 +1257,12 @@ class Layer(KeymapProvider, MousemapProvider, ABC):
         force : bool
             Bool that forces a redraw to occur when `True`.
         """
-        pass
 
     def refresh(self, event=None):
         """Refresh all layer data based on current view slice."""
         if self.visible:
             self.set_view_slice()
-            self.events.set_data()  # refresh is called in _update_dims which means that extent cache is invalidated. Then, base on this event extent cache in layerlist is invalidated.
+            self.events.set_data()
             self._update_thumbnail()
             self._set_highlight(force=True)
 
@@ -1331,38 +1386,41 @@ class Layer(KeymapProvider, MousemapProvider, ABC):
         vector_data_ndisplay /= np.linalg.norm(vector_data_ndisplay)
         return vector_data_ndisplay
 
-    def _world_to_data_dims_displayed(
-        self, dims_displayed: List[int], ndim_world: int
+    def _world_to_layer_dims(
+        self, *, world_dims: List[int], ndim_world: int
     ) -> List[int]:
-        """Convert indices of displayed dims from world to data coordinates.
+        """Map world dimensions to layer dimensions while maintaining order.
 
-        This accounts for differences in dimensionality between the world
-        and the data coordinates. For example a world dims order of
-        [2, 1, 0, 3] would be [0, 1] for a layer that only has two dimensions
-        or [1, 0, 2] for a layer with three as that corresponds to the
-        relative order of the last two and three dimensions respectively
+        This is used to map dimensions from the full world space defined by ``Dims``
+        to the subspace that a layer inhabits, so that those can be used to index the
+        layer's data and associated coordinates.
+
+        For example a world ``Dims.order`` of [2, 1, 0, 3] would map to [0, 1] for a
+        layer with two dimensions and [1, 0, 2] for a layer with three dimensions
+        as those correspond to the relative order of the last two and three world dimensions
+        respectively.
 
         Parameters
         ----------
-        dims_displayed : List[int]
-            The world displayed dimensions.
+        world_dims : List[int]
+            The world dimensions.
         ndim_world : int
             The number of dimensions in the world coordinate system.
 
         Returns
         -------
-        dims_displayed_data : List[int]
-            The displayed dimensions in data coordinates.
+        List[int]
+            The corresponding layer dimensions with the same ordering as the given world dimensions.
         """
         offset = ndim_world - self.ndim
-        order = np.array(dims_displayed)
+        order = np.array(world_dims)
         if offset <= 0:
             return list(range(-offset)) + list(order - offset)
-        else:
-            return list(order[order >= offset] - offset)
+
+        return list(order[order >= offset] - offset)
 
     def _display_bounding_box(self, dims_displayed: np.ndarray):
-        """An axis aligned (self._ndisplay, 2) bounding box around the data"""
+        """An axis aligned (ndisplay, 2) bounding box around the data"""
         return self._extent_data[:, dims_displayed].T
 
     def click_plane_from_click_data(
@@ -1503,7 +1561,6 @@ class Layer(KeymapProvider, MousemapProvider, ABC):
                 position, dims_displayed
             )
         else:
-
             # adjust for any offset between viewer and data coordinates
             position = self._get_offset_data_position(position)
 
@@ -1538,20 +1595,6 @@ class Layer(KeymapProvider, MousemapProvider, ABC):
 
         return start_point, end_point
 
-    @property
-    def _displayed_axes(self):
-        # assignment upfront to avoid repeated computation of properties
-        _dims_displayed = self._dims_displayed
-        _dims_displayed_order = self._dims_displayed_order
-        displayed_axes = [_dims_displayed[i] for i in _dims_displayed_order]
-        return displayed_axes
-
-    @property
-    def _corner_pixels_displayed(self):
-        displayed_axes = self._displayed_axes
-        corner_pixels_displayed = self.corner_pixels[:, displayed_axes]
-        return corner_pixels_displayed
-
     def _update_draw(
         self, scale_factor, corner_pixels_displayed, shape_threshold
     ):
@@ -1572,11 +1615,8 @@ class Layer(KeymapProvider, MousemapProvider, ABC):
         """
         self.scale_factor = scale_factor
 
-        displayed_axes = self._displayed_axes
-        # must adjust displayed_axes according to _dims_order
-        displayed_axes = np.asarray(
-            [self._dims_order[d] for d in displayed_axes]
-        )
+        displayed_axes = self._slice_input.displayed
+
         # we need to compute all four corners to compute a complete,
         # data-aligned bounding box, because top-left/bottom-right may not
         # remain top-left and bottom-right after transformations.
@@ -1597,7 +1637,7 @@ class Layer(KeymapProvider, MousemapProvider, ABC):
             [np.floor(data_bbox[0]), np.ceil(data_bbox[1])]
         ).astype(int)
 
-        if self._ndisplay == 2 and self.multiscale:
+        if self._slice_input.ndisplay == 2 and self.multiscale:
             level, scaled_corners = compute_multiscale_level_and_corners(
                 data_bbox_int,
                 shape_threshold,
@@ -1624,12 +1664,16 @@ class Layer(KeymapProvider, MousemapProvider, ABC):
 
         else:
             # The stored corner_pixels attribute must contain valid indices.
-            displayed_extent = self.extent.data[:, displayed_axes]
-            data_bbox_clipped = np.clip(
-                data_bbox_int, displayed_extent[0], displayed_extent[1]
-            )
             corners = np.zeros((2, self.ndim), dtype=int)
-            corners[:, displayed_axes] = data_bbox_clipped
+            # Some empty layers (e.g. Points) may have a data extent that only
+            # contains nans, in which case the integer valued corner pixels
+            # cannot be meaningfully set.
+            displayed_extent = self.extent.data[:, displayed_axes]
+            if not np.all(np.isnan(displayed_extent)):
+                data_bbox_clipped = np.clip(
+                    data_bbox_int, displayed_extent[0], displayed_extent[1]
+                )
+                corners[:, displayed_axes] = data_bbox_clipped
             self.corner_pixels = corners
 
     def _get_source_info(self):
@@ -1645,7 +1689,7 @@ class Layer(KeymapProvider, MousemapProvider, ABC):
                 components['plugin'] = self.source.reader_plugin
             return components
 
-        elif self.source.sample:
+        if self.source.sample:
             components['layer_base'] = self.name
             components['source_type'] = 'sample'
             try:
@@ -1656,20 +1700,18 @@ class Layer(KeymapProvider, MousemapProvider, ABC):
                 components['plugin'] = self.source.sample[0]
             return components
 
-        elif self.source.widget:
+        if self.source.widget:
             components['layer_base'] = self.name
             components['source_type'] = 'widget'
             components['plugin'] = self.source.widget._function.__name__
             return components
 
-        else:
-            components['layer_base'] = self.name
-            components['source_type'] = ''
-            components['plugin'] = ''
-            return components
+        components['layer_base'] = self.name
+        components['source_type'] = ''
+        components['plugin'] = ''
+        return components
 
     def get_source_str(self):
-
         source_info = self._get_source_info()
 
         return (
@@ -1722,7 +1764,7 @@ class Layer(KeymapProvider, MousemapProvider, ABC):
 
         source_info = self._get_source_info()
         source_info['coordinates'] = generate_layer_coords_status(
-            position, value
+            position[-self.ndim :], value
         )
         return source_info
 
@@ -1777,7 +1819,7 @@ class Layer(KeymapProvider, MousemapProvider, ABC):
         list of str
             File paths of any files that were written.
         """
-        from ...plugins.io import save_layers
+        from napari.plugins.io import save_layers
 
         return save_layers(path, [self], plugin=plugin)
 
@@ -1837,8 +1879,8 @@ class Layer(KeymapProvider, MousemapProvider, ABC):
         >>> Layer.create(*data)
 
         """
-        from ... import layers
-        from ..image._image_utils import guess_labels
+        from napari import layers
+        from napari.layers.image._image_utils import guess_labels
 
         layer_type = (layer_type or '').lower()
 
@@ -1860,9 +1902,9 @@ class Layer(KeymapProvider, MousemapProvider, ABC):
 
         try:
             return Cls(data, **(meta or {}))
-        except Exception as exc:
+        except Exception as exc:  # noqa: BLE001
             if 'unexpected keyword argument' not in str(exc):
-                raise exc
+                raise
 
             bad_key = str(exc).split('keyword argument ')[-1]
             raise TypeError(
