@@ -1,50 +1,40 @@
 from __future__ import annotations
 
 import napari
-
-from ...layers.utils.interactivity_utils import (
+from napari.layers.base._base_constants import Mode
+from napari.layers.image.image import Image
+from napari.layers.utils.interactivity_utils import (
     orient_plane_normal_around_cursor,
 )
-from ...layers.utils.layer_utils import register_layer_action
-from ...utils.translations import trans
-from ._image_constants import Mode
-from .image import Image
+from napari.layers.utils.layer_utils import register_layer_action
+from napari.utils.translations import trans
 
 
-def register_image_action(description: str):
-    return register_layer_action(Image, description=description)
+def register_image_action(description: str, repeatable: bool = False):
+    return register_layer_action(Image, description, repeatable)
 
 
-@Image.bind_key('z')
+@Image.bind_key('z', overwrite=True)
 @register_image_action(trans._('Orient plane normal along z-axis'))
 def orient_plane_normal_along_z(layer: Image):
     orient_plane_normal_around_cursor(layer, plane_normal=(1, 0, 0))
 
 
-@Image.bind_key('y')
+@Image.bind_key('y', overwrite=True)
 @register_image_action(trans._('orient plane normal along y-axis'))
 def orient_plane_normal_along_y(layer: Image):
     orient_plane_normal_around_cursor(layer, plane_normal=(0, 1, 0))
 
 
-@Image.bind_key('x')
+@Image.bind_key('x', overwrite=True)
 @register_image_action(trans._('orient plane normal along x-axis'))
 def orient_plane_normal_along_x(layer: Image):
     orient_plane_normal_around_cursor(layer, plane_normal=(0, 0, 1))
 
 
+@Image.bind_key('o', overwrite=True)
 @register_image_action(trans._('orient plane normal along view direction'))
 def orient_plane_normal_along_view_direction(layer: Image):
-    viewer = napari.viewer.current_viewer()
-    if viewer.dims.ndisplay != 3:
-        return
-    layer.plane.normal = layer._world_to_displayed_data_ray(
-        viewer.camera.view_direction, dims_displayed=[-3, -2, -1]
-    )
-
-
-@Image.bind_key('o')
-def synchronise_plane_normal_with_view_direction(layer: Image):
     viewer = napari.viewer.current_viewer()
     if viewer.dims.ndisplay != 3:
         return
@@ -65,7 +55,7 @@ def synchronise_plane_normal_with_view_direction(layer: Image):
     )
 
 
-@Image.bind_key('Space')
+@Image.bind_key('Space', overwrite=True)
 def hold_to_pan_zoom(layer):
     """Hold to pan and zoom in the viewer."""
     if layer._mode != Mode.PAN_ZOOM:
@@ -80,10 +70,16 @@ def hold_to_pan_zoom(layer):
 
 
 @register_image_action(trans._('Transform'))
-def activate_image_select_mode(layer):
+def activate_image_transform_mode(layer):
     layer.mode = Mode.TRANSFORM
 
 
 @register_image_action(trans._('Pan/zoom'))
-def activate_image_pan_zoom_mode(layer):
+def activate_image_pan_zoom_mode(layer: Image):
     layer.mode = Mode.PAN_ZOOM
+
+
+image_fun_to_mode = [
+    (activate_image_pan_zoom_mode, Mode.PAN_ZOOM),
+    (activate_image_transform_mode, Mode.TRANSFORM),
+]
