@@ -12,13 +12,15 @@ import zarr
 from numpy.core.numerictypes import issubdtype
 from numpy.testing import assert_array_almost_equal, assert_raises
 from skimage import data
+from vispy.color import Colormap as VispyColormap
 
 from napari._tests.utils import check_layer_world_data_extent
 from napari.components import ViewerModel
 from napari.layers import Labels
 from napari.layers.labels._labels_constants import LabelsRendering
+from napari.layers.labels._labels_utils import get_contours
 from napari.utils import Colormap
-from napari.utils.colormaps import low_discrepancy_image
+from napari.utils.colormaps import label_colormap, low_discrepancy_image
 
 
 def test_random_labels():
@@ -58,7 +60,7 @@ def test_3D_labels():
     assert layer.editable is True
 
     layer._slice_dims(ndisplay=3)
-    assert layer._ndisplay == 3
+    assert layer._slice_input.ndisplay == 3
     assert layer.editable is True
     assert layer.mode == 'pan_zoom'
 
@@ -136,23 +138,23 @@ def test_changing_modes():
     data = np.random.randint(20, size=(10, 15))
     layer = Labels(data)
     assert layer.mode == 'pan_zoom'
-    assert layer.interactive is True
+    assert layer.mouse_pan is True
 
     layer.mode = 'fill'
     assert layer.mode == 'fill'
-    assert layer.interactive is False
+    assert layer.mouse_pan is False
 
     layer.mode = 'paint'
     assert layer.mode == 'paint'
-    assert layer.interactive is False
+    assert layer.mouse_pan is False
 
     layer.mode = 'pick'
     assert layer.mode == 'pick'
-    assert layer.interactive is False
+    assert layer.mouse_pan is False
 
     layer.mode = 'pan_zoom'
     assert layer.mode == 'pan_zoom'
-    assert layer.interactive is True
+    assert layer.mouse_pan is True
 
     layer.mode = 'paint'
     assert layer.mode == 'paint'
@@ -288,31 +290,31 @@ def test_properties():
 
     current_label = layer.get_value((0, 0))
     layer_message = layer.get_status((0, 0))
-    assert layer_message.endswith(f'Class {current_label - 1}')
+    assert layer_message['coordinates'].endswith(f'Class {current_label - 1}')
 
     properties = {'class': ['Background']}
     layer = Labels(data, properties=properties)
     layer_message = layer.get_status((0, 0))
-    assert layer_message.endswith("[No Properties]")
+    assert layer_message['coordinates'].endswith("[No Properties]")
 
     properties = {'class': ['Background', 'Class 12'], 'index': [0, 12]}
     label_index = {0: 0, 12: 1}
     layer = Labels(data, properties=properties)
     layer_message = layer.get_status((0, 0))
     assert layer._label_index == label_index
-    assert layer_message.endswith('Class 12')
+    assert layer_message['coordinates'].endswith('Class 12')
 
     layer = Labels(data)
     layer.properties = properties
     layer_message = layer.get_status((0, 0))
     assert layer._label_index == label_index
-    assert layer_message.endswith('Class 12')
+    assert layer_message['coordinates'].endswith('Class 12')
 
     layer = Labels(data)
     layer.properties = pd.DataFrame(properties)
     layer_message = layer.get_status((0, 0))
     assert layer._label_index == label_index
-    assert layer_message.endswith('Class 12')
+    assert layer_message['coordinates'].endswith('Class 12')
 
 
 def test_default_properties_assignment():
@@ -349,19 +351,19 @@ def test_multiscale_properties():
 
     current_label = layer.get_value((0, 0))[1]
     layer_message = layer.get_status((0, 0))
-    assert layer_message.endswith(f'Class {current_label - 1}')
+    assert layer_message['coordinates'].endswith(f'Class {current_label - 1}')
 
     properties = {'class': ['Background']}
     layer = Labels(data, properties=properties)
     layer_message = layer.get_status((0, 0))
-    assert layer_message.endswith("[No Properties]")
+    assert layer_message['coordinates'].endswith("[No Properties]")
 
     properties = {'class': ['Background', 'Class 12'], 'index': [0, 12]}
     label_index = {0: 0, 12: 1}
     layer = Labels(data, properties=properties)
     layer_message = layer.get_status((0, 0))
     assert layer._label_index == label_index
-    assert layer_message.endswith('Class 12')
+    assert layer_message['coordinates'].endswith('Class 12')
 
 
 def test_colormap():
@@ -377,6 +379,17 @@ def test_colormap():
     assert layer.colormap.name == 'label_colormap'
 
 
+def test_label_colormap():
+    """Test a label colormap."""
+    colormap = label_colormap(num_colors=4)
+
+    # Make sure color 0 is transparent
+    assert not np.any(colormap.map([0.0]))
+
+    # Test that out-of-range values map to last value
+    assert np.all(colormap.map([1.0, 1.1, 2.0]) == colormap.colors[-1])
+
+
 def test_custom_color_dict():
     """Test custom color dict."""
     np.random.seed(0)
@@ -392,12 +405,63 @@ def test_custom_color_dict():
     assert (layer.get_color(4) == layer.get_color(16)).all()
     assert (layer.get_color(8) == layer.get_color(32)).all()
 
+    # Test to see if our label mapped control points map to those in the colormap
+    # with an extra half step.
+    local_controls = np.array(
+        sorted(np.unique([*layer._label_color_index.values(), 1.0]))
+    )
+    colormap_controls = np.array(layer._colormap.controls)
+    assert np.max(np.abs(local_controls - colormap_controls)) == pytest.approx(
+        0.5 / (len(colormap_controls) - 1)
+    )
+
     # test disable custom color dict
     # should not initialize as white since we are using random.seed
     layer.color_mode = 'auto'
     assert not (layer.get_color(1) == np.array([1.0, 1.0, 1.0, 1.0])).all()
 
 
+def test_large_custom_color_dict():
+    """Confirm that the napari & vispy colormaps behave the same."""
+
+    label_count = 897
+    colors = {
+        color: (0, (color / 256.0) / 256.0, (color % 256) / 256.0)
+        for color in range(label_count)
+    }
+    data, _ = np.meshgrid(range(label_count), range(5))
+    layer = Labels(data, color=colors)
+
+    # Get color list using layer interface & napari.utils.colormap.ColorMap
+    label_color = layer.get_color(list(range(label_count)))
+
+    # Get the color by converting to control points with the layer and passing
+    # that to a vispy.color.colormap.Colormap
+    vispy_colormap = VispyColormap(
+        colors=layer.colormap.colors,
+        controls=layer.colormap.controls,
+        interpolation='zero',
+    )
+    label_color_controls = [
+        layer._label_color_index[x] for x in range(label_count)
+    ]
+    vispy_colors = vispy_colormap.map(np.array(list(label_color_controls)))
+
+    assert (label_color == vispy_colors).all()
+
+
+def test_warning_too_many_colors():
+    label_count = 1500
+    colors = {
+        color: (0, (color / 256.0) / 256.0, (color % 256) / 256.0)
+        for color in range(label_count)
+    }
+    data, _ = np.meshgrid(range(label_count), range(5))
+    with pytest.warns(UserWarning):
+        # Expect a warning for 1500 colors > 1024 in LUT
+        Labels(data, color=colors)
+
+
 def test_add_colors():
     """Test adding new colors"""
     data = np.random.randint(20, size=(40, 40))
@@ -564,6 +628,9 @@ def test_contour(input_data, expected_data_view):
         layer._raw_to_displayed(input_data), layer._data_view
     )
 
+    with pytest.raises(ValueError, match='contour value must be >= 0'):
+        layer.contour = -1
+
 
 def test_contour_large_new_labels():
     """Check that new labels larger than the lookup table work in contour mode.
@@ -584,6 +651,37 @@ def test_contour_large_new_labels():
     viewer.dims.set_point(axis=0, value=4)
 
 
+def test_contour_local_updates():
+    """Checks if contours are rendered correctly with local updates"""
+    data = np.zeros((7, 7), dtype=np.int32)
+
+    layer = Labels(data)
+    layer.contour = 1
+    assert np.allclose(
+        layer._raw_to_displayed(layer._slice.image.raw),
+        np.zeros((7, 7), dtype=np.float32),
+    )
+
+    painting_mask = np.array(
+        [
+            [0, 0, 0, 0, 0, 0, 0],
+            [0, 0, 0, 0, 0, 0, 0],
+            [0, 0, 1, 1, 1, 0, 0],
+            [0, 0, 1, 1, 1, 0, 0],
+            [0, 0, 1, 1, 1, 0, 0],
+            [0, 0, 0, 0, 0, 0, 0],
+            [0, 0, 0, 0, 0, 0, 0],
+        ],
+        dtype=np.int32,
+    )
+
+    layer.data_setitem(np.nonzero(painting_mask), 1, refresh=True)
+
+    assert np.alltrue(
+        (layer._slice.image.view > 0) == get_contours(painting_mask, 1, 0)
+    )
+
+
 def test_selecting_label():
     """Test selecting label."""
     np.random.seed(0)
@@ -630,9 +728,7 @@ def test_show_selected_label():
 
     # color of all others is none color
     other_labels = np.unique(layer.data)[2:]
-    other_colors = np.array(
-        list(map(lambda x: layer.get_color(x), other_labels))
-    )
+    other_colors = np.array([layer.get_color(x) for x in other_labels])
     assert np.allclose(other_colors, none_color)
 
 
@@ -801,7 +897,7 @@ def test_message():
     data = np.random.randint(20, size=(10, 15))
     layer = Labels(data)
     msg = layer.get_status((0, 0))
-    assert type(msg) == str
+    assert type(msg) == dict
 
 
 def test_thumbnail():
@@ -898,7 +994,7 @@ def test_ndim_paint():
     assert not np.any(layer.data[0]) and not np.any(layer.data[2:])
 
     layer.n_edit_dimensions = 2  # 3x3 square
-    layer._dims_order = [1, 2, 0, 3]
+    layer._slice_dims(order=[1, 2, 0, 3])
     layer.paint((4, 5, 6, 7), 8)
     assert len(np.flatnonzero(layer.data == 8)) == 4  # 2D square is in corner
     np.testing.assert_array_equal(
@@ -935,8 +1031,7 @@ def test_switching_display_func_during_slicing():
     label_array = (5e6 * np.ones((2, 2, 2))).astype(np.uint64)
     label_array[0, :, :] = [[0, 1], [2, 3]]
     layer = Labels(label_array)
-    layer._dims_point = (1, 0, 0)
-    layer._set_view_slice()
+    layer._slice_dims(point=(1, 0, 0))
     assert layer._color_lookup_func == layer._lookup_with_low_discrepancy_image
     assert layer._all_vals.size < 1026
 
@@ -1008,7 +1103,7 @@ def test_fill_with_xarray():
 def test_paint_3d_negative_scale(scale):
     labels = np.zeros((3, 5, 11, 11), dtype=int)
     labels_layer = Labels(
-        labels, scale=(1,) + scale, translate=(-200, 100, 100)
+        labels, scale=(1, *scale), translate=(-200, 100, 100)
     )
     labels_layer.n_edit_dimensions = 3
     labels_layer.brush_size = 8
@@ -1120,7 +1215,7 @@ def test_get_value_ray_3d_rolled():
 
 def test_get_value_ray_3d_transposed():
     """Test using _get_value_ray to interrogate labels in 3D
-    with the dimensions trasposed.
+    with the dimensions transposed.
     """
     # make a mock mouse event
     mouse_event = MouseEvent(
@@ -1334,6 +1429,94 @@ def test_is_default_color():
     assert layer.color_mode == 'direct'
 
 
+def test_large_labels_direct_color():
+    """Make sure direct color works with large label ranges"""
+    data = np.array([[0, 1], [2**16, 2**20]], dtype=np.uint32)
+    colors = {1: 'white', 2**16: 'green', 2**20: 'magenta'}
+    layer = Labels(data)
+    layer.color = colors
+
+    assert layer.color_mode == 'direct'
+    np.testing.assert_allclose(layer.get_color(2**20), [1.0, 0.0, 1.0, 1.0])
+
+
+def test_invalidate_cache_when_change_color_mode():
+    """Checks if the cache is invalidated when color mode is changed."""
+    data = np.zeros((4, 10), dtype=np.int32)
+    data[1, :] = np.arange(0, 10)
+
+    layer = Labels(data)
+    layer.selected_label = 0
+    gt_auto = layer._raw_to_displayed(layer._slice.image.raw)
+    assert gt_auto.dtype == np.float32
+
+    layer.color_mode = 'direct'
+    layer._cached_labels = None
+    assert layer._raw_to_displayed(layer._slice.image.raw).dtype == np.float32
+
+    layer.color_mode = 'auto'
+    # If the cache is not invalidated, it returns colors for
+    # the direct color mode instead of the color for the auto mode
+    assert np.allclose(
+        layer._raw_to_displayed(layer._slice.image.raw), gt_auto
+    )
+
+
+def test_color_mapping_when_color_is_changed():
+    """Checks if the color mapping is computed correctly when the color palette is changed."""
+
+    data = np.zeros((4, 5), dtype=np.int32)
+    data[1, :] = np.arange(0, 5)
+    layer = Labels(data, color={1: 'green', 2: 'red', 3: 'white'})
+    gt_direct_3colors = layer._raw_to_displayed(layer._slice.image.raw)
+
+    layer = Labels(data, color={1: 'green', 2: 'red'})
+    assert layer._raw_to_displayed(layer._slice.image.raw).dtype == np.float32
+    layer.color = {1: 'green', 2: 'red', 3: 'white'}
+
+    assert np.allclose(
+        layer._raw_to_displayed(layer._slice.image.raw), gt_direct_3colors
+    )
+
+
+def test_color_mapping_with_show_selected_label():
+    """Checks if the color mapping is computed correctly when show_selected_label is activated."""
+
+    data = np.arange(5, dtype=np.int32)[:, np.newaxis].repeat(5, axis=1)
+    layer = Labels(data)
+    mapped_colors_all = layer._raw_to_displayed(layer._slice.image.raw).copy()
+
+    layer.selected_label = 1
+    layer.show_selected_label = True
+
+    for selected_label in range(1, 5):
+        layer.selected_label = selected_label
+        label_mask = data == selected_label
+        mapped_colors = layer._raw_to_displayed(layer._slice.image.raw)
+
+        assert np.allclose(
+            mapped_colors[label_mask], mapped_colors_all[label_mask]
+        )
+        assert np.allclose(mapped_colors[np.logical_not(label_mask)], 0)
+
+    layer.show_selected_label = False
+    assert np.allclose(
+        layer._raw_to_displayed(layer._slice.image.raw), mapped_colors_all
+    )
+
+
+def test_color_mapping_when_seed_is_changed():
+    """Checks if the color mapping is updated when the color palette seed is changed."""
+    np.random.seed(0)
+    layer = Labels(np.random.randint(50, size=(10, 10)))
+    mapped_colors1 = layer._raw_to_displayed(layer._slice.image.raw)
+
+    layer.new_colormap()
+    mapped_colors2 = layer._raw_to_displayed(layer._slice.image.raw)
+
+    assert not np.allclose(mapped_colors1, mapped_colors2)
+
+
 def test_negative_label():
     """Test negative label values are supported."""
     data = np.random.randint(low=-1, high=20, size=(10, 10))
@@ -1352,8 +1535,7 @@ def test_negative_label_slicing():
     data = np.array([[[0, 1], [-1, -1]], [[100, 100], [-1, -2]]])
     layer = Labels(data)
     assert tuple(layer.get_color(1)) != tuple(layer.get_color(-1))
-    layer._dims_point = (1, 0, 0)
-    layer._set_view_slice()
+    layer._slice_dims(point=(1, 0, 0))
     assert tuple(layer.get_color(-1)) != tuple(layer.get_color(100))
     assert tuple(layer.get_color(-2)) != tuple(layer.get_color(100))
 
@@ -1370,8 +1552,7 @@ def test_negative_label_doesnt_flicker():
         ]
     )
     layer = Labels(data)
-    layer._dims_point = (1, 0, 0)
-    layer._set_view_slice()
+    layer._slice_dims(point=(1, 0, 0))
     # this is expected to fail: -1 doesn't trigger an index error in
     # layer._all_vals, it instead just wraps to 5, the previous max label.
     assert tuple(layer.get_color(-1)) != tuple(layer.get_color(5))
@@ -1395,6 +1576,41 @@ def test_get_status_with_custom_index():
         {'text1': [1, 3], 'text2': [7, -2], 'index': [1, 2]}, index=[1, 2]
     )
     layer.properties = df
-    assert layer.get_status((0, 0)) == 'Labels [0 0]: 0; [No Properties]'
-    assert layer.get_status((3, 3)) == 'Labels [3 3]: 1; text1: 1, text2: 7'
-    assert layer.get_status((6, 6)) == 'Labels [6 6]: 2; text1: 3, text2: -2'
+    assert (
+        layer.get_status((0, 0))['coordinates'] == ' [0 0]: 0; [No Properties]'
+    )
+    assert (
+        layer.get_status((3, 3))['coordinates']
+        == ' [3 3]: 1; text1: 1, text2: 7'
+    )
+    assert (
+        layer.get_status((6, 6))['coordinates']
+        == ' [6 6]: 2; text1: 3, text2: -2'
+    )
+
+
+def test_labels_features_event():
+    event_emitted = False
+
+    def on_event():
+        nonlocal event_emitted
+        event_emitted = True
+
+    layer = Labels(np.zeros((4, 5), dtype=np.uint8))
+    layer.events.features.connect(on_event)
+
+    layer.features = {'some_feature': []}
+
+    assert event_emitted
+
+
+class TestLabels:
+    @staticmethod
+    def get_objects():
+        return [(Labels(np.zeros((10, 10), dtype=np.uint8)))]
+
+    def test_events_defined(self, event_define_check, obj):
+        event_define_check(
+            obj,
+            {"seed", "num_colors", "color"},
+        )
