@@ -5,14 +5,17 @@ from unittest.mock import Mock
 import numpy as np
 import pandas as pd
 import pytest
+from psygnal.containers import Selection
 from pydantic import ValidationError
 from vispy.color import get_colormap
 
 from napari._tests.utils import (
+    assert_colors_equal,
     assert_layer_state_equal,
     check_layer_world_data_extent,
 )
 from napari.layers import Points
+from napari.layers.base._base_constants import ActionType
 from napari.layers.points._points_constants import Mode
 from napari.layers.points._points_utils import points_to_squares
 from napari.layers.utils._text_constants import Anchor
@@ -46,6 +49,27 @@ def test_empty_points():
     assert pts.data.shape == (0, 2)
 
 
+def test_empty_points_with_features():
+    """See the following for the issues this covers:
+    https://github.com/napari/napari/issues/5632
+    https://github.com/napari/napari/issues/5634
+    """
+    points = Points(
+        features={'a': np.empty(0, int)},
+        feature_defaults={'a': 0},
+        face_color='a',
+        face_color_cycle=list('rgb'),
+    )
+
+    points.add([0, 0])
+    points.feature_defaults['a'] = 1
+    points.add([50, 50])
+    points.feature_defaults = {'a': 2}
+    points.add([100, 100])
+
+    assert_colors_equal(points.face_color, list('rgb'))
+
+
 def test_empty_points_with_properties():
     """Test instantiating an empty Points layer with properties
 
@@ -210,7 +234,7 @@ def test_integer_points():
     """Test instantiating Points layer with integer data."""
     shape = (10, 2)
     np.random.seed(0)
-    data = np.random.randint(20, size=(10, 2))
+    data = np.random.randint(20, size=shape)
     layer = Points(data)
     assert np.all(layer.data == data)
     assert layer.ndim == shape[1]
@@ -380,7 +404,7 @@ def test_removing_selected_points():
     layer.remove_selected()
     assert len(layer.data) == shape[0] - 2
     assert len(layer.selected_data) == 0
-    keep = [1, 2] + list(range(4, 10))
+    keep = [1, 2, *range(4, 10)]
     assert np.all(layer.data == data[keep])
     assert layer._value is None
 
@@ -419,13 +443,20 @@ def test_remove_selected_updates_value():
     data = 20 * np.random.random(shape)
     layer = Points(data)
 
+    layer.events.data = Mock()
     # set the value
     layer._value = 3
     layer._value_stored = 3
 
-    layer.selected_data = {0, 5, 6, 7}
+    selection = {0, 5, 6, 7}
+    layer.selected_data = selection
     layer.remove_selected()
-
+    assert layer.events.data.call_args[1] == {
+        "value": layer.data,
+        "action": ActionType.REMOVE.value,
+        "data_indices": tuple(selection),
+        "vertex_indices": ((),),
+    }
     assert layer._value == 2
 
 
@@ -436,6 +467,7 @@ def test_remove_selected_removes_corresponding_attributes():
     np.random.seed(0)
     data = 20 * np.random.random(shape)
     size = np.random.rand(shape[0])
+    symbol = np.random.choice(['o', 's'], shape[0])
     color = np.random.rand(shape[0], 4)
     feature = np.random.rand(shape[0])
     shown = np.random.randint(2, size=shape[0]).astype(bool)
@@ -445,6 +477,7 @@ def test_remove_selected_removes_corresponding_attributes():
         data,
         size=size,
         edge_width=size,
+        symbol=symbol,
         features={'feature': feature},
         face_color=color,
         edge_color=color,
@@ -455,8 +488,10 @@ def test_remove_selected_removes_corresponding_attributes():
     layer_expected = Points(
         data[1:],
         size=size[1:],
+        symbol=symbol[1:],
         edge_width=size[1:],
         features={'feature': feature[1:]},
+        feature_defaults={'feature': feature[0]},
         face_color=color[1:],
         edge_color=color[1:],
         text=text,  # computed from feature
@@ -479,6 +514,7 @@ def test_move():
     data = 20 * np.random.random(shape)
     unmoved = copy(data)
     layer = Points(data)
+    layer.events.data = Mock()
 
     # Move one point relative to an initial drag start location
     layer._move([0], [0, 0])
@@ -486,10 +522,22 @@ def test_move():
     layer._drag_start = None
     assert np.all(layer.data[0] == unmoved[0] + [10, 10])
     assert np.all(layer.data[1:] == unmoved[1:])
+    assert layer.events.data.call_args[1] == {
+        "value": layer.data,
+        "action": ActionType.CHANGE.value,
+        "data_indices": (0,),
+        "vertex_indices": ((),),
+    }
 
     # Move two points relative to an initial drag start location
     layer._move([1, 2], [2, 2])
     layer._move([1, 2], np.add([2, 2], [-3, 4]))
+    assert layer.events.data.call_args[1] == {
+        "value": layer.data,
+        "action": ActionType.CHANGE.value,
+        "data_indices": (1, 2),
+        "vertex_indices": ((),),
+    }
     assert np.all(layer.data[1:2] == unmoved[1:2] + [-3, 4])
 
 
@@ -500,18 +548,18 @@ def test_changing_modes():
     data = 20 * np.random.random(shape)
     layer = Points(data)
     assert layer.mode == 'pan_zoom'
-    assert layer.interactive is True
+    assert layer.mouse_pan is True
 
     layer.mode = 'add'
     assert layer.mode == 'add'
 
     layer.mode = 'select'
     assert layer.mode == 'select'
-    assert layer.interactive is False
+    assert layer.mouse_pan is False
 
     layer.mode = 'pan_zoom'
     assert layer.mode == 'pan_zoom'
-    assert layer.interactive is True
+    assert layer.mouse_pan is True
 
     with pytest.raises(ValueError):
         layer.mode = 'not_a_mode'
@@ -588,13 +636,18 @@ def test_symbol():
     np.random.seed(0)
     data = 20 * np.random.random(shape)
     layer = Points(data)
-    assert layer.symbol == 'disc'
+    assert np.all(layer.symbol == 'disc')
 
     layer.symbol = 'cross'
-    assert layer.symbol == 'cross'
+    assert np.all(layer.symbol == 'cross')
+
+    symbol = ['o', 's'] * 5
+    expected = ['disc', 'square'] * 5
+    layer.symbol = symbol
+    assert np.all(layer.symbol == expected)
 
     layer = Points(data, symbol='star')
-    assert layer.symbol == 'star'
+    assert np.all(layer.symbol == 'star')
 
 
 properties_array = {'point_type': _make_cycled_properties(['A', 'B'], 10)}
@@ -754,7 +807,7 @@ def test_setting_current_properties():
     }
 
     coerced_current_properties = layer.current_properties
-    for k, v in coerced_current_properties.items():
+    for k in coerced_current_properties:
         value = coerced_current_properties[k]
         assert isinstance(value, np.ndarray)
         np.testing.assert_equal(value, expected_current_properties[k])
@@ -797,14 +850,14 @@ def test_text_from_property_fstring(properties):
     layer.selected_data = {0}
     layer._copy_data()
     layer._paste_data()
-    expected_text_3 = expected_text_2 + ['type-ish: A']
+    expected_text_3 = [*expected_text_2, "type-ish: A"]
     np.testing.assert_equal(layer.text.values, expected_text_3)
 
     # add point
     layer.selected_data = {0}
     new_shape = np.random.random((1, 2))
     layer.add(new_shape)
-    expected_text_4 = expected_text_3 + ['type-ish: A']
+    expected_text_4 = [*expected_text_3, "type-ish: A"]
     np.testing.assert_equal(layer.text.values, expected_text_4)
 
 
@@ -827,10 +880,10 @@ def test_set_text_with_kwarg_dict(properties):
     expected_text = ['type: ' + v for v in properties['point_type']]
     np.testing.assert_equal(layer.text.values, expected_text)
 
-    for property, value in text_kwargs.items():
-        if property == 'string':
+    for property_, value in text_kwargs.items():
+        if property_ == 'string':
             continue
-        layer_value = getattr(layer._text, property)
+        layer_value = getattr(layer._text, property_)
         np.testing.assert_equal(layer_value, value)
 
 
@@ -855,6 +908,18 @@ def test_select_properties_object_dtype():
     assert pl.selected_data == selection
 
 
+def test_select_properties_unsortable():
+    """selecting multiple points when they have properties that cannot be sorted should not fail
+
+    see https://github.com/napari/napari/issues/5174
+    """
+    properties = pd.DataFrame({'unsortable': [{}, {}]})
+    pl = Points(np.ones((2, 2)), properties=properties)
+    selection = {0, 1}
+    pl.selected_data = selection
+    assert pl.selected_data == selection
+
+
 def test_refresh_text():
     """Test refreshing the text after setting new properties"""
     shape = (10, 2)
@@ -1044,9 +1109,8 @@ def test_colormap_with_categorical_properties(attribute):
     properties = {'point_type': _make_cycled_properties(['A', 'B'], shape[0])}
     layer = Points(data, properties=properties)
 
-    with pytest.raises(TypeError):
-        with pytest.warns(UserWarning):
-            setattr(layer, f'{attribute}_color_mode', 'colormap')
+    with pytest.raises(TypeError), pytest.warns(UserWarning):
+        setattr(layer, f'{attribute}_color_mode', 'colormap')
 
 
 @pytest.mark.parametrize("attribute", ['edge', 'face'])
@@ -1071,9 +1135,18 @@ def test_add_point_direct(attribute: str):
     """Test adding points to layer directly"""
     layer = Points()
     assert len(getattr(layer, f'{attribute}_color')) == 0
+
+    layer.events.data = Mock()
     setattr(layer, f'current_{attribute}_color', 'red')
     coord = [18, 18]
+
     layer.add(coord)
+    assert layer.events.data.call_args[1] == {
+        "value": layer.data,
+        "action": ActionType.ADD.value,
+        "data_indices": (-1,),
+        "vertex_indices": ((),),
+    }
     np.testing.assert_allclose(
         [[1, 0, 0, 1]], getattr(layer, f'{attribute}_color')
     )
@@ -1372,33 +1445,33 @@ def test_size():
     data = 20 * np.random.random(shape)
     layer = Points(data)
     assert layer.current_size == 10
-    assert layer.size.shape == shape
+    assert layer.size.shape == (10,)
     assert np.unique(layer.size)[0] == 10
 
     # Add a new point, it should get current size
     coord = [17, 17]
     layer.add(coord)
-    assert layer.size.shape == (11, 2)
+    assert layer.size.shape == (11,)
     assert np.unique(layer.size)[0] == 10
 
     # Setting size affects newly added points not current points
     layer.current_size = 20
     assert layer.current_size == 20
-    assert layer.size.shape == (11, 2)
+    assert layer.size.shape == (11,)
     assert np.unique(layer.size)[0] == 10
 
     # Add new point, should have new size
     coord = [18, 18]
     layer.add(coord)
-    assert layer.size.shape == (12, 2)
+    assert layer.size.shape == (12,)
     assert np.unique(layer.size[:11])[0] == 10
-    assert np.all(layer.size[11] == [20, 20])
+    assert np.all(layer.size[11] == 20)
 
     # Select data and change size
     layer.selected_data = {0, 1}
     assert layer.current_size == 10
     layer.current_size = 16
-    assert layer.size.shape == (12, 2)
+    assert layer.size.shape == (12,)
     assert np.unique(layer.size[2:11])[0] == 10
     assert np.unique(layer.size[:2])[0] == 16
 
@@ -1407,141 +1480,59 @@ def test_size():
     assert layer.current_size == 20
 
 
-def test_size_with_arrays():
+@pytest.mark.parametrize('ndim', [2, 3])
+def test_size_with_arrays(ndim):
     """Test setting size with arrays."""
-    shape = (10, 2)
+    shape = (10, ndim)
     np.random.seed(0)
     data = 20 * np.random.random(shape)
     layer = Points(data)
-    sizes = 5 * np.random.random(shape)
+    sizes = 5 * np.random.random(10)
     layer.size = sizes
     assert np.all(layer.size == sizes)
 
-    # Test broadcasting of sizes
-    sizes = [5, 5]
-    layer.size = sizes
-    assert np.all(layer.size[0] == sizes)
-
-    # Test broadcasting of transposed sizes
-    sizes = np.random.randint(low=1, high=5, size=shape[::-1])
-    layer.size = sizes
-    np.testing.assert_equal(layer.size, sizes.T)
-
     # Un-broadcastable array should raise an exception
-    bad_sizes = np.random.randint(low=1, high=5, size=(3, 8))
-    with pytest.raises(ValueError):
-        layer.size = bad_sizes
-
-    # Create new layer with new size array data
-    sizes = 5 * np.random.random(shape)
-    layer = Points(data, size=sizes)
-    assert layer.current_size == 10
-    assert layer.size.shape == shape
-    assert np.all(layer.size == sizes)
-
-    # Create new layer with new size array data
     sizes = [5, 5]
+    with pytest.raises(ValueError):
+        layer.size = sizes
+
+    # Create new layer with new size array data
+    sizes = 5 * np.random.random(10)
     layer = Points(data, size=sizes)
     assert layer.current_size == 10
-    assert layer.size.shape == shape
-    assert np.all(layer.size[0] == sizes)
+    assert layer.size.shape == (10,)
+    np.testing.assert_array_equal(layer.size, sizes)
 
     # Add new point, should have new size
-    coord = [18, 18]
+    coord = [18] * ndim
     layer.current_size = 13
     layer.add(coord)
-    assert layer.size.shape == (11, 2)
-    assert np.unique(layer.size[:10])[0] == 5
-    assert np.all(layer.size[10] == [13, 13])
+    assert layer.size.shape == (11,)
+    np.testing.assert_array_equal(layer.size[:10], sizes[:10])
+    assert layer.size[10] == 13
 
     # Select data and change size
     layer.selected_data = {0, 1}
-    assert layer.current_size == 5
+    # current_size does not change because idx 0 and 1 are different sizes
+    assert layer.current_size == 13
     layer.current_size = 16
-    assert layer.size.shape == (11, 2)
-    assert np.unique(layer.size[2:10])[0] == 5
-    assert np.unique(layer.size[:2])[0] == 16
+    assert layer.size.shape == (11,)
+    np.testing.assert_array_equal(layer.size[2:10], sizes[2:10])
+    np.testing.assert_array_equal(layer.size[:2], 16)
 
-    # Check removing data adjusts colors correctly
+    # check that current size is correctly set if all points are the same size
+    layer.selected_data = {10}
+    assert layer.current_size == 13
+    layer.selected_data = {0, 1}
+    assert layer.current_size == 16
+
+    # Check removing data adjusts sizes correctly
     layer.selected_data = {0, 2}
     layer.remove_selected()
     assert len(layer.data) == 9
     assert len(layer.size) == 9
-    assert np.all(layer.size[0] == [16, 16])
-    assert np.all(layer.size[1] == [5, 5])
-
-
-def test_size_with_3D_arrays():
-    """Test setting size with 3D arrays."""
-    shape = (10, 3)
-    np.random.seed(0)
-    data = 20 * np.random.random(shape)
-    data[:2, 0] = 0
-    layer = Points(data)
-    assert layer.current_size == 10
-    assert layer.size.shape == shape
-    assert np.unique(layer.size)[0] == 10
-
-    sizes = 5 * np.random.random(shape)
-    layer.size = sizes
-    assert np.all(layer.size == sizes)
-
-    # Test broadcasting of sizes
-    sizes = [1, 5, 5]
-    layer.size = sizes
-    assert np.all(layer.size[0] == sizes)
-
-    # Create new layer with new size array data
-    sizes = 5 * np.random.random(shape)
-    layer = Points(data, size=sizes)
-    assert layer.current_size == 10
-    assert layer.size.shape == shape
-    assert np.all(layer.size == sizes)
-
-    # Create new layer with new size array data
-    sizes = [1, 5, 5]
-    layer = Points(data, size=sizes)
-    assert layer.current_size == 10
-    assert layer.size.shape == shape
-    assert np.all(layer.size[0] == sizes)
-
-    # Add new point, should have new size in last dim only
-    coord = [4, 18, 18]
-    layer.current_size = 13
-    layer.add(coord)
-    assert layer.size.shape == (11, 3)
-    assert np.unique(layer.size[:10, 1:])[0] == 5
-    assert np.all(layer.size[10] == [1, 13, 13])
-
-    # Select data and change size
-    layer.selected_data = {0, 1}
-    assert layer.current_size == 5
-    layer.current_size = 16
-    assert layer.size.shape == (11, 3)
-    assert np.unique(layer.size[2:10, 1:])[0] == 5
-    assert np.all(layer.size[0] == [16, 16, 16])
-
-    # Create new 3D layer with new 2D points size data
-    sizes = [0, 5, 5]
-    layer = Points(data, size=sizes)
-    assert layer.current_size == 10
-    assert layer.size.shape == shape
-    assert np.all(layer.size[0] == sizes)
-
-    # Add new point, should have new size only in last 2 dimensions
-    coord = [4, 18, 18]
-    layer.current_size = 13
-    layer.add(coord)
-    assert layer.size.shape == (11, 3)
-    assert np.all(layer.size[10] == [0, 13, 13])
-
-    # Select data and change size
-    layer.selected_data = {0, 1}
-    assert layer.current_size == 5
-    layer.current_size = 16
-    assert layer.size.shape == (11, 3)
-    assert np.unique(layer.size[2:10, 1:])[0] == 5
-    assert np.all(layer.size[0] == [0, 16, 16])
+    assert layer.size[0] == 16
+    assert layer.size[1] == sizes[3]
 
 
 def test_copy_and_paste():
@@ -1721,33 +1712,26 @@ def test_view_data():
     layer = Points(coords)
 
     layer._slice_dims([0, slice(None), slice(None)])
-    assert np.all(
-        layer._view_data == coords[np.ix_([0, 1], layer._dims_displayed)]
-    )
+    assert np.all(layer._view_data == coords[np.ix_([0, 1], [1, 2])])
 
     layer._slice_dims([1, slice(None), slice(None)])
-    assert np.all(
-        layer._view_data == coords[np.ix_([2], layer._dims_displayed)]
-    )
+    assert np.all(layer._view_data == coords[np.ix_([2], [1, 2])])
 
     layer._slice_dims([1, slice(None), slice(None)], ndisplay=3)
     assert np.all(layer._view_data == coords)
 
 
 def test_view_size():
+    """Test out of slice point rendering and slicing with no points."""
     coords = np.array([[0, 1, 1], [0, 2, 2], [1, 3, 3], [3, 3, 3]])
-    sizes = np.array([[3, 5, 5], [3, 5, 5], [3, 3, 3], [2, 2, 3]])
+    sizes = np.array([5, 5, 3, 3])
     layer = Points(coords, size=sizes, out_of_slice_display=False)
 
     layer._slice_dims([0, slice(None), slice(None)])
-    assert np.all(
-        layer._view_size == sizes[np.ix_([0, 1], layer._dims_displayed)]
-    )
+    assert np.all(layer._view_size == sizes[[0, 1]])
 
     layer._slice_dims([1, slice(None), slice(None)])
-    assert np.all(
-        layer._view_size == sizes[np.ix_([2], layer._dims_displayed)]
-    )
+    assert np.all(layer._view_size == sizes[[2]])
 
     layer.out_of_slice_display = True
     assert len(layer._view_size) == 3
@@ -1810,24 +1794,6 @@ def test_world_data_extent():
     check_layer_world_data_extent(layer, extent, (3, 1, 1), (10, 20, 5), False)
 
 
-def test_slice_data():
-    data = [
-        (10, 2, 4),
-        (10 + 2 * 1e-7, 4, 6),
-        (8, 1, 7),
-        (10.1, 7, 2),
-        (10 - 2 * 1e-7, 1, 6),
-    ]
-    layer = Points(data)
-    assert len(layer._slice_data((8, slice(None), slice(None)))[0]) == 1
-    assert len(layer._slice_data((10, slice(None), slice(None)))[0]) == 4
-    assert (
-        len(layer._slice_data((10 + 2 * 1e-12, slice(None), slice(None)))[0])
-        == 4
-    )
-    assert len(layer._slice_data((10.1, slice(None), slice(None)))[0]) == 4
-
-
 def test_scale_init():
     layer = Points(None, scale=(1, 1, 1, 1))
     assert layer.ndim == 4
@@ -1971,7 +1937,7 @@ def test_to_mask_2d_with_size_4_bottom_right():
 
 
 def test_to_mask_2d_with_diff_sizes():
-    points = Points([[2, 2], [1, 4]], size=[[1, 1], [2, 2]])
+    points = Points([[2, 2], [1, 4]], size=[1, 2])
 
     mask = points.to_mask(shape=(5, 7))
 
@@ -2303,40 +2269,6 @@ def test_text_param_and_setter_are_consistent():
     )
 
 
-def test_editable_2d_layer_ndisplay_3():
-    """Interactivity doesn't work for 2D points layers
-    being rendered in 3D. Verify that layer.editable is set
-    to False upon switching to 3D rendering mode.
-
-    See: https://github.com/napari/napari/pull/4184
-    """
-    data = np.random.random((10, 2))
-    layer = Points(data, size=5)
-    assert layer.editable is True
-
-    # simulate switching to 3D rendering
-    # layer should no longer b editable
-    layer._slice_dims([0, 0, 0], ndisplay=3)
-    assert layer.editable is False
-
-
-def test_editable_3d_layer_ndisplay_3():
-    """Interactivity works for 3D points layers
-    being rendered in 3D. Verify that layer.editable remains
-    True upon switching to 3D rendering mode.
-
-    See: https://github.com/napari/napari/pull/4184
-    """
-    data = np.random.random((10, 3))
-    layer = Points(data, size=5)
-    assert layer.editable is True
-
-    # simulate switching to 3D rendering
-    # layer should no longer b editable
-    layer._slice_dims([0, 0, 0], ndisplay=3)
-    assert layer.editable is True
-
-
 def test_shown():
     """Test setting shown property"""
     shape = (10, 2)
@@ -2360,24 +2292,6 @@ def test_shown():
     assert layer.shown[-1] == True  # noqa
 
 
-def test_selected_data_with_non_uniform_sizes():
-    data = np.zeros((3, 2))
-    size = [[1, 3], [1, 4], [1, 3]]
-    layer = Points(data, size=size)
-    # Current size is the default 10 because passed size is not a scalar.
-    assert layer.current_size == 10
-
-    # The first two points have different mean sizes, so the current size
-    # should not change.
-    layer.selected_data = (0, 1)
-    assert layer.current_size == 10
-
-    # The first and last point have the same mean size, so the current size
-    # should change to that mean.
-    layer.selected_data = (0, 2)
-    assert layer.current_size == 2
-
-
 def test_shown_view_size_and_view_data_have_the_same_dimension():
     data = [[0, 0, 0], [1, 1, 1]]
     # Data with default settings
@@ -2453,7 +2367,7 @@ def test_empty_data_from_tuple():
 @pytest.mark.parametrize(
     'attribute, new_value',
     [
-        ("size", [20, 20]),
+        ("size", 20),
         ("face_color", np.asarray([0.0, 0.0, 1.0, 1.0])),
         ("edge_color", np.asarray([0.0, 0.0, 1.0, 1.0])),
         ("edge_width", np.asarray([0.2])),
@@ -2495,10 +2409,32 @@ def test_set_drag_start():
     assert layer._drag_start is None
     position = (0, 1)
     layer._set_drag_start({0}, position=position)
-    assert all(
-        layer._drag_start[i] == position[i] for i in layer._dims_displayed
-    )
+    np.testing.assert_array_equal(layer._drag_start, position)
     layer._set_drag_start({0}, position=(1, 2))
-    assert all(
-        layer._drag_start[i] == position[i] for i in layer._dims_displayed
-    )
+    np.testing.assert_array_equal(layer._drag_start, position)
+
+
+def test_editable_and_visible_are_independent():
+    """See https://github.com/napari/napari/issues/1346"""
+    data = np.empty((0, 2))
+    layer = Points(data)
+    assert layer.editable
+    assert layer.visible
+
+    layer.editable = False
+    layer.visible = False
+    assert not layer.editable
+    assert not layer.visible
+
+    layer.visible = True
+
+    assert not layer.editable
+
+
+def test_point_selection_remains_evented_after_update():
+    """Existing evented selection model should be updated rather than replaced."""
+    data = np.empty((3, 2))
+    layer = Points(data)
+    assert isinstance(layer.selected_data, Selection)
+    layer.selected_data = {0, 1}
+    assert isinstance(layer.selected_data, Selection)
