@@ -1,7 +1,8 @@
 import numpy as np
 
-from .._shapes_utils import find_corners, rectangle_to_box
-from .shape import Shape
+from napari.layers.shapes._shapes_models.shape import Shape
+from napari.layers.shapes._shapes_utils import find_corners, rectangle_to_box
+from napari.utils.translations import trans
 
 
 class Rectangle(Shape):
@@ -23,9 +24,14 @@ class Rectangle(Shape):
     """
 
     def __init__(
-        self, data, *, edge_width=1, z_index=0, dims_order=None, ndisplay=2,
-    ):
-
+        self,
+        data,
+        *,
+        edge_width=1,
+        z_index=0,
+        dims_order=None,
+        ndisplay=2,
+    ) -> None:
         super().__init__(
             edge_width=edge_width,
             z_index=z_index,
@@ -39,8 +45,7 @@ class Rectangle(Shape):
 
     @property
     def data(self):
-        """(4, D) array: rectangle vertices.
-        """
+        """(4, D) array: rectangle vertices."""
         return self._data
 
     @data.setter
@@ -56,9 +61,11 @@ class Rectangle(Shape):
         if len(data) != 4:
             print(data)
             raise ValueError(
-                f"""Data shape does not match a rectangle.
-                             Rectangle expects four corner vertices,
-                             {len(data)} provided."""
+                trans._(
+                    "Data shape does not match a rectangle. Rectangle expects four corner vertices, {number} provided.",
+                    deferred=True,
+                    number=len(data),
+                )
             )
 
         self._data = data
