@@ -1,16 +1,25 @@
 import warnings
+from typing import Any, List, Optional, Tuple, Union
 
 import numpy as np
 
-from ...utils.colormaps import AVAILABLE_COLORMAPS
-from ...utils.events import Event
-from ...utils.translations import trans
-from ..base import Layer
-from ..intensity_mixin import IntensityVisualizationMixin
-from ..utils.layer_utils import calc_data_range
-from ._surface_constants import Shading
-from .normals import SurfaceNormals
-from .wireframe import SurfaceWireframe
+from napari.layers.base import Layer
+from napari.layers.intensity_mixin import IntensityVisualizationMixin
+from napari.layers.surface._surface_constants import Shading
+from napari.layers.surface._surface_utils import (
+    calculate_barycentric_coordinates,
+)
+from napari.layers.surface.normals import SurfaceNormals
+from napari.layers.surface.wireframe import SurfaceWireframe
+from napari.layers.utils.interactivity_utils import (
+    nd_line_segment_to_displayed_data_ray,
+)
+from napari.layers.utils.layer_utils import calc_data_range
+from napari.utils.colormaps import AVAILABLE_COLORMAPS
+from napari.utils.events import Event
+from napari.utils.events.event_utils import connect_no_arg
+from napari.utils.geometry import find_nearest_triangle_intersection
+from napari.utils.translations import trans
 
 
 # Mixin must come before Layer
@@ -18,15 +27,29 @@ class Surface(IntensityVisualizationMixin, Layer):
     """
     Surface layer renders meshes onto the canvas.
 
+    Surfaces may be colored by:
+        * setting `vertex_values`, which colors the surface with the selected
+          `colormap` (default is uniform ones)
+        * setting `vertex_colors`, which replaces/overrides any color from
+          `vertex_values`
+        * setting both `texture` and `texcoords`, which blends a the value from
+          a texture (image) with the underlying color from `vertex_values` or
+          `vertex_colors`. Blending is achieved by multiplying the texture
+          color by the underlying color - an underlying value of "white" will
+          result in the unaltered texture color.
+
     Parameters
     ----------
     data : 2-tuple or 3-tuple of array
         The first element of the tuple is an (N, D) array of vertices of
-        mesh triangles. The second is an (M, 3) array of int of indices
-        of the mesh triangles. The optional third element is the
-        (K0, ..., KL, N) array of values used to color vertices where the
-        additional L dimensions are used to color the same mesh with
-        different values. If not provided, it defaults to ones.
+        mesh triangles.
+
+        The second is an (M, 3) array of int of indices of the mesh triangles.
+
+        The optional third element is the (K0, ..., KL, N) array of values
+        (vertex_values) used to color vertices where the additional L
+        dimensions are used to color the same mesh with different values. If
+        not provided, it defaults to ones.
     colormap : str, napari.utils.Colormap, tuple, dict
         Colormap to use for luminance images. If a string must be the name
         of a supported colormap from vispy or matplotlib. If a tuple the
@@ -34,6 +57,20 @@ class Surface(IntensityVisualizationMixin, Layer):
         the second item must be a Colormap. If a dict the key must be a
         string to assign as a name to a colormap and the value must be a
         Colormap.
+    texture: (I, J) or (I, J, C) array
+        A 2D texture to be mapped onto the mesh using `texcoords`.
+        C may be 3 (RGB) or 4 (RGBA) channels for a color texture.
+    texcoords: (N, 2) array
+        2D coordinates for each vertex, mapping into the texture.
+        The number of texture coords must match the number of vertices (N).
+        Coordinates should be in [0.0, 1.0] and are scaled to sample the 2D
+        texture. Coordinates outside this range will wrap, but this behavior
+        should be considered an implementation detail: there are no plans to
+        change it, but it's a feature of the underlying vispy visual.
+    vertex_colors: (N, C) or (K0, ..., KL, N, C) array of color values
+        Take care that the (optional) L additional dimensions match those of
+        vertex_values for proper slicing.
+        C may be 3 (RGB) or 4 (RGBA) channels.
     contrast_limits : list (2,)
         Color limits to be used for determining the colormap bounds for
         luminance images. If not passed is calculated as the min and max of
@@ -84,9 +121,9 @@ class Surface(IntensityVisualizationMixin, Layer):
     cache : bool
         Whether slices of out-of-core datasets should be cached upon retrieval.
         Currently, this only applies to dask arrays.
-    wireframe : dict or SurfaceWireframe
+    wireframe : None, dict or SurfaceWireframe
         Whether and how to display the edges of the surface mesh with a wireframe.
-    normals : dict or SurfaceNormals
+    normals : None, dict or SurfaceNormals
         Whether and how to display the face and vertex normals of the surface mesh.
 
     Attributes
@@ -164,8 +201,10 @@ class Surface(IntensityVisualizationMixin, Layer):
         experimental_clipping_planes=None,
         wireframe=None,
         normals=None,
-    ):
-
+        texture=None,
+        texcoords=None,
+        vertex_colors=None,
+    ) -> None:
         ndim = data[0].shape[1]
 
         super().__init__(
@@ -189,13 +228,17 @@ class Surface(IntensityVisualizationMixin, Layer):
             interpolation=Event,
             rendering=Event,
             shading=Event,
+            wireframe=Event,
+            normals=Event,
+            texture=Event,
+            texcoords=Event,
         )
 
         # assign mesh data and establish default behavior
         if len(data) not in (2, 3):
             raise ValueError(
                 trans._(
-                    'Surface data tuple must be 2 or 3, specifying verictes, faces, and optionally vertex values, instead got length {length}.',
+                    'Surface data tuple must be 2 or 3, specifying vertices, faces, and optionally vertex values, instead got length {length}.',
                     deferred=True,
                     length=len(data),
                 )
@@ -207,6 +250,10 @@ class Surface(IntensityVisualizationMixin, Layer):
         else:
             self._vertex_values = np.ones(len(self._vertices))
 
+        self._texture = texture
+        self._texcoords = texcoords
+        self._vertex_colors = vertex_colors
+
         # Set contrast_limits and colormaps
         self._gamma = gamma
         if contrast_limits is None:
@@ -218,18 +265,27 @@ class Surface(IntensityVisualizationMixin, Layer):
         self.contrast_limits = self._contrast_limits
 
         # Data containing vectors in the currently viewed slice
-        self._data_view = np.zeros((0, self._ndisplay))
+        self._data_view = np.zeros((0, self._slice_input.ndisplay))
         self._view_faces = np.zeros((0, 3))
         self._view_vertex_values = []
+        self._view_vertex_colors = []
 
-        # Trigger generation of view slice and thumbnail
+        # Trigger generation of view slice and thumbnail.
+        # Use _update_dims instead of refresh here because _get_ndim is
+        # dependent on vertex_values as well as vertices.
         self._update_dims()
 
         # Shading mode
         self._shading = shading
 
-        self.wireframe = wireframe or SurfaceWireframe()
-        self.normals = normals or SurfaceNormals()
+        # initialize normals and wireframe
+        self._wireframe = SurfaceWireframe()
+        self._normals = SurfaceNormals()
+        connect_no_arg(self.wireframe.events, self.events, 'wireframe')
+        connect_no_arg(self.normals.events, self.events, 'normals')
+
+        self.wireframe = wireframe
+        self.normals = normals
 
     def _calc_data_range(self, mode='data'):
         return calc_data_range(self.vertex_values)
@@ -261,6 +317,7 @@ class Surface(IntensityVisualizationMixin, Layer):
 
         self._update_dims()
         self.events.data(value=self.data)
+        self._reset_editable()
         if self._keep_auto_contrast:
             self.reset_contrast_limits()
 
@@ -275,9 +332,8 @@ class Surface(IntensityVisualizationMixin, Layer):
         self._vertices = vertices
 
         self._update_dims()
-        self.refresh()
         self.events.data(value=self.data)
-        self._set_editable()
+        self._reset_editable()
 
     @property
     def vertex_values(self) -> np.ndarray:
@@ -285,13 +341,41 @@ class Surface(IntensityVisualizationMixin, Layer):
 
     @vertex_values.setter
     def vertex_values(self, vertex_values: np.ndarray):
-        """Array of values used to color vertices.."""
+        """Array of values (n, 1) used to color vertices with a colormap."""
+        if vertex_values is None:
+            vertex_values = np.ones(len(self._vertices))
 
         self._vertex_values = vertex_values
 
-        self.refresh()
+        self._update_dims()
         self.events.data(value=self.data)
-        self._set_editable()
+        self._reset_editable()
+
+    @property
+    def vertex_colors(self) -> Optional[np.ndarray]:
+        return self._vertex_colors
+
+    @vertex_colors.setter
+    def vertex_colors(self, vertex_colors: Optional[np.ndarray]):
+        """Values used to directly color vertices.
+
+        Note that dims sliders for this layer are based on vertex_values, so
+        make sure the shape of vertex_colors matches the shape of vertex_values
+        for proper slicing. That is: vertex_colors should be None, one set
+        (N, C), or completely match the dimensions of vertex_values
+        (K0, ..., KL, N, C).
+        """
+        if vertex_colors is not None and not isinstance(
+            vertex_colors, np.ndarray
+        ):
+            msg = (
+                f"texture should be None or ndarray; got {type(vertex_colors)}"
+            )
+            raise ValueError(msg)
+        self._vertex_colors = vertex_colors
+        self._update_dims()
+        self.events.data(value=self.data)
+        self._reset_editable()
 
     @property
     def faces(self) -> np.ndarray:
@@ -299,13 +383,13 @@ class Surface(IntensityVisualizationMixin, Layer):
 
     @faces.setter
     def faces(self, faces: np.ndarray):
-        """Array of indices of mesh triangles.."""
+        """Array of indices of mesh triangles."""
 
         self.faces = faces
 
         self.refresh()
         self.events.data(value=self.data)
-        self._set_editable()
+        self._reset_editable()
 
     def _get_ndim(self):
         """Determine number of dimensions of the layer."""
@@ -330,7 +414,9 @@ class Surface(IntensityVisualizationMixin, Layer):
             # dimensionality of the vertices themselves
             if self.vertex_values.ndim > 1:
                 mins = [0] * (self.vertex_values.ndim - 1) + list(mins)
-                maxs = list(self.vertex_values.shape[:-1]) + list(maxs)
+                maxs = [n - 1 for n in self.vertex_values.shape[:-1]] + list(
+                    maxs
+                )
             extrema = np.vstack([mins, maxs])
         return extrema
 
@@ -346,6 +432,76 @@ class Surface(IntensityVisualizationMixin, Layer):
             self._shading = Shading(shading)
         self.events.shading(value=self._shading)
 
+    @property
+    def wireframe(self) -> SurfaceWireframe:
+        return self._wireframe
+
+    @wireframe.setter
+    def wireframe(self, wireframe: Union[dict, SurfaceWireframe, None]):
+        if wireframe is None:
+            self._wireframe.reset()
+        elif isinstance(wireframe, (SurfaceWireframe, dict)):
+            self._wireframe.update(wireframe)
+        else:
+            raise ValueError(
+                f'wireframe should be None, a dict, or SurfaceWireframe; got {type(wireframe)}'
+            )
+        self.events.wireframe(value=self._wireframe)
+
+    @property
+    def normals(self) -> SurfaceNormals:
+        return self._normals
+
+    @normals.setter
+    def normals(self, normals: Union[dict, SurfaceNormals, None]):
+        if normals is None:
+            self._normals.reset()
+        elif not isinstance(normals, (SurfaceNormals, dict)):
+            raise ValueError(
+                f'normals should be None, a dict, or SurfaceNormals; got {type(normals)}'
+            )
+        else:
+            if isinstance(normals, SurfaceNormals):
+                normals = {k: dict(v) for k, v in normals.dict().items()}
+            # ignore modes, they are unmutable cause errors
+            for norm_type in ('face', 'vertex'):
+                normals.get(norm_type, {}).pop('mode', None)
+            self._normals.update(normals)
+        self.events.normals(value=self._normals)
+
+    @property
+    def texture(self) -> Optional[np.ndarray]:
+        return self._texture
+
+    @texture.setter
+    def texture(self, texture: np.ndarray):
+        if texture is not None and not isinstance(texture, np.ndarray):
+            msg = f"texture should be None or ndarray; got {type(texture)}"
+            raise ValueError(msg)
+        self._texture = texture
+        self.events.texture(value=self._texture)
+
+    @property
+    def texcoords(self) -> Optional[np.ndarray]:
+        return self._texcoords
+
+    @texcoords.setter
+    def texcoords(self, texcoords: np.ndarray):
+        if texcoords is not None and not isinstance(texcoords, np.ndarray):
+            msg = f"texcoords should be None or ndarray; got {type(texcoords)}"
+            raise ValueError(msg)
+        self._texcoords = texcoords
+        self.events.texcoords(value=self._texcoords)
+
+    @property
+    def _has_texture(self) -> bool:
+        """Whether the layer has sufficient data for texturing"""
+        return bool(
+            self.texture is not None
+            and self.texcoords is not None
+            and len(self.texcoords)
+        )
+
     def _get_state(self):
         """Get dictionary of layer state.
 
@@ -364,58 +520,89 @@ class Surface(IntensityVisualizationMixin, Layer):
                 'data': self.data,
                 'wireframe': self.wireframe.dict(),
                 'normals': self.normals.dict(),
+                'texture': self.texture,
+                'texcoords': self.texcoords,
+                'vertex_colors': self.vertex_colors,
             }
         )
         return state
 
+    def _slice_associated_data(
+        self,
+        data: np.ndarray,
+        vertex_ndim: int,
+        dims: int = 1,
+    ) -> Union[List[Any], np.ndarray]:
+        """Return associated layer data (e.g. vertex values, colors) within
+        the current slice.
+        """
+        if data is None:
+            return []
+
+        data_ndim = data.ndim - 1
+        if data_ndim >= dims:
+            # Get indices for axes corresponding to data dimensions
+            data_indices = self._slice_indices[:-vertex_ndim]
+            data = data[data_indices]
+            if data.ndim > dims:
+                warnings.warn(
+                    trans._(
+                        "Assigning multiple data per vertex after slicing "
+                        "is not allowed. All dimensions corresponding to "
+                        "vertex data must be non-displayed dimensions. Data "
+                        "may not be visible.",
+                        deferred=True,
+                    ),
+                    category=UserWarning,
+                    stacklevel=2,
+                )
+                return []
+        return data
+
     def _set_view_slice(self):
         """Sets the view given the indices to slice with."""
         N, vertex_ndim = self.vertices.shape
         values_ndim = self.vertex_values.ndim - 1
 
-        # Take vertex_values dimensionality into account if more than one value
-        # is provided per vertex.
-        if values_ndim > 0:
-            # Get indices for axes corresponding to values dimensions
-            values_indices = self._slice_indices[:-vertex_ndim]
-            values = self.vertex_values[values_indices]
-            if values.ndim > 1:
-                warnings.warn(
-                    trans._(
-                        "Assigning multiple values per vertex after slicing is not allowed. All dimensions corresponding to vertex_values must be non-displayed dimensions. Data will not be visible.",
-                        deferred=True,
-                    )
-                )
-                self._data_view = np.zeros((0, self._ndisplay))
-                self._view_faces = np.zeros((0, 3))
-                self._view_vertex_values = []
-                return
+        self._view_vertex_values = self._slice_associated_data(
+            self.vertex_values,
+            vertex_ndim,
+        )
 
-            self._view_vertex_values = values
-            # Determine which axes of the vertices data are being displayed
-            # and not displayed, ignoring the additional dimensions
-            # corresponding to the vertex_values.
+        self._view_vertex_colors = self._slice_associated_data(
+            self.vertex_colors,
+            vertex_ndim,
+            dims=2,
+        )
+
+        if len(self._view_vertex_values) == 0:
+            self._data_view = np.zeros((0, self._slice_input.ndisplay))
+            self._view_faces = np.zeros((0, 3))
+            return
+
+        if values_ndim > 0:
             indices = np.array(self._slice_indices[-vertex_ndim:])
             disp = [
                 d
-                for d in np.subtract(self._dims_displayed, values_ndim)
+                for d in np.subtract(self._slice_input.displayed, values_ndim)
                 if d >= 0
             ]
             not_disp = [
                 d
-                for d in np.subtract(self._dims_not_displayed, values_ndim)
+                for d in np.subtract(
+                    self._slice_input.not_displayed, values_ndim
+                )
                 if d >= 0
             ]
         else:
-            self._view_vertex_values = self.vertex_values
             indices = np.array(self._slice_indices)
-            not_disp = list(self._dims_not_displayed)
-            disp = list(self._dims_displayed)
+            not_disp = list(self._slice_input.not_displayed)
+            disp = list(self._slice_input.displayed)
 
         self._data_view = self.vertices[:, disp]
         if len(self.vertices) == 0:
             self._view_faces = np.zeros((0, 3))
-        elif vertex_ndim > self._ndisplay:
+        elif vertex_ndim > self._slice_input.ndisplay:
             vertices = self.vertices[:, not_disp].astype('int')
             triangles = vertices[self.faces]
             matches = np.all(triangles == indices[not_disp], axis=(1, 2))
@@ -432,7 +619,6 @@ class Surface(IntensityVisualizationMixin, Layer):
 
     def _update_thumbnail(self):
         """Update thumbnail with current surface."""
-        pass
 
     def _get_value(self, position):
         """Value of the data at a position in data coordinates.
@@ -447,4 +633,69 @@ class Surface(IntensityVisualizationMixin, Layer):
         value : None
             Value of the data at the coord.
         """
-        return None
+        return
+
+    def _get_value_3d(
+        self,
+        start_point: np.ndarray,
+        end_point: np.ndarray,
+        dims_displayed: List[int],
+    ) -> Tuple[Union[None, float, int], Optional[int]]:
+        """Get the layer data value along a ray
+
+        Parameters
+        ----------
+        start_point : np.ndarray
+            The start position of the ray used to interrogate the data.
+        end_point : np.ndarray
+            The end position of the ray used to interrogate the data.
+        dims_displayed : List[int]
+            The indices of the dimensions currently displayed in the Viewer.
+
+        Returns
+        -------
+        value
+            The data value along the supplied ray.
+        vertex : None
+            Index of vertex if any that is at the coordinates.
+        """
+        if len(dims_displayed) != 3:
+            # only applies to 3D
+            return None, None
+        if (start_point is None) or (end_point is None):
+            # return None if the ray doesn't intersect the data bounding box
+            return None, None
+
+        start_position, ray_direction = nd_line_segment_to_displayed_data_ray(
+            start_point=start_point,
+            end_point=end_point,
+            dims_displayed=dims_displayed,
+        )
+
+        # get the mesh triangles
+        mesh_triangles = self._data_view[self._view_faces]
+
+        # get the triangles intersection
+        intersection_index, intersection = find_nearest_triangle_intersection(
+            ray_position=start_position,
+            ray_direction=ray_direction,
+            triangles=mesh_triangles,
+        )
+
+        if intersection_index is None:
+            return None, None
+
+        # add the full nD coords to intersection
+        intersection_point = start_point.copy()
+        intersection_point[dims_displayed] = intersection
+
+        # calculate the value from the intersection
+        triangle_vertex_indices = self._view_faces[intersection_index]
+        triangle_vertices = self._data_view[triangle_vertex_indices]
+        barycentric_coordinates = calculate_barycentric_coordinates(
+            intersection, triangle_vertices
+        )
+        vertex_values = self._view_vertex_values[triangle_vertex_indices]
+        intersection_value = (barycentric_coordinates * vertex_values).sum()
+
+        return intersection_value, intersection_index
