@@ -9,13 +9,14 @@ from weakref import ReferenceType, ref
 if TYPE_CHECKING:
     from napari.layers import Layer
 
-from ...utils.translations import trans
+from napari.utils.events.event import WarningEmitter
+from napari.utils.translations import trans
 
 #: Record of already linked layers... to avoid duplicating callbacks
 #  in the form of {(id(layer1), id(layer2), attribute_name) -> callback}
 LinkKey = Tuple['ReferenceType[Layer]', 'ReferenceType[Layer]', str]
 Unlinker = Callable[[], None]
-_UNLINKERS: dict[LinkKey, Unlinker] = dict()
+_UNLINKERS: dict[LinkKey, Unlinker] = {}
 _LINKED_LAYERS: DefaultDict[
     ReferenceType[Layer], Set[ReferenceType[Layer]]
 ] = DefaultDict(set)
@@ -83,7 +84,7 @@ def link_layers(
     >>> link_layers(viewer.layers)  # doctest: +SKIP
     """
 
-    from ...utils.misc import pick_equality_operator
+    from napari.utils.misc import pick_equality_operator
 
     valid_attrs = _get_common_evented_attributes(layers)
 
@@ -107,7 +108,6 @@ def link_layers(
     # now, connect requested attributes between all requested layers.
     links = []
     for (lay1, lay2), attribute in product(permutations(layers, 2), attr_set):
-
         key = _link_key(lay1, lay2, attribute)
         # if the layers and attribute are already linked then ignore
         if key in _UNLINKERS:
@@ -187,7 +187,9 @@ def layers_linked(layers: Iterable[Layer], attributes: Iterable[str] = ()):
 
 def _get_common_evented_attributes(
     layers: Iterable[Layer],
-    exclude: set[str] = {'thumbnail', 'status', 'name', 'data'},
+    exclude: set[str] = frozenset(
+        ('thumbnail', 'status', 'name', 'data', 'extent')
+    ),
     with_private=False,
 ) -> set[str]:
     """Get the set of common, non-private evented attributes in ``layers``.
@@ -221,9 +223,17 @@ def _get_common_evented_attributes(
                 "``layers`` iterable must have at least one layer",
                 deferred=True,
             )
-        )
+        ) from None
 
-    common_events = set.intersection(*(set(lay.events) for lay in layers))
+    layer_events = [
+        {
+            e
+            for e in lay.events
+            if not isinstance(lay.events[e], WarningEmitter)
+        }
+        for lay in layers
+    ]
+    common_events = set.intersection(*layer_events)
     common_attrs = set.intersection(*(set(dir(lay)) for lay in layers))
     if not with_private:
         common_attrs = {x for x in common_attrs if not x.startswith("_")}
@@ -255,6 +265,6 @@ def _unlink_keys(keys: Iterable[LinkKey]):
 
 def _rebuild_link_index():
     links = DefaultDict(set)
-    for l1, l2, attr in _UNLINKERS:
+    for l1, l2, _attr in _UNLINKERS:
         links[l1].add(l2)
     return links
