@@ -13,8 +13,10 @@ from napari.layers.utils.layer_utils import (
     dims_displayed_world_to_layer,
     get_current_properties,
     prepare_properties,
+    register_layer_attr_action,
     segment_normal,
 )
+from napari.utils.key_bindings import KeymapHandler, KeymapProvider
 
 data_dask = da.random.random(
     size=(100_000, 1000, 1000), chunks=(1, 1000, 1000)
@@ -222,7 +224,7 @@ def test_coerce_current_properties_valid_values():
     }
     coerced_current_properties = coerce_current_properties(current_properties)
 
-    for k, v in coerced_current_properties.items():
+    for k in coerced_current_properties:
         value = coerced_current_properties[k]
         assert isinstance(value, np.ndarray)
         np.testing.assert_equal(value, expected_current_properties[k])
@@ -273,6 +275,16 @@ def test_feature_table_from_layer_with_num_data_only():
     assert feature_table.defaults.shape == (1, 0)
 
 
+def test_feature_table_from_layer_with_empty_int_features():
+    feature_table = _FeatureTable.from_layer(
+        features={'a': np.empty(0, dtype=np.int64)}
+    )
+    assert feature_table.values['a'].dtype == np.int64
+    assert len(feature_table.values['a']) == 0
+    assert feature_table.defaults['a'].dtype == np.int64
+    assert feature_table.defaults['a'][0] == 0
+
+
 def test_feature_table_from_layer_with_properties_and_num_data():
     properties = {
         'class': np.array(['sky', 'person', 'building', 'person']),
@@ -381,13 +393,12 @@ def test_feature_table_from_layer_with_properties_as_dataframe():
     pd.testing.assert_frame_equal(feature_table.values, TEST_FEATURES)
 
 
-def _make_feature_table():
+@pytest.fixture
+def feature_table():
     return _FeatureTable(TEST_FEATURES.copy(deep=True), num_data=4)
 
 
-def test_feature_table_resize_smaller():
-    feature_table = _make_feature_table()
-
+def test_feature_table_resize_smaller(feature_table: _FeatureTable):
     feature_table.resize(2)
 
     features = feature_table.values
@@ -396,8 +407,8 @@ def test_feature_table_resize_smaller():
     np.testing.assert_array_equal(features['confidence'], [0.2, 0.5])
 
 
-def test_feature_table_resize_larger():
-    feature_table = _make_feature_table()
+def test_feature_table_resize_larger(feature_table: _FeatureTable):
+    expected_dtypes = feature_table.values.dtypes
 
     feature_table.resize(6)
 
@@ -411,10 +422,10 @@ def test_feature_table_resize_larger():
         features['confidence'],
         [0.2, 0.5, 1, 0.8, 0.8, 0.8],
     )
+    np.testing.assert_array_equal(features.dtypes, expected_dtypes)
 
 
-def test_feature_table_append():
-    feature_table = _make_feature_table()
+def test_feature_table_append(feature_table: _FeatureTable):
     to_append = pd.DataFrame(
         {
             'class': ['sky', 'building'],
@@ -436,9 +447,7 @@ def test_feature_table_append():
     )
 
 
-def test_feature_table_remove():
-    feature_table = _make_feature_table()
-
+def test_feature_table_remove(feature_table: _FeatureTable):
     feature_table.remove([1, 3])
 
     features = feature_table.values
@@ -459,3 +468,78 @@ def test_feature_table_from_layer_with_custom_index_and_num_data():
     feature_table = _FeatureTable.from_layer(features=features, num_data=2)
     expected = features.reset_index(drop=True)
     pd.testing.assert_frame_equal(feature_table.values, expected)
+
+
+def test_feature_table_from_layer_with_unordered_pd_series_properties():
+    properties = {
+        'a': pd.Series([1, 3], index=[3, 4]),
+        'b': pd.Series([7.5, -2.1], index=[1, 2]),
+    }
+    feature_table = _FeatureTable.from_layer(properties=properties, num_data=2)
+    expected = pd.DataFrame({'a': [1, 3], 'b': [7.5, -2.1]}, index=[0, 1])
+    pd.testing.assert_frame_equal(feature_table.values, expected)
+
+
+def test_feature_table_from_layer_with_unordered_pd_series_features():
+    features = {
+        'a': pd.Series([1, 3], index=[3, 4]),
+        'b': pd.Series([7.5, -2.1], index=[1, 2]),
+    }
+    feature_table = _FeatureTable.from_layer(features=features, num_data=2)
+    expected = pd.DataFrame({'a': [1, 3], 'b': [7.5, -2.1]}, index=[0, 1])
+    pd.testing.assert_frame_equal(feature_table.values, expected)
+
+
+def test_feature_table_set_defaults_with_same_columns(feature_table):
+    defaults = {'class': 'building', 'confidence': 1}
+    assert feature_table.defaults['class'][0] != defaults['class']
+    assert feature_table.defaults['confidence'][0] != defaults['confidence']
+
+    feature_table.set_defaults(defaults)
+
+    assert feature_table.defaults['class'][0] == defaults['class']
+    assert feature_table.defaults['confidence'][0] == defaults['confidence']
+
+
+def test_feature_table_set_defaults_with_extra_column(feature_table):
+    defaults = {'class': 'building', 'confidence': 0, 'cat': 'kermit'}
+    assert 'cat' not in feature_table.values.columns
+    with pytest.raises(ValueError):
+        feature_table.set_defaults(defaults)
+
+
+def test_feature_table_set_defaults_with_missing_column(feature_table):
+    defaults = {'class': 'building'}
+    assert len(feature_table.values.columns) > 1
+    with pytest.raises(ValueError):
+        feature_table.set_defaults(defaults)
+
+
+def test_register_label_attr_action(monkeypatch):
+    monkeypatch.setattr(time, "time", lambda: 1)
+
+    class Foo(KeymapProvider):
+        def __init__(self) -> None:
+            super().__init__()
+            self.value = 0
+
+    foo = Foo()
+
+    handler = KeymapHandler()
+    handler.keymap_providers = [foo]
+
+    @register_layer_attr_action(Foo, "value desc", "value", "K")
+    def set_value_1(x):
+        x.value = 1
+
+    handler.press_key("K")
+    assert foo.value == 1
+    handler.release_key("K")
+    assert foo.value == 1
+
+    foo.value = 0
+    handler.press_key("K")
+    assert foo.value == 1
+    monkeypatch.setattr(time, "time", lambda: 2)
+    handler.release_key("K")
+    assert foo.value == 0
