@@ -3,26 +3,26 @@ from dataclasses import dataclass
 from typing import Any, Dict, Optional, Tuple, Union
 
 import numpy as np
-from pydantic import root_validator, validator
+from pydantic import Field, root_validator, validator
 
-from ...utils.colormaps import Colormap
-from ...utils.colormaps.categorical_colormap import CategoricalColormap
-from ...utils.colormaps.colormap_utils import ColorType, ensure_colormap
-from ...utils.events import EventedModel
-from ...utils.events.custom_types import Array
-from ...utils.translations import trans
-from ._color_manager_constants import ColorMode
-from .color_manager_utils import (
+from napari.layers.utils._color_manager_constants import ColorMode
+from napari.layers.utils.color_manager_utils import (
     _validate_colormap_mode,
     _validate_cycle_mode,
     guess_continuous,
     is_color_mapped,
 )
-from .color_transformations import (
+from napari.layers.utils.color_transformations import (
     normalize_and_broadcast_colors,
     transform_color,
     transform_color_with_defaults,
 )
+from napari.utils.colormaps import Colormap
+from napari.utils.colormaps.categorical_colormap import CategoricalColormap
+from napari.utils.colormaps.colormap_utils import ColorType, ensure_colormap
+from napari.utils.events import EventedModel
+from napari.utils.events.custom_types import Array
+from napari.utils.translations import trans
 
 
 @dataclass
@@ -60,13 +60,13 @@ class ColorProperties:
                     # ensure the values are a numpy array
                     val['values'] = np.asarray(val['values'])
                     color_properties = cls(**val)
-                except ValueError:
+                except ValueError as e:
                     raise ValueError(
                         trans._(
                             'color_properties dictionary should have keys: name, values, and optionally current_value',
                             deferred=True,
                         )
-                    )
+                    ) from e
 
         elif isinstance(val, cls):
             color_properties = val
@@ -96,8 +96,8 @@ class ColorProperties:
             )
 
             return np.all([name_eq, values_eq, current_value_eq])
-        else:
-            return False
+
+        return False
 
 
 class ColorManager(EventedModel):
@@ -146,32 +146,35 @@ class ColorManager(EventedModel):
     current_color: Optional[Array[float, (4,)]] = None
     color_mode: ColorMode = ColorMode.DIRECT
     color_properties: Optional[ColorProperties] = None
-    continuous_colormap: Colormap = 'viridis'
+    continuous_colormap: Colormap = ensure_colormap('viridis')
     contrast_limits: Optional[Tuple[float, float]] = None
-    categorical_colormap: CategoricalColormap = [0, 0, 0, 1]
-    colors: Array[float, (-1, 4)] = []
+    categorical_colormap: CategoricalColormap = CategoricalColormap.from_array(
+        [0, 0, 0, 1]
+    )
+    colors: Array[float, (-1, 4)] = Field(
+        default_factory=lambda: np.empty((0, 4))
+    )
 
     # validators
     @validator('continuous_colormap', pre=True)
     def _ensure_continuous_colormap(cls, v):
-        coerced_colormap = ensure_colormap(v)
-        return coerced_colormap
+        return ensure_colormap(v)
 
     @validator('colors', pre=True)
-    def _ensure_color_array(cls, v, values):
+    def _ensure_color_array(cls, v):
         if len(v) > 0:
             return transform_color(v)
-        else:
-            return np.empty((0, 4))
+
+        return np.empty((0, 4))
 
     @validator('current_color', pre=True)
     def _coerce_current_color(cls, v):
         if v is None:
             return v
-        elif len(v) == 0:
+        if len(v) == 0:
             return None
-        else:
-            return transform_color(v)[0]
+
+        return transform_color(v)[0]
 
     @root_validator()
     def _validate_colors(cls, values):
@@ -180,14 +183,12 @@ class ColorManager(EventedModel):
             colors, values = _validate_cycle_mode(values)
         elif color_mode == ColorMode.COLORMAP:
             colors, values = _validate_colormap_mode(values)
-        elif color_mode == ColorMode.DIRECT:
+        else:  # color_mode == ColorMode.DIRECT:
             colors = values['colors']
 
-        # FIXME Local variable 'colors' might be referenced before assignment
-
         # set the current color to the last color/property value
         # if it wasn't already set
-        if values['current_color'] is None and len(colors) > 0:
+        if values.get("current_color") is None and len(colors) > 0:
             values['current_color'] = colors[-1]
             if color_mode in [ColorMode.CYCLE, ColorMode.COLORMAP]:
                 property_values = values['color_properties']
@@ -298,10 +299,7 @@ class ColorManager(EventedModel):
             (i.e., reset the range to 0-new_max_value).
         """
         if self.color_mode == ColorMode.DIRECT:
-            if color is None:
-                new_color = self.current_color
-            else:
-                new_color = color
+            new_color = self.current_color if color is None else color
             transformed_color = transform_color_with_defaults(
                 num_entries=n_colors,
                 colors=new_color,
@@ -424,7 +422,7 @@ class ColorManager(EventedModel):
                     )
 
     def _update_current_color(
-        self, current_color: np.ndarray, update_indices: list = []
+        self, current_color: np.ndarray, update_indices: list = ()
     ):
         """Update the current color and update the colors if requested.
 
@@ -441,7 +439,7 @@ class ColorManager(EventedModel):
             will change the mode to DIRECT.
         """
         self.current_color = transform_color(current_color)[0]
-        if len(update_indices) > 0:
+        if update_indices:
             self.color_mode = ColorMode.DIRECT
             cur_colors = self.colors.copy()
             cur_colors[update_indices] = self.current_color
@@ -460,12 +458,15 @@ class ColorManager(EventedModel):
         ] = None,
         color_mode: Optional[Union[ColorMode, str]] = None,
         current_color: Optional[np.ndarray] = None,
-        default_color_cycle: np.ndarray = np.array([1, 1, 1, 1]),
+        default_color_cycle: ColorType = None,
     ):
         """Initialize a ColorManager object from layer kwargs. This is a convenience
         function to coerce possible inputs into ColorManager kwargs
 
         """
+        if default_color_cycle is None:
+            default_color_cycle = np.array([1, 1, 1, 1])
+
         properties = {k: np.asarray(v) for k, v in properties.items()}
         if isinstance(colors, dict):
             # if the kwargs are passed as a dictionary, unpack them
@@ -490,13 +491,13 @@ class ColorManager(EventedModel):
                     color_properties = ColorProperties(
                         name=prop_name, values=prop_values
                     )
-                except KeyError:
+                except KeyError as e:
                     raise KeyError(
                         trans._(
                             'if color_properties is a string, it should be a property name',
                             deferred=True,
                         )
-                    )
+                    ) from e
         else:
             color_values = colors
             color_properties = None
