@@ -3,7 +3,10 @@ import pandas as pd
 import pytest
 from vispy.color import get_colormap
 
-from napari._tests.utils import check_layer_world_data_extent
+from napari._tests.utils import (
+    assert_colors_equal,
+    check_layer_world_data_extent,
+)
 from napari.layers import Vectors
 from napari.utils.colormaps.standardize_color import transform_color
 
@@ -65,6 +68,27 @@ def test_empty_vectors():
     assert layer._view_data.shape[2] == 2
 
 
+def test_empty_vectors_with_features():
+    """See the following for the points issues this covers:
+    https://github.com/napari/napari/issues/5632
+    https://github.com/napari/napari/issues/5634
+    """
+    vectors = Vectors(
+        features={'a': np.empty(0, int)},
+        feature_defaults={'a': 0},
+        edge_color='a',
+        edge_color_cycle=list('rgb'),
+    )
+
+    vectors.data = np.concatenate((vectors.data, [[[0, 0], [1, 1]]]))
+    vectors.feature_defaults['a'] = 1
+    vectors.data = np.concatenate((vectors.data, [[[1, 1], [2, 2]]]))
+    vectors.feature_defaults = {'a': 2}
+    vectors.data = np.concatenate((vectors.data, [[[2, 2], [3, 3]]]))
+
+    assert_colors_equal(vectors.edge_color, list('rgb'))
+
+
 def test_empty_vectors_with_property_choices():
     """Test instantiating Vectors layer with empty coordinate-like 2D data."""
     shape = (0, 2, 2)
@@ -475,7 +499,6 @@ def test_edge_color_map_non_numeric_property():
         layer.edge_color_mode = 'colormap'
 
 
-@pytest.mark.filterwarnings("ignore:elementwise comparis:FutureWarning:numpy")
 def test_switching_edge_color_mode():
     """Test transitioning between all color modes"""
     np.random.seed(0)
@@ -625,7 +648,7 @@ def test_message():
     data[:, 0, :] = 20 * data[:, 0, :]
     layer = Vectors(data)
     msg = layer.get_status((0,) * 2)
-    assert type(msg) == str
+    assert type(msg) == dict
 
 
 def test_world_data_extent():
@@ -637,3 +660,36 @@ def test_world_data_extent():
     layer = Vectors(np.array(data))
     extent = np.array((min_val, max_val))
     check_layer_world_data_extent(layer, extent, (3, 1, 1), (10, 20, 5), False)
+
+
+def test_out_of_slice_display():
+    """Test setting out_of_slice_display flag for 2D and 4D data."""
+    shape = (10, 2, 2)
+    np.random.seed(0)
+    data = 20 * np.random.random(shape)
+    layer = Vectors(data)
+    assert layer.out_of_slice_display is False
+
+    layer.out_of_slice_display = True
+    assert layer.out_of_slice_display is True
+
+    layer = Vectors(data, out_of_slice_display=True)
+    assert layer.out_of_slice_display is True
+
+    shape = (10, 2, 4)
+    data = 20 * np.random.random(shape)
+    layer = Vectors(data)
+    assert layer.out_of_slice_display is False
+
+    layer.out_of_slice_display = True
+    assert layer.out_of_slice_display is True
+
+    layer = Vectors(data, out_of_slice_display=True)
+    assert layer.out_of_slice_display is True
+
+
+def test_empty_data_from_tuple():
+    """Test that empty data raises an error."""
+    layer = Vectors(name="vector", ndim=3)
+    layer2 = Vectors.create(*layer.as_layer_data_tuple())
+    assert layer2.data.size == 0
