@@ -8,34 +8,19 @@ from napari_plugin_engine import napari_hook_implementation
 if TYPE_CHECKING:
     from napari.plugins._plugin_manager import NapariPluginManager
 
-from napari._tests.utils import slow
 
-
-@slow(15)
 def test_plugin_discovery_is_delayed():
     """Test that plugins are not getting discovered at napari import time."""
     cmd = [
         sys.executable,
         '-c',
         'import sys; from napari.plugins import plugin_manager; '
-        'sys.exit(int("svg" in plugin_manager.plugins))',
+        'sys.exit(len(plugin_manager.plugins) > 2)',  # we have 2 'builtins'
     ]
     # will fail if plugin discovery happened at import
     proc = subprocess.run(cmd, capture_output=True)
     assert not proc.returncode, 'Plugins were discovered at import time!'
 
-    cmd = [
-        sys.executable,
-        '-c',
-        'import sys; '
-        'from napari.plugins import plugin_manager; '
-        'plugin_manager.discover(); '
-        'sys.exit(not int("svg" in plugin_manager.plugins))',
-    ]
-    # will fail if napari-svg is not in the environment and test needs fixing
-    proc = subprocess.run(cmd, capture_output=True)
-    assert not proc.returncode, 'napari-svg unavailable, this test is broken!'
-
 
 def test_plugin_events(napari_plugin_manager):
     """Test event emission by plugin manager."""
@@ -93,11 +78,13 @@ def test_plugin_extension_assignment(napari_plugin_manager):
         def napari_get_reader(path):
             if path.endswith('.png'):
                 return lambda x: None
+            return None
 
         @napari_hook_implementation
         def napari_get_writer(path, *args):
             if path.endswith('.png'):
                 return lambda x: None
+            return None
 
     tnpm: NapariPluginManager = napari_plugin_manager
     tnpm.register(Plugin, name='test_plugin')
