@@ -1,7 +1,12 @@
 from typing import List, Optional
 
-from ..utils.translations import trans
-from ._icons import ICON_PATH, ICONS, get_colorized_svg, get_icon_path
+from napari.resources._icons import (
+    ICON_PATH,
+    ICONS,
+    get_colorized_svg,
+    get_icon_path,
+)
+from napari.utils.translations import trans
 
 __all__ = ['get_colorized_svg', 'get_icon_path', 'ICON_PATH', 'ICONS']
 
@@ -18,6 +23,6 @@ def get_stylesheet(extra: Optional[List[str]] = None) -> str:
         category=DeprecationWarning,
         stacklevel=2,
     )
-    from .._qt.qt_resources import get_stylesheet as qt_get_stylesheet
+    from napari._qt.qt_resources import get_stylesheet as qt_get_stylesheet
 
     return qt_get_stylesheet(extra)
