@@ -1,14 +1,14 @@
 from pydantic import Field
 
-from ..utils.events.evented_model import EventedModel
-from ..utils.theme import available_themes
-from ..utils.translations import trans
-from ._fields import Theme
+from napari.settings._fields import Theme
+from napari.utils.events.evented_model import EventedModel
+from napari.utils.theme import available_themes
+from napari.utils.translations import trans
 
 
 class AppearanceSettings(EventedModel):
     theme: Theme = Field(
-        "dark",
+        Theme("dark"),
         title=trans._("Theme"),
         description=trans._("Select the user interface theme."),
         env="napari_theme",
