@@ -2,22 +2,43 @@ from __future__ import annotations
 
 from typing import List, Optional, Tuple
 
+from psutil import virtual_memory
 from pydantic import Field, validator
 
-from ..utils._base import _DEFAULT_LOCALE
-from ..utils.events.custom_types import conint
-from ..utils.events.evented_model import EventedModel
-from ..utils.notifications import NotificationSeverity
-from ..utils.translations import trans
-from ._constants import LoopMode
-from ._fields import Language
+from napari.settings._constants import LoopMode
+from napari.settings._fields import Language
+from napari.utils._base import _DEFAULT_LOCALE
+from napari.utils.events.custom_types import conint
+from napari.utils.events.evented_model import EventedModel
+from napari.utils.notifications import NotificationSeverity
+from napari.utils.translations import trans
 
 GridStride = conint(ge=-50, le=50, ne=0)
 GridWidth = conint(ge=-1, ne=0)
 GridHeight = conint(ge=-1, ne=0)
 
+_DEFAULT_MEM_FRACTION = 0.25
+MAX_CACHE = virtual_memory().total * 0.5 / 1e9
+
+
+class DaskSettings(EventedModel):
+    enabled: bool = True
+    cache: float = Field(
+        virtual_memory().total * _DEFAULT_MEM_FRACTION / 1e9,
+        ge=0,
+        le=MAX_CACHE,
+        title="Cache size (GB)",
+    )
+
 
 class ApplicationSettings(EventedModel):
+    def __init__(self, **kwargs):
+        super().__init__(**kwargs)
+        self.dask.events.connect(self._dask_changed)
+
+    def _dask_changed(self):
+        self.events.dask(value=self.dask)
+
     first_time: bool = Field(
         True,
         title=trans._('First time'),
@@ -33,7 +54,7 @@ class ApplicationSettings(EventedModel):
         ),
     )
     language: Language = Field(
-        _DEFAULT_LOCALE,
+        Language(_DEFAULT_LOCALE),
         title=trans._("Language"),
         description=trans._(
             "Select the display language for the user interface."
@@ -169,6 +190,14 @@ class ApplicationSettings(EventedModel):
             "This affects certain actions where a short press and a long press have different behaviors, such as changing the mode of a layer permanently or only during the long press."
         ),
     )
+    # convert cache (and max cache) from bytes to mb for widget
+    dask: DaskSettings = Field(
+        default=DaskSettings(),
+        title=trans._("Dask cache"),
+        description=trans._(
+            "Settings for dask cache (does not work with distributed arrays)"
+        ),
+    )
 
     @validator('window_state')
     def _validate_qbtye(cls, v):
