@@ -1,12 +1,10 @@
-from enum import Enum
 from typing import Dict, List, Set
 
 from pydantic import Field
 from typing_extensions import TypedDict
 
-from ..utils.events.evented_model import EventedModel
-from ..utils.misc import running_as_bundled_app, running_as_constructor_app
-from ..utils.translations import trans
+from napari.settings._base import EventedSettings
+from napari.utils.translations import trans
 
 
 class PluginHookOption(TypedDict):
@@ -19,19 +17,16 @@ class PluginHookOption(TypedDict):
 CallOrderDict = Dict[str, List[PluginHookOption]]
 
 
-class PluginAPI(str, Enum):
-    napari_hub = 'napari hub'
-    pypi = 'PyPI'
-
-
-class PluginsSettings(EventedModel):
-    plugin_api: PluginAPI = Field(
-        PluginAPI.napari_hub,
-        title=trans._("Plugin API"),
+class PluginsSettings(EventedSettings):
+    use_npe2_adaptor: bool = Field(
+        False,
+        title=trans._("Use npe2 adaptor"),
         description=trans._(
-            "Use the following API for querying plugin information.",
+            "Use npe2-adaptor for first generation plugins.\nWhen an npe1 plugin is found, this option will\nimport its contributions and create/cache\na 'shim' npe2 manifest that allows it to be treated\nlike an npe2 plugin (with delayed imports, etc...)",
         ),
+        requires_restart=True,
     )
+
     call_order: CallOrderDict = Field(
         default_factory=dict,
         title=trans._("Plugin sort order"),
@@ -71,6 +66,3 @@ class PluginsSettings(EventedModel):
             'disabled_plugins',
             'extension2writer',
         ]
-
-        if running_as_bundled_app() or running_as_constructor_app():
-            preferences_exclude.append('plugin_api')
