@@ -1,3 +1,5 @@
+from typing import Tuple, Union
+
 import numpy as np
 
 _np_uints = {
@@ -15,6 +17,7 @@ _np_ints = {
 }
 
 _np_floats = {
+    16: np.float16,
     32: np.float32,
     64: np.float64,
 }
@@ -46,6 +49,7 @@ def _normalize_str_by_bit_depth(dtype_str, kind):
         return bit_dict[32]
     if '64' in dtype_str:
         return bit_dict[64]
+    return None
 
 
 def normalize_dtype(dtype_spec):
@@ -77,3 +81,33 @@ def normalize_dtype(dtype_spec):
         return _normalize_str_by_bit_depth(dtype_str, 'complex')
     if 'bool' in dtype_str:
         return np.bool_
+    # If we don't find one of the named dtypes, return the dtype_spec
+    # unchanged. This allows NumPy big endian types to work. See
+    # https://github.com/napari/napari/issues/3421
+
+    return dtype_spec
+
+
+def get_dtype_limits(dtype_spec) -> Tuple[float, float]:
+    """Return machine limits for numeric types.
+
+    Parameters
+    ----------
+    dtype_spec : numpy dtype, numpy type, torch dtype, tensorstore dtype, etc
+        A type that can be interpreted as a NumPy numeric data type, e.g.
+        'uint32', np.uint8, torch.float32, etc.
+
+    Returns
+    -------
+    limits : tuple
+        The smallest/largest numbers expressible by the type.
+    """
+    dtype = normalize_dtype(dtype_spec)
+    info: Union[np.iinfo, np.finfo]
+    if np.issubdtype(dtype, np.integer):
+        info = np.iinfo(dtype)
+    elif dtype and np.issubdtype(dtype, np.floating):
+        info = np.finfo(dtype)
+    else:
+        raise TypeError(f'Unrecognized or non-numeric dtype: {dtype_spec}')
+    return info.min, info.max
