@@ -16,6 +16,10 @@ if TYPE_CHECKING:
 _SAVE_GRAPH_OPNAME = "--save-leaked-object-graph"
 
 
+def _empty(*_, **__):
+    """Empty function for mocking"""
+
+
 def pytest_addoption(parser):
     parser.addoption(
         "--show-napari-viewer",
@@ -43,10 +47,10 @@ def fail_obj_graph(Klass):
 
     try:
         import objgraph
-    except ImportError:
+    except ModuleNotFoundError:
         return
 
-    if not len(Klass._instances) == 0:
+    if len(Klass._instances) != 0:
         global COUNTER
         COUNTER += 1
         import gc
@@ -74,8 +78,6 @@ def napari_plugin_manager(monkeypatch):
     Or, to re-enable global discovery, use:
     `napari_plugin_manager.discovery_blocker.stop()`
     """
-    from unittest.mock import patch
-
     import napari
     import napari.plugins.io
     from napari.plugins._plugin_manager import NapariPluginManager
@@ -86,11 +88,8 @@ def napari_plugin_manager(monkeypatch):
     # get this test version for the duration of the test.
     monkeypatch.setattr(napari.plugins, 'plugin_manager', pm)
     monkeypatch.setattr(napari.plugins.io, 'plugin_manager', pm)
-    try:
+    with suppress(AttributeError):
         monkeypatch.setattr(napari._qt.qt_main_window, 'plugin_manager', pm)
-    except AttributeError:  # headless tests
-        pass
-
     # prevent discovery of plugins in the environment
     # you can still use `pm.register` to explicitly register something.
     pm.discovery_blocker = patch.object(pm, 'discover')
@@ -100,9 +99,40 @@ def napari_plugin_manager(monkeypatch):
     pm.discovery_blocker.stop()
 
 
+GCPASS = 0
+
+
+@pytest.fixture(autouse=True)
+def clean_themes():
+    from napari.utils import theme
+
+    themes = set(theme.available_themes())
+    yield
+    for name in theme.available_themes():
+        if name not in themes:
+            del theme._themes[name]
+
+
+@pytest.hookimpl(tryfirst=True, hookwrapper=True)
+def pytest_runtest_makereport(item, call):
+    # https://docs.pytest.org/en/latest/example/simple.html#making-test-result-information-available-in-fixtures
+    # execute all other hooks to obtain the report object
+    outcome = yield
+    rep = outcome.get_result()
+
+    # set a report attribute for each phase of a call, which can
+    # be "setup", "call", "teardown"
+
+    setattr(item, f"rep_{rep.when}", rep)
+
+
 @pytest.fixture
 def make_napari_viewer(
-    qtbot, request: 'FixtureRequest', napari_plugin_manager
+    qtbot,
+    request: 'FixtureRequest',
+    napari_plugin_manager,
+    monkeypatch,
+    clean_themes,
 ):
     """A fixture function that creates a napari viewer for use in testing.
 
@@ -149,7 +179,13 @@ def make_napari_viewer(
     from napari._qt.qt_viewer import QtViewer
     from napari.settings import get_settings
 
-    gc.collect()
+    global GCPASS
+    GCPASS += 1
+
+    if GCPASS % 50 == 0:
+        gc.collect()
+    else:
+        gc.collect(1)
 
     _do_not_inline_below = len(QtViewer._instances)
     # # do not inline to avoid pytest trying to compute repr of expression.
@@ -176,13 +212,21 @@ def make_napari_viewer(
     prior_exception = getattr(sys, 'last_value', None)
     is_internal_test = request.module.__name__.startswith("napari.")
 
+    # disable throttling cursor event in tests
+    monkeypatch.setattr(
+        "napari._qt.qt_main_window._QtMainWindow._throttle_cursor_to_status_connection",
+        _empty,
+    )
+
     def actual_factory(
         *model_args,
         ViewerClass=Viewer,
-        strict_qt=is_internal_test or os.getenv("NAPARI_STRICT_QT"),
+        strict_qt=None,
         block_plugin_discovery=True,
         **model_kwargs,
     ):
+        if strict_qt is None:
+            strict_qt = is_internal_test or os.getenv("NAPARI_STRICT_QT")
         nonlocal _strict
         _strict = strict_qt
 
@@ -213,12 +257,22 @@ def make_napari_viewer(
         else:
             viewer.close()
 
-    gc.collect()
+    if GCPASS % 50 == 0 or len(QtViewer._instances):
+        gc.collect()
+    else:
+        gc.collect(1)
 
     if request.config.getoption(_SAVE_GRAPH_OPNAME):
         fail_obj_graph(QtViewer)
 
+    if request.node.rep_call.failed:
+        # IF test failed do not check for leaks
+        QtViewer._instances.clear()
+
     _do_not_inline_below = len(QtViewer._instances)
+
+    QtViewer._instances.clear()  # clear to prevent fail of next test
+
     # do not inline to avoid pytest trying to compute repr of expression.
     # it fails if C++ object gone but not Python object.
     assert _do_not_inline_below == 0
@@ -230,7 +284,7 @@ def make_napari_viewer(
         leak = set(QApplication.topLevelWidgets()).difference(initial)
         # still not sure how to clean up some of the remaining vispy
         # vispy.app.backends._qt.CanvasBackendDesktop widgets...
-        if any([n.__class__.__name__ != 'CanvasBackendDesktop' for n in leak]):
+        if any(n.__class__.__name__ != 'CanvasBackendDesktop' for n in leak):
             # just a warning... but this can be converted to test errors
             # in pytest with `-W error`
             msg = f"""The following Widgets leaked!: {leak}.
@@ -245,12 +299,44 @@ def make_napari_viewer(
             # in particular with VisPyCanvas, it looks like if a traceback keeps
             # contains the type, then instances are still attached to the type.
             # I'm not too sure why this is the case though.
-            if _strict == 'raise':
+            if _strict:
                 raise AssertionError(msg)
             else:
                 warnings.warn(msg)
 
 
+@pytest.fixture
+def make_napari_viewer_proxy(make_napari_viewer, monkeypatch):
+    """Fixture that returns a function for creating a napari viewer wrapped in proxy.
+    Use in the same way like `make_napari_viewer` fixture.
+
+    Parameters
+    ----------
+    make_napari_viewer : fixture
+        The make_napari_viewer fixture.
+
+    Returns
+    -------
+    function
+        A function that creates a napari viewer.
+    """
+    from napari.utils._proxies import PublicOnlyProxy
+
+    proxies = []
+
+    def actual_factory(*model_args, ensure_main_thread=True, **model_kwargs):
+        monkeypatch.setenv(
+            "NAPARI_ENSURE_PLUGIN_MAIN_THREAD", str(ensure_main_thread)
+        )
+        viewer = make_napari_viewer(*model_args, **model_kwargs)
+        proxies.append(PublicOnlyProxy(viewer))
+        return proxies[-1]
+
+    proxies.clear()
+
+    yield actual_factory
+
+
 @pytest.fixture
 def MouseEvent():
     """Create a subclass for simulating vispy mouse events.
