@@ -96,3 +96,17 @@ def test_colormap_equality():
     cmap_3 = Colormap(colors, name='testing', controls=[0, 0.25, 1])
     assert cmap_1 == cmap_2
     assert cmap_1 != cmap_3
+
+
+def test_colormap_recreate():
+    c_map = Colormap("black")
+    Colormap(**c_map.dict())
+
+
+@pytest.mark.parametrize('ndim', range(1, 5))
+def test_mapped_shape(ndim):
+    np.random.seed(0)
+    img = np.random.random((5,) * ndim)
+    cmap = Colormap(colors=['red'])
+    mapped = cmap.map(img)
+    assert mapped.shape == img.shape + (4,)
