@@ -17,19 +17,19 @@ def make_colorbar(cmap, size=(18, 28), horizontal=True):
     Returns
     -------
     cbar : array
-        Array of colorbar.
+        Array of colorbar in uint8.
     """
 
     if horizontal:
-        input = np.linspace(0, 1, size[1])
-        bar = np.tile(np.expand_dims(input, 1), size[0]).transpose((1, 0))
+        basic_values = np.linspace(0, 1, size[1])
+        bar = np.tile(np.expand_dims(basic_values, 1), size[0]).transpose(
+            (1, 0)
+        )
     else:
-        input = np.linspace(0, 1, size[0])
-        bar = np.tile(np.expand_dims(input, 1), size[1])
+        basic_values = np.linspace(0, 1, size[0])
+        bar = np.tile(np.expand_dims(basic_values, 1), size[1])
 
-    # cmap.__getitem__ returns a vispy.color.ColorArray
-    color_array = cmap[bar.ravel()]
-    # the ColorArray.RGBA method returns a normalized uint8 array
-    cbar = color_array.RGBA.reshape(bar.shape + (4,))
+    color_array = cmap.map(bar.ravel())
+    cbar = color_array.reshape((*bar.shape, 4))
 
-    return cbar
+    return np.round(255 * cbar).astype(np.uint8).copy(order='C')
