@@ -4,11 +4,11 @@ from typing import Optional
 import numpy as np
 from pydantic import PrivateAttr, validator
 
-from ..events import EventedModel
-from ..events.custom_types import Array
-from ..translations import trans
-from .colorbars import make_colorbar
-from .standardize_color import transform_color
+from napari.utils.color import ColorArray
+from napari.utils.colormaps.colorbars import make_colorbar
+from napari.utils.events import EventedModel
+from napari.utils.events.custom_types import Array
+from napari.utils.translations import trans
 
 
 class ColormapInterpolationMode(str, Enum):
@@ -46,24 +46,21 @@ class Colormap(EventedModel):
     """
 
     # fields
-    colors: Array[float, (-1, 4)]
+    colors: ColorArray
     name: str = 'custom'
     _display_name: Optional[str] = PrivateAttr(None)
     interpolation: ColormapInterpolationMode = ColormapInterpolationMode.LINEAR
-    controls: Array[float, (-1,)] = None
+    controls: Array[np.float32, (-1,)] = None
 
-    def __init__(self, colors, display_name: Optional[str] = None, **data):
+    def __init__(
+        self, colors, display_name: Optional[str] = None, **data
+    ) -> None:
         if display_name is None:
             display_name = data.get('name', 'custom')
 
         super().__init__(colors=colors, **data)
         self._display_name = display_name
 
-    # validators
-    @validator('colors', pre=True)
-    def _ensure_color_array(cls, v):
-        return transform_color(v)
-
     # controls validator must be called even if None for correct initialization
     @validator('controls', pre=True, always=True)
     def _check_controls(cls, v, values):
@@ -72,7 +69,7 @@ class Colormap(EventedModel):
             n_controls = len(values['colors']) + int(
                 values['interpolation'] == ColormapInterpolationMode.ZERO
             )
-            return np.linspace(0, 1, n_controls)
+            return np.linspace(0, 1, n_controls, dtype=np.float32)
 
         # Check control end points are correct
         if v[0] != 0 or (len(v) > 1 and v[-1] != 1):
@@ -122,11 +119,14 @@ class Colormap(EventedModel):
                 np.interp(values, self.controls, self.colors[:, i])
                 for i in range(4)
             ]
-            cols = np.stack(cols, axis=1)
+            cols = np.stack(cols, axis=-1)
         elif self.interpolation == ColormapInterpolationMode.ZERO:
             # One color per bin
+            # Colors beyond max clipped to final bin
             indices = np.clip(
-                np.searchsorted(self.controls, values) - 1, 0, len(self.colors)
+                np.searchsorted(self.controls, values, side="right") - 1,
+                0,
+                len(self.colors) - 1,
             )
             cols = self.colors[indices.astype(np.int32)]
         else:
