@@ -13,7 +13,7 @@ from typing import (
     overload,
 )
 
-from ....utils.translations import trans
+from napari.utils.translations import trans
 
 logger = logging.getLogger(__name__)
 
@@ -24,11 +24,6 @@ _T = TypeVar("_T")
 _L = TypeVar("_L")
 
 
-class B(MutableSequence[int]):
-    def __new__(self, data):
-        self._data = data
-
-
 class TypedMutableSequence(MutableSequence[_T]):
     """List mixin that enforces item type, and enables custom indexing.
 
@@ -54,8 +49,10 @@ class TypedMutableSequence(MutableSequence[_T]):
         data: Iterable[_T] = (),
         *,
         basetype: Union[Type[_T], Sequence[Type[_T]]] = (),
-        lookup: Dict[Type[_L], Callable[[_T], Union[_T, _L]]] = dict(),
-    ):
+        lookup: Dict[Type[_L], Callable[[_T], Union[_T, _L]]] = None,
+    ) -> None:
+        if lookup is None:
+            lookup = {}
         self._list: List[_T] = []
         self._basetypes = (
             basetype if isinstance(basetype, Sequence) else (basetype,)
@@ -78,14 +75,14 @@ class TypedMutableSequence(MutableSequence[_T]):
         return id(self)
 
     @overload
-    def __setitem__(self, key: int, value: _T):  # noqa: F811
+    def __setitem__(self, key: int, value: _T):
         ...  # pragma: no cover
 
     @overload
-    def __setitem__(self, key: slice, value: Iterable[_T]):  # noqa: F811
+    def __setitem__(self, key: slice, value: Iterable[_T]):
         ...  # pragma: no cover
 
-    def __setitem__(self, key, value):  # noqa: F811
+    def __setitem__(self, key, value):
         if isinstance(key, slice):
             if not isinstance(value, Iterable):
                 raise TypeError(
@@ -112,14 +109,14 @@ class TypedMutableSequence(MutableSequence[_T]):
         return super().__contains__(key)
 
     @overload
-    def __getitem__(self, key: int) -> _T:  # noqa: F811
+    def __getitem__(self, key: int) -> _T:
         ...  # pragma: no cover
 
     @overload
-    def __getitem__(self, key: slice) -> 'TypedMutableSequence[_T]':  # noqa
+    def __getitem__(self, key: slice) -> 'TypedMutableSequence[_T]':
         ...  # pragma: no cover
 
-    def __getitem__(self, key):  # noqa: F811
+    def __getitem__(self, key):
         """Get an item from the list
 
         Parameters
@@ -224,13 +221,8 @@ class TypedMutableSequence(MutableSequence[_T]):
             stop += len(self)
 
         convert = self._lookup.get(type(value), _noop)
-        special_lookup = type(value) in self._lookup
-        # A "special lookup" means that they type of the value being searched
-        # is in the `self._lookups` dict.  The most common internal use of this
-        # pattern is `layers['name']`.  So we do a "deep" traversal in that
-        # case, which will let the nestable variant search throughout.
-        # we may or may not want that behavior?
-        for i in self._iter_indices(start, stop, deep=special_lookup):
+
+        for i in self._iter_indices(start, stop):
             v = convert(self[i])
             if v is value or v == value:
                 return i
@@ -243,7 +235,7 @@ class TypedMutableSequence(MutableSequence[_T]):
             )
         )
 
-    def _iter_indices(self, start=0, stop=None, deep=False):
+    def _iter_indices(self, start=0, stop=None):
         """Iter indices from start to stop.
 
         While this is trivial for this basic sequence type, this method lets
@@ -254,6 +246,7 @@ class TypedMutableSequence(MutableSequence[_T]):
     def _ipython_key_completions_(self):
         if str in self._lookup:
             return (self._lookup[str](x) for x in self)
+        return None  # type: ignore
 
 
 def _noop(x):
