@@ -4,9 +4,7 @@ import weakref
 from typing import TYPE_CHECKING
 
 if TYPE_CHECKING:
-    from typing import Callable
-
-    from typing_extensions import Protocol
+    from typing import Callable, Protocol
 
     class Emitter(Protocol):
         def connect(self, callback: Callable):
@@ -34,13 +32,19 @@ def connect_setattr(emitter: Emitter, obj, attr: str):
     ref = weakref.ref(obj)
 
     def _cb(*value):
-        setattr(ref(), attr, value[0] if len(value) == 1 else value)
+        if (ob := ref()) is None:
+            emitter.disconnect(_cb)
+            return
+        setattr(ob, attr, value[0] if len(value) == 1 else value)
 
     emitter.connect(_cb)
     # There are scenarios where emitter is deleted before obj.
     # Also there is no option to create weakref to QT Signal
     # but even if keep reference to base object and signal name it is possible to meet
     # problem with C++ "wrapped C/C++ object has been deleted"
+
+    # In all of these 3 functions, this should be uncommented instead of using
+    # the if clause in _cb but that causes a segmentation fault in tests
     # weakref.finalize(obj, emitter.disconnect, _cb)
 
 
@@ -48,7 +52,10 @@ def connect_no_arg(emitter: Emitter, obj, attr: str):
     ref = weakref.ref(obj)
 
     def _cb(*_value):
-        getattr(ref(), attr)()
+        if (ob := ref()) is None:
+            emitter.disconnect(_cb)
+            return
+        getattr(ob, attr)()
 
     emitter.connect(_cb)
     # as in connect_setattr
@@ -60,6 +67,10 @@ def connect_setattr_value(emitter: Emitter, obj, attr: str):
     ref = weakref.ref(obj)
 
     def _cb(value):
-        setattr(ref(), attr, value.value)
+        if (ob := ref()) is None:
+            emitter.disconnect(_cb)
+            return
+        setattr(ob, attr, value.value)
 
     emitter.connect(_cb)
+    # weakref.finalize(obj, emitter.disconnect, _cb)
