@@ -7,9 +7,9 @@ from typing import Any, Callable, ClassVar, Dict, Set, Union
 import numpy as np
 from pydantic import BaseModel, PrivateAttr, main, utils
 
-from ...utils.misc import pick_equality_operator
-from ..translations import trans
-from .event import EmitterGroup, Event
+from napari.utils.events.event import EmitterGroup, Event
+from napari.utils.misc import pick_equality_operator
+from napari.utils.translations import trans
 
 # encoders for non-napari specific field types.  To declare a custom encoder
 # for a napari type, add a `_json_encode` method to the class itself.
@@ -94,22 +94,59 @@ class EventedMetaclass(main.ModelMetaclass):
                 # required for pydantic>=1.8.0 due to:
                 # https://github.com/samuelcolvin/pydantic/pull/2064
                 EventedModel.__config__.json_encoders[f.type_] = encoder
-        # check for @_.setters defined on the class, so we can allow them
-        # in EventedModel.__setattr__
-        cls.__property_setters__ = {}
+        # check for properties defined on the class, so we can allow them
+        # in EventedModel.__setattr__ and create events
+        cls.__properties__ = {}
         for name, attr in namespace.items():
-            if isinstance(attr, property) and attr.fset is not None:
-                cls.__property_setters__[name] = attr
+            if isinstance(attr, property):
+                cls.__properties__[name] = attr
+                # determine compare operator
+                if (
+                    hasattr(attr.fget, "__annotations__")
+                    and "return" in attr.fget.__annotations__
+                    and not isinstance(
+                        attr.fget.__annotations__["return"], str
+                    )
+                ):
+                    cls.__eq_operators__[name] = pick_equality_operator(
+                        attr.fget.__annotations__["return"]
+                    )
+
         cls.__field_dependents__ = _get_field_dependents(cls)
         return cls
 
 
+def _update_dependents_from_property_code(
+    cls, prop_name, prop, deps, visited=()
+):
+    """Recursively find all the dependents of a property by inspecting the code object.
+
+    Update the given deps dictionary with the new findings.
+    """
+    for name in prop.fget.__code__.co_names:
+        if name in cls.__fields__:
+            deps.setdefault(name, set()).add(prop_name)
+        elif name in cls.__properties__ and name not in visited:
+            # to avoid infinite recursion, we shouldn't re-check getter we've already seen
+            visited = visited + (name,)
+            # sub_prop is the new property, but we leave prop_name the same
+            sub_prop = cls.__properties__[name]
+            _update_dependents_from_property_code(
+                cls, prop_name, sub_prop, deps, visited
+            )
+
+
 def _get_field_dependents(cls: 'EventedModel') -> Dict[str, Set[str]]:
     """Return mapping of field name -> dependent set of property names.
 
-    Dependencies may be declared in the Model Config to emit an event
-    for a computed property when a model field that it depends on changes
-    e.g.  (@property 'c' depends on model fields 'a' and 'b')
+    Dependencies will be guessed by inspecting the code of each property
+    in order to emit an event for a computed property when a model field
+    that it depends on changes (e.g: @property 'c' depends on model fields
+    'a' and 'b'). Alternatvely, dependencies may be declared excplicitly
+    in the Model Config.
+
+    Note: accessing a field with `getattr()` instead of dot notation won't
+    be automatically detected.
 
     Examples
     --------
@@ -125,33 +162,42 @@ def _get_field_dependents(cls: 'EventedModel') -> Dict[str, Set[str]]:
             def c(self, val: Sequence[int]):
                 self.a, self.b = val
 
+            @property
+            def d(self) -> int:
+                return sum(self.c)
+
+            @d.setter
+            def d(self, val: int):
+                self.c = [val // 2, val // 2]
+
             class Config:
-                dependencies={'c': ['a', 'b']}
+                dependencies={
+                    'c': ['a', 'b'],
+                    'd': ['a', 'b']
+                }
     """
-    if not cls.__property_setters__:
+    if not cls.__properties__:
         return {}
 
     deps: Dict[str, Set[str]] = {}
 
     _deps = getattr(cls.__config__, 'dependencies', None)
     if _deps:
-        for prop, fields in _deps.items():
-            if prop not in cls.__property_setters__:
+        for prop_name, fields in _deps.items():
+            if prop_name not in cls.__properties__:
                 raise ValueError(
-                    'Fields with dependencies must be property.setters. '
-                    f'{prop!r} is not.'
+                    'Fields with dependencies must be properties. '
+                    f'{prop_name!r} is not.'
                 )
             for field in fields:
                 if field not in cls.__fields__:
                     warnings.warn(f"Unrecognized field dependency: {field}")
-                deps.setdefault(field, set()).add(prop)
+                deps.setdefault(field, set()).add(prop_name)
     else:
         # if dependencies haven't been explicitly defined, we can glean
         # them from the property.fget code object:
-        for prop, setter in cls.__property_setters__.items():
-            for name in setter.fget.__code__.co_names:
-                if name in cls.__fields__:
-                    deps.setdefault(name, set()).add(prop)
+        for prop_name, prop in cls.__properties__.items():
+            _update_dependents_from_property_code(cls, prop_name, prop, deps)
     return deps
 
 
@@ -165,8 +211,8 @@ class EventedModel(BaseModel, metaclass=EventedMetaclass):
     # add private attributes for event emission
     _events: EmitterGroup = PrivateAttr(default_factory=EmitterGroup)
 
-    # mapping of name -> property obj for methods that are property setters
-    __property_setters__: ClassVar[Dict[str, property]]
+    # mapping of name -> property obj for methods that are properties
+    __properties__: ClassVar[Dict[str, property]]
     # mapping of field name -> dependent set of property names
     # when field is changed, an event for dependent properties will be emitted.
     __field_dependents__: ClassVar[Dict[str, Set[str]]]
@@ -192,7 +238,7 @@ class EventedModel(BaseModel, metaclass=EventedMetaclass):
         # field declares a _json_encode method.
         json_encoders = _BASE_JSON_ENCODERS
 
-    def __init__(self, **kwargs):
+    def __init__(self, **kwargs) -> None:
         super().__init__(**kwargs)
 
         self._events.source = self
@@ -204,7 +250,7 @@ class EventedModel(BaseModel, metaclass=EventedMetaclass):
         ]
 
         self._events.add(
-            **dict.fromkeys(field_events + list(self.__property_setters__))
+            **dict.fromkeys(field_events + list(self.__properties__))
         )
 
         # while seemingly redundant, this next line is very important to maintain
@@ -215,10 +261,14 @@ class EventedModel(BaseModel, metaclass=EventedMetaclass):
 
     def _super_setattr_(self, name: str, value: Any) -> None:
         # pydantic will raise a ValueError if extra fields are not allowed
-        # so we first check to see if this field has a property.setter.
+        # so we first check to see if this field is a property
         # if so, we use it instead.
-        if name in self.__property_setters__:
-            self.__property_setters__[name].fset(self, value)
+        if name in self.__properties__:
+            setter = self.__properties__[name].fset
+            if setter is None:
+                # raise same error as normal properties
+                raise AttributeError(f"can't set attribute '{name}'")
+            setter(self, value)
         else:
             super().__setattr__(name, value)
 
@@ -229,20 +279,60 @@ class EventedModel(BaseModel, metaclass=EventedMetaclass):
             return
 
         # grab current value
+        field_dep = self.__field_dependents__.get(name, {})
+        has_callbacks = {
+            name: bool(getattr(self.events, name).callbacks)
+            for name in field_dep
+        }
+        emitter = getattr(self.events, name)
+        # equality comparisons may be expensive, so just avoid them if
+        # event has no callbacks connected
+        if not (
+            emitter.callbacks
+            or self._events.callbacks
+            or any(has_callbacks.values())
+        ):
+            self._super_setattr_(name, value)
+            return
+
+        dep_with_callbacks = [
+            dep for dep, has_cb in has_callbacks.items() if has_cb
+        ]
+
         before = getattr(self, name, object())
+        before_deps = {}
+        with warnings.catch_warnings():
+            # we still need to check for deprecated properties
+            warnings.simplefilter("ignore", DeprecationWarning)
+            for dep in dep_with_callbacks:
+                before_deps[dep] = getattr(self, dep, object())
 
         # set value using original setter
         self._super_setattr_(name, value)
 
         # if different we emit the event with new value
         after = getattr(self, name)
-        are_equal = self.__eq_operators__.get(name, operator.eq)
-        if not are_equal(after, before):
-            getattr(self.events, name)(value=after)  # emit event
+        after_deps = {}
+        with warnings.catch_warnings():
+            # we still need to check for deprecated properties
+            warnings.simplefilter("ignore", DeprecationWarning)
+            for dep in dep_with_callbacks:
+                after_deps[dep] = getattr(self, dep, object())
 
-            # emit events for any dependent computed property setters as well
-            for dep in self.__field_dependents__.get(name, {}):
-                getattr(self.events, dep)(value=getattr(self, dep))
+        are_equal = self.__eq_operators__.get(name, operator.eq)
+        if are_equal(after, before):
+            # no change
+            return
+        emitter(value=after)  # emit event
+
+        # emit events for any dependent computed properties as well
+        for dep, value_ in before_deps.items():
+            if dep in self.__eq_operators__:
+                are_equal = self.__eq_operators__[dep]
+            else:
+                are_equal = pick_equality_operator(after_deps[dep])
+            if not are_equal(after_deps[dep], value_):
+                getattr(self.events, dep)(value=after_deps[dep])
 
     # expose the private EmitterGroup publically
     @property
@@ -299,7 +389,7 @@ class EventedModel(BaseModel, metaclass=EventedMetaclass):
         if isinstance(values, self.__class__):
             values = values.dict()
         if not isinstance(values, dict):
-            raise ValueError(
+            raise TypeError(
                 trans._(
                     "Unsupported update from {values}",
                     deferred=True,
@@ -326,17 +416,15 @@ class EventedModel(BaseModel, metaclass=EventedMetaclass):
         like arrays, whose truth value is often ambiguous. ``__eq_operators__``
         is constructed in ``EqualityMetaclass.__new__``
         """
+        if self is other:
+            return True
         if not isinstance(other, EventedModel):
             return self.dict() == other
-
-        for f_name, eq in self.__eq_operators__.items():
-            if f_name not in other.__eq_operators__:
-                return False
-            if (
-                hasattr(self, f_name)
-                and hasattr(other, f_name)
-                and not eq(getattr(self, f_name), getattr(other, f_name))
-            ):
+        if self.__class__ != other.__class__:
+            return False
+        for f_name in self.__fields__:
+            eq = self.__eq_operators__[f_name]
+            if not eq(getattr(self, f_name), getattr(other, f_name)):
                 return False
         return True
 
