@@ -9,8 +9,8 @@ from enum import auto
 from types import TracebackType
 from typing import Callable, List, Optional, Sequence, Tuple, Type, Union
 
-from .events import Event, EventEmitter
-from .misc import StringEnum
+from napari.utils.events import Event, EventEmitter
+from napari.utils.misc import StringEnum
 
 try:
     from napari_error_reporter import capture_exception, install_error_reporter
@@ -108,9 +108,9 @@ class Notification(Event):
         ] = NotificationSeverity.WARNING,
         actions: ActionSequence = (),
         **kwargs,
-    ):
+    ) -> None:
         self.severity = NotificationSeverity(severity)
-        super().__init__(type=str(self.severity).lower(), **kwargs)
+        super().__init__(type_name=str(self.severity).lower(), **kwargs)
         self._message = message
         self.actions = actions
 
@@ -144,14 +144,14 @@ class ErrorNotification(Notification):
 
     exception: BaseException
 
-    def __init__(self, exception: BaseException, *args, **kwargs):
+    def __init__(self, exception: BaseException, *args, **kwargs) -> None:
         msg = getattr(exception, 'message', str(exception))
         actions = getattr(exception, 'actions', ())
         super().__init__(msg, NotificationSeverity.ERROR, actions)
         self.exception = exception
 
     def as_html(self):
-        from ._tracebacks import get_tb_formatter
+        from napari.utils._tracebacks import get_tb_formatter
 
         fmt = get_tb_formatter()
         exc_info = (
@@ -162,7 +162,7 @@ class ErrorNotification(Notification):
         return fmt(exc_info, as_html=True)
 
     def as_text(self):
-        from ._tracebacks import get_tb_formatter
+        from napari.utils._tracebacks import get_tb_formatter
 
         fmt = get_tb_formatter()
         exc_info = (
@@ -173,7 +173,7 @@ class ErrorNotification(Notification):
         return fmt(exc_info, as_html=False, color="NoColor")
 
     def __str__(self):
-        from ._tracebacks import get_tb_formatter
+        from napari.utils._tracebacks import get_tb_formatter
 
         fmt = get_tb_formatter()
         exc_info = (
@@ -193,7 +193,7 @@ class WarningNotification(Notification):
 
     def __init__(
         self, warning: Warning, filename=None, lineno=None, *args, **kwargs
-    ):
+    ) -> None:
         msg = getattr(warning, 'message', str(warning))
         actions = getattr(warning, 'actions', ())
         super().__init__(msg, NotificationSeverity.WARNING, actions)
@@ -308,11 +308,7 @@ class NotificationManager:
         if not self.catch_error:
             sys.__excepthook__(exctype, value, traceback)
             return
-
-        try:
-            self.dispatch(Notification.from_exception(value))
-        except Exception:
-            pass
+        self.dispatch(Notification.from_exception(value))
 
     def receive_warning(
         self,
@@ -368,7 +364,7 @@ def show_console_notification(notification: Notification):
     Show a notification in the console.
     """
     try:
-        from ..settings import get_settings
+        from napari.settings import get_settings
 
         if (
             notification.severity
@@ -401,7 +397,7 @@ def _setup_thread_excepthook():
         def run_with_except_hook(*args2, **kwargs2):
             try:
                 _run(*args2, **kwargs2)
-            except Exception:
+            except Exception:  # noqa BLE001
                 sys.excepthook(*sys.exc_info())
 
         self.run = run_with_except_hook
